/******************************************************************************
 * DISCLAIMER
 * This software is supplied by Renesas Electronics Corporation and is only
 * intended for use with Renesas products. No other uses are authorized. This
 * software is owned by Renesas Electronics Corporation and is protected under
 * all applicable laws, including copyright laws.
 * THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
 * THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
 * LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
 * AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
 * TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
 * ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
 * FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
 * ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
 * BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 * Renesas reserves the right, without notice, to make changes to this software
 * and to discontinue the availability of this software. By using this software,
 * you agree to the additional terms and conditions found by accessing the
 * following link:
 * http://www.renesas.com/disclaimer
 * Copyright (C) 2014 Renesas Electronics Corporation. All rights reserved.
 *****************************************************************************/
/******************************************************************************
 * File Name    : console.h
 * Device(s)    : __#device#__
 * Tool-Chain   : __#tool_chain#__
 * H/W Platform : __#platform#__
 * $Rev         : 0 $
 * $Date        : 0#$
 * Description  : Simple command line console implemenation
 *****************************************************************************/
/******************************************************************************
 * History : DD.MM.YYYY Version  Description
 *         : 05.08.2010 1.00    First Release
 *****************************************************************************/

/******************************************************************************
 WARNING!  IN ACCORDANCE WITH THE USER LICENCE THIS CODE MUST NOT BE CONVEYED
 OR REDISTRIBUTED IN COMBINATION WITH ANY SOFTWARE LICENSED UNDER TERMS THE
 SAME AS OR SIMILAR TO THE GNU GENERAL PUBLIC LICENCE
 ******************************************************************************/

#ifndef CONSOLE_H_
#define CONSOLE_H_

/******************************************************************************
 Includes   <System Includes> , "Project Includes"
 *****************************************************************************/

#include "r_typedefs.h"

/******************************************************************************
 Macro definitions
 *****************************************************************************/

/* Define default settings */
#ifndef CMD_READER_LINE_SIZE
#define CMD_READER_LINE_SIZE        1040
#endif

#ifndef CMD_MAX_PATH
#define CMD_MAX_PATH                2600
#endif

#ifndef CMD_MAX_ARG_LENGTH
#define CMD_MAX_ARG_LENGTH          128
#endif

#ifndef CMD_MAX_ARG
#define CMD_MAX_ARG                 8
#endif

#define CMD_ESCAPE_CHARACTER        0x1B

#define _SERIAL_

/* Function macro to remove the unused variable information in command
 processor functions */
#define AVOID_UNUSED_WARNING        (void)iArgCount; (void)ppszArgument; (void)pCom;

/******************************************************************************
 Typedef definitions
 *****************************************************************************/

typedef struct _CMDTAB CMDTAB, *PCMDTAB;
typedef const struct _CMDTAB *CPCMDTAB;

/* Enumerate the escape sequences states */
typedef enum
{
    ESC_NO_ESCAPE = 0, ESC_ESCAPE_SEQUENCE
} CMDESC;

/* Structure of Variables used by command reader
 These are the values that change as the command is read in */
typedef struct _CONVA
{
    /* Buffer for bytes as parsed */
    char     pszBuffer[CMD_READER_LINE_SIZE];
    char     pszCommand[CMD_READER_LINE_SIZE];

    /* Buffer for arguments */
    char     pszArguments[(CMD_MAX_ARG * CMD_MAX_ARG_LENGTH)];

    /* The escape sequence state */
    CMDESC   escapeSequence;

    /* Index into command line buffer */
    uint32_t uiBufIdx;

    /* The destination data byte */
    int      iData;

    /* The number of characters read */
    size_t   stReadCount;
} CONVA;

/* Structure of the Command Set passed to the parser and the commands */
struct _CONVAR
{
    /* Variables used by command reader */
    CONVA    va;

    /* Pointer to the required prompt */
    char     *pszPrompt;

    /* The stream for input */
    FILE     *pIn;

    /* The stream for output */
    FILE     *pOut;

    /* A flag to make echo characters secret */
    _Bool    bfSecrecy;

    /* Pointer to the user's function set */
    CPCMDTAB *ppFn;
    int      iNumTables;

    /* The working drive letter */
    char     chWorkingDrive;

    /* The temporary directory paths */
    char     pszTempFile1[CMD_MAX_PATH];
    char     pszTempFile2[CMD_MAX_PATH];

    /* The working directory */
    char     pszWorkingDir[CMD_MAX_PATH];

    /* The prompt containing the working directory */
    char     pszWorkingDirPrompt[CMD_MAX_PATH];

    /* The default prompt */
    char     *pszDefaultPrompt;
};

typedef struct _CONVAR COMSET, *PCOMSET, * const CPCOMSET;

/* Define the return codes */
typedef enum _CMDERR
{
    CMD_OK = 0,
    CMD_NO_PROMPT,
    CMD_LINE_TOO_LONG,
    CMD_ESCAPE_SEQUENCE,
    CMD_UNKNOWN,
    CMD_USER_EXIT,
    CMD_LOG_OUT,
    CMD_ERROR_IN_IO,
    CMD_USER_CODE_BASE
} CMDERR, *PCMDERR;

/******************************************************************************
 * Function Name: (CMDFUNC)
 * Description  : Function prototype to handle commands passed to the parser
 * Arguments    : IN  iArgCount - The number of arcument strings in the array
 * Arguments    : IN  ppszArgument - Pointer to the argument list
 * Arguments    : IN  pCom - Pointer to the command object
 * Return Value : Return value: 0 for success otherwise error code
 *****************************************************************************/
typedef int (* const CMDFUNC) (int iArgCount, char **ppszArgument, PCOMSET pCom);

/* Structure of a set of commands used by the command line parser
 Each command set requires one of these. */
typedef struct _CMDFNASS
{
    char    *pszCommand; /* The command letters */
    CMDFUNC pFunction; /* Pointer to the handling function */
    char    *pszCommandDescription; /* A pointer to the command description */
} CMDFNASS, *PCMDFNASS, * const CPCMDFNASS;

struct _CMDTAB
{
    PCMDFNASS pCommandList;
    uint32_t  dwNumberOfCommands; /* The number of commands in the table */
};

/* Define the escape key handler function type */
typedef CMDESC (* const PESCFN) (PCOMSET);

/******************************************************************************
 Variable External definitions and Function External definitions
 *****************************************************************************/

/******************************************************************************
 Exported global functions (to be accessed by other files)
 *****************************************************************************/

/******************************************************************************
 Function Prototypes
 *****************************************************************************/

#ifdef __cplusplus
extern "C"
{
#endif

/******************************************************************************
 * Function Name: Console
 * Description  : Function to implement a simple command console using the ANSI C
 run time library IO functions
 * Arguments    : IN  pCom - Pointer to the command object data
 * Arguments    : IN  ppComFunctions - Pointer to a table of command tables
 * Arguments    : IN  iNumTables - The number of command tables
 * Arguments    : IN  pIn - Pointer to the file stream for input
 * Arguments    : IN  pOut - Pointer to the file stream for output
 * Arguments    : IN  pszPrompt - Pointer to the prompt string
 * Return Value : Return value:  0 for success otherwise error code
 *****************************************************************************/

extern CMDERR Console (PCOMSET pCom, CPCMDTAB *ppComFunctions, int iNumTables, FILE *pIn, FILE *pOut, char *pszPrompt);

/******************************************************************************
 * Function Name: conGetLastCommandLine
 * Description  : Function to bring up the last command line for editing
 * Arguments    : IN  pCom - pointer to the command object
 * Return Value : Return value:  0 for success otherwise error code
 *****************************************************************************/

extern void conGetLastCommandLine (PCOMSET pCom);

/******************************************************************************
 * Function Name: conDoLastCommandLine
 * Description  : Function to get do the last command again
 * Arguments    : IN  pCom - pointer to command table
 Return value:  0 for success otherwise error code
 *****************************************************************************/

extern CMDERR conDoLastCommandLine (PCOMSET pCom);

/******************************************************************************
 * Function Name: conProcessChar
 * Description  : Function to process a character
 * Arguments    : IN  pCom - pointer to the command object
 * Arguments    : IN  chChar - The character to process
 * Arguments    : IN  pbfCommand - Pointer to a flag that is set when a
 command is received
 * Return Value : Return value:  0 for success otherwise error code
 *****************************************************************************/

extern CMDERR conProcessChar (PCOMSET pCom, char chChar, _Bool *pbfCommand);

/******************************************************************************
 Function Name: read_input_num
 Description:
 Arguments:     IN  iArgCount The number of arguments in the argument list
 IN  ppszArgument - The argument list
 IN  pCom - Pointer to the command object
 Return value:  CMD_OK for success
 ******************************************************************************/
extern void read_input_num(PCOMSET pCom, uint32_t *out);

/******************************************************************************
 Function Name: read_input_hex
 Description:
 Arguments:     IN  pCom - Pointer to the command object
 Return value:  CMD_OK for success
 ******************************************************************************/
extern void read_input_hex(PCOMSET pCom, uint32_t *out);

/******************************************************************************
 Function Name: read_input_char
 Description:
 Arguments:     IN  iArgCount The number of arguments in the argument list
 IN  ppszArgument - The argument list
 IN  pCom - Pointer to the command object
 Return value:  CMD_OK for success
 ******************************************************************************/
extern void read_input_char(PCOMSET pCom, uint32_t limit, uint8_t *out);

#ifdef __cplusplus
}
#endif

#endif /* CONSOLE_H_ */

/******************************************************************************
 End  Of File
 *****************************************************************************/
