/******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only 
* intended for use with Renesas products. No other uses are authorized. This 
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS 
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE 
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the 
* following link:
* http://www.renesas.com/disclaimer
*******************************************************************************
* Copyright (C) 2012 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************
* File Name    : console.c
* Version      : 1.01
* Device(s)    : Renesas
* Tool-Chain   : N/A
* OS           : N/A
* H/W Platform : Renesas
* Description  : Simple command line console implemenation
*******************************************************************************
* History      : DD.MM.YYYY Version Description
*              : 04.02.2010 1.00    First Release
*              : 10.06.2010 1.01    Updated type definitions
******************************************************************************/

/******************************************************************************
  WARNING!  IN ACCORDANCE WITH THE USER LICENCE THIS CODE MUST NOT BE CONVEYED
  OR REDISTRIBUTED IN COMBINATION WITH ANY SOFTWARE LICENSED UNDER TERMS THE
  SAME AS OR SIMILAR TO THE GNU GENERAL PUBLIC LICENCE
******************************************************************************/

/******************************************************************************
Includes   <System Includes> , "Project Includes"
******************************************************************************/

#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include "Console.h"
#include "Control.h"
#include "wildCompare.h"

/******************************************************************************
Typedef definitions
******************************************************************************/

/******************************************************************************
Macro definitions
******************************************************************************/

/******************************************************************************
Imported global variables and functions (from other files)
******************************************************************************/

/******************************************************************************
Exported global variables and functions (to be accessed by other files)
******************************************************************************/

/******************************************************************************
Private global variables and functions
******************************************************************************/

static int conPrintPrompt(PCOMSET pCom);
static CMDERR conParseCommand(PCOMSET pCom);

/******************************************************************************
Public Functions
******************************************************************************/

/******************************************************************************
Function Name: Console
Description:   Function to implement a simple command console using the ANSI C
               run time library IO functions
Arguments:     IN  pCom - Pointer to the command object data
               IN  pCmdFunctions - Pointer to a table of command tables
               IN  pIn - Pointer to the file stream for input
               IN  pOut - Pointer to the file stream for output
               IN  pszPrompt - Pointer to the prompt string
Return value:  0 for success otherwise error code
******************************************************************************/
CMDERR Console(PCOMSET  pCom,
               CPCMDTAB *ppComFunctions,
               int      iNumTables,
               FILE     *pIn,
               FILE     *pOut,
               char     *pszPrompt)
{
    /* Initialise our variables */
    memset(pCom, 0, sizeof(COMSET));
    pCom->pIn = pIn;
    pCom->pOut = pOut;
    pCom->ppFn = ppComFunctions;
    pCom->iNumTables = iNumTables;
    pCom->pszPrompt = pszPrompt;
    pCom->pszDefaultPrompt = pszPrompt;
    pCom->chWorkingDrive = -1;
    /* Initialise the last command with the help string */
    strcpy(pCom->va.pszCommand, "?");
    /* Print a prompt */
    conPrintPrompt(pCom);
    fflush(pOut);
    /* Disable buffering for single character input */
    setvbuf(pIn, NULL, _IONBF, 0);
    /* Until there is an error */
    while (true)
    {
        _Bool   bfCommand = false;
        CMDERR  errorCode = (CMDERR)0;
        int     iData;
        /* Read a character from the input stream */
        iData = fgetc(pIn);
        if (iData == EOF)
        {
            clearerr(pIn);
            return CMD_ERROR_IN_IO;
        }
        pCom->va.iData = iData;
        /* Bump the read count */
        pCom->va.stReadCount++;
        /* Process the character */
        fflush(pOut);
        errorCode = conProcessChar(pCom, (char)pCom->va.iData, &bfCommand);
        /* If an error occurs then return it */
        if (errorCode > CMD_UNKNOWN)
        {
            return errorCode;
        }
        /* If a command has been recieved then print the prompt if required */
        if ((bfCommand)
        &&  (errorCode != CMD_NO_PROMPT))
        {
            if (conPrintPrompt(pCom) < 0)
            {
                return CMD_ERROR_IN_IO;
            }
        }
        fflush(pOut);
    }
    return CMD_OK;
}
/******************************************************************************
End of function Console
******************************************************************************/

/******************************************************************************
Function Name: conGetLastCommandLine
Description:   Function to bring up the last command line for editing
Arguments:     IN  pCom - pointer to the command object
Return value:  0 for success otherwise error code
******************************************************************************/
void conGetLastCommandLine(PCOMSET pCom)
{
    strcpy(pCom->va.pszBuffer, pCom->va.pszCommand);
    fprintf(pCom->pOut, "%s", pCom->va.pszBuffer);
    pCom->va.uiBufIdx = (uint32_t)strlen(pCom->va.pszCommand);
}
/******************************************************************************
End of function conGetLastCommandLine
******************************************************************************/

/******************************************************************************
* Function Name: conDoLastCommandLine
* Description  : Function to get do the last command again
* Arguments    : IN  pCom - pointer to command table
* Return Value : 0 for success otherwise error code
******************************************************************************/
CMDERR conDoLastCommandLine(PCOMSET pCom)
{
    conGetLastCommandLine(pCom);
    fprintf(pCom->pOut, "\r\n");
    return conParseCommand(pCom);
}
/******************************************************************************
End of function conDoLastCommandLine
******************************************************************************/

/******************************************************************************
Function Name: conProcessChar
Description:   Function to process a character
Arguments:     IN  pCom - pointer to the command object
               IN  chChar - The character to process
               IN  pbfCommand - Pointer to a flag that is set when a
                                command is received
Return value:  0 for success otherwise error code
******************************************************************************/
CMDERR conProcessChar(PCOMSET pCom, char chChar, _Bool *pbfCommand)
{
    *pbfCommand = false;
    switch (chChar)
    {
        /* Ignore escape sequence start */
        case CMD_ESCAPE_CHARACTER:
                                    /* suppress echo */
        pCom->va.escapeSequence = ESC_ESCAPE_SEQUENCE;
        pCom->va.uiBufIdx = (uint32_t)0;
        break;
        /* White out on back space */
        case '\b':
        /* PuTTY sends non-ascii back space */
        case  0x7F:                 
        if (pCom->va.uiBufIdx != 0)
        {
#ifdef _SERIAL_
            fprintf(pCom->pOut, "\b \b");
#endif
            pCom->va.uiBufIdx--;
        }
        break;
#ifdef _SERIAL_
        /* New line - ignore */
        case '\n':
        /* Ignore */
        break;
#endif
        /* End of escape sequence */
        case ';':
        if (pCom->va.escapeSequence == ESC_NO_ESCAPE)
        {
            goto BUFFER;
        }
        /* Fall Through */
        case '~':
        if (pCom->va.escapeSequence == ESC_NO_ESCAPE)
        {
            goto BUFFER;
        }
        /* Fall Through */
        case 0:
        if (pCom->va.escapeSequence == ESC_NO_ESCAPE)
        {
            break;
        }
        /* Fall Through */
        /* return - do function */
#ifdef _SERIAL_
        case '\r':
#else
        case '\n':
#endif
        if ((pCom->va.escapeSequence == ESC_NO_ESCAPE)
        &&  (pCom->va.pszBuffer[0] != '@'))
        {
#ifdef _SERIAL_
            fprintf(pCom->pOut, "\r\n");
#endif
        }
        /* Terminate the string */
        pCom->va.pszBuffer[pCom->va.uiBufIdx] = (uint8_t)'\0';
        *pbfCommand = true;
        /* Pars the command line */
        return conParseCommand(pCom);
        /* All other characters */
        BUFFER:
        default:
        if (pCom->va.uiBufIdx < (CMD_READER_LINE_SIZE - 1))
        {
            /* buffer */
            pCom->va.pszBuffer[pCom->va.uiBufIdx++] = chChar;
            /* Two character escape sequence termination tests - F Keys */
            if ((pCom->va.escapeSequence == ESC_ESCAPE_SEQUENCE)
            &&  (pCom->va.uiBufIdx == 2)
            &&  (*pCom->va.pszBuffer == 'O'))
            {
                pCom->va.pszBuffer[pCom->va.uiBufIdx] = (uint8_t)'\0';
                *pbfCommand = true;
                /* Pars the command line */
                return conParseCommand(pCom);
            }
            /* Arrow keys */
            else if ((pCom->va.escapeSequence == ESC_ESCAPE_SEQUENCE)
                 &&  (pCom->va.uiBufIdx == 2)
                 &&  (*pCom->va.pszBuffer == '[')
                 &&  (chChar != '1'))
            {
                pCom->va.pszBuffer[pCom->va.uiBufIdx] = (uint8_t)'\0';
                *pbfCommand = true;
                /* Pars the command line */
                return conParseCommand(pCom);
            }
        }
        else
        {
            return CMD_LINE_TOO_LONG;
        }
#ifdef _SERIAL_
        /* echo to console */
        if ((pCom->va.escapeSequence == ESC_NO_ESCAPE)
        &&  (pCom->va.pszBuffer[0] != '@'))
        {
            if (pCom->bfSecrecy)
            {
                fputc((int)'*', pCom->pOut);
            }
            else
            {
                fputc((int)chChar, pCom->pOut);
            }
        }
#endif
        break;
   }

   return CMD_OK;
}
/******************************************************************************
End of function conProcessChar
******************************************************************************/

/******************************************************************************
* Function Name: conPrintPrompt
* Description  : Function to print the command prompt
* Arguments    : IN  pCom - pointer to the command object
* Return Value : 0 for success otherwise error code
******************************************************************************/
static int conPrintPrompt(PCOMSET pCom)
{
    int iResult = 0;
    /* Don't print a NULL prompt */
    if (pCom->pszPrompt)
    {
        iResult = fprintf(pCom->pOut, "\r\n%s ", pCom->pszPrompt);
    }
    return iResult;
}
/******************************************************************************
End of function conPrintPrompt
******************************************************************************/

/******************************************************************************
* Function Name: conSplitLine
* Description  : Function to split the command line in to the argument array
* Arguments    : IN  pszLine - Pointer to the command line
*                IN  pszArguments - Pointer to the arguments
* Return Value : The number of arguments split
******************************************************************************/
static int conSplitLine(char * pszLine, char **pszArguments)
{
    int      iArgCount;
    int      iCount;
    char * pszArgument;
    for (iArgCount = 0; iArgCount < CMD_MAX_ARG;)
    {
        pszArgument = pszArguments[iArgCount];
        /* Look for the first white space */
        while ((*pszLine)
        &&     (isspace(*pszLine)))
        {
            pszLine++;
        }
        /* Initialise the argument length counter */
        iCount = 0;
        /* Allow inverted commas to specify file names with spaces in */
        if (*pszLine == '\"')
        {
           pszLine++;
           /* Copy the string until the next inverted comma or end of line */
            while ((*pszLine)
            &&     (*pszLine != '\"'))
            {
                iCount++;
                if (iCount >= CMD_MAX_ARG_LENGTH)
                {
                    *pszArgument = '\0';
                    break;
                }
                *pszArgument++ = *pszLine++;
            }
            /* Skip the " character */
            pszLine++;
        }
        /* Copy and argument until the end of the line or the next space */
        while ((*pszLine)
        &&     (!isspace(*pszLine)))
        {
            iCount++;
            if (iCount >= CMD_MAX_ARG_LENGTH)
            {
                *pszArgument = '\0';
                break;
            }
            *pszArgument++ = *pszLine++;
        }
        /* Check for the end of the command line */
        if (!*pszLine)
        {
            break;
        }
        else
        {
            /* Increment the argument count */
            iArgCount++;
        }
    }
    iArgCount++;
    if (iArgCount > CMD_MAX_ARG)
    {
        iArgCount = CMD_MAX_ARG;
    }
    return iArgCount;
}
/******************************************************************************
End of function  conSplitLine
******************************************************************************/

/******************************************************************************
* Function Name: conInitArgList
* Description  : Function to initilaise the argument list
* Arguments    : IN  pCom - pointer to the command object
* Return Value : none
******************************************************************************/
static void conInitArgList(PCOMSET  pCom)
{
    int      iCount = CMD_MAX_ARG;
    char **pszArguments = (char **)pCom->va.pszArguments;
    char  *pszArgument = ((char *)pCom->va.pszArguments) +
                            (CMD_MAX_ARG * sizeof(char *));
    /* Intialise all the data */
    memset(pCom->va.pszArguments, 0, (CMD_MAX_ARG * CMD_MAX_ARG_LENGTH));
    /* Set the pointers to the strings */
    while (iCount--)
    {
        *pszArguments++ = pszArgument;
        pszArgument += CMD_MAX_ARG_LENGTH;
    }
}
/******************************************************************************
End of function  conInitArgList
******************************************************************************/

/******************************************************************************
* Function Name: conExecute
* Description  : Function to execute the command function
* Arguments    : IN  pCom - pointer to the command object
*                IN  pCmdList - pointer to a list of commands to match against
*                OUT pbfValidCommand - pointer to a flag set true when command
*                                      matched
* Return Value : 0 for success otherwise error code
******************************************************************************/
static CMDERR conExecute(PCOMSET  pCom,
                         CPCMDTAB pCmdList,
                         _Bool     *pbfValidCommand)
{
    /* Check for a valid command list pointer */
    if (pCmdList != NULL)
    {
        uint32_t uiCmdLen, uiLength;
        int      iFuncIndex, iArgCount;
        /* Initialise the argument list */
        conInitArgList(pCom);
        /* Split the command line */
        iArgCount = conSplitLine(pCom->va.pszBuffer,
                                 (char**)pCom->va.pszArguments);
        /* Get the length of the command */
        uiCmdLen = (uint32_t)strlen((*(char **)pCom->va.pszArguments));
        /* check against list of command tokens */
        for (iFuncIndex = 0; iFuncIndex < (int)pCmdList->dwNumberOfCommands;
             iFuncIndex++)
        {
            /* Get pointer to the associated command */
            char * pszString = pCmdList->pCommandList[iFuncIndex].pszCommand;
            /* Calculate command length */
            uiLength = (uint32_t)strlen(pszString);
            /* Check the length of the command */
            if (uiCmdLen == uiLength)
            {
                /* Command line has been matched */
                if (wildCompare(pszString, (*(char **)pCom->va.pszArguments)))
                {
                    /* Show that we are processing a valid command */
                    *pbfValidCommand = true;
                    /* Execute the command */
                    return (CMDERR)pCmdList->pCommandList[iFuncIndex].
                                   pFunction(iArgCount,
                                             (char **)pCom->va.pszArguments,
                                             pCom);
                }
            }
        }
    }
    /* No command matched */
    *pbfValidCommand = false;
    return CMD_OK;
}
/******************************************************************************
End of function conExecute
******************************************************************************/

/******************************************************************************
* Function Name: conParseCommand
* Description  : Function to pars the command and call the handling function
* Arguments    : IN  pCom - pointer to command table
* Return Value : 0 for success otherwise error code
******************************************************************************/
static CMDERR conParseCommand(PCOMSET pCom)
{
    uint32_t uiEatSpaceCount = 0U;
    uint32_t uiLength;
    _Bool    bfValidCommand = false;
    int      iCount = 0;
    CMDERR  errorCode = CMD_OK;
    /* Eat any white space before the command */
    while ((pCom->va.pszBuffer[uiEatSpaceCount] == ' ')
    || (pCom->va.pszBuffer[uiEatSpaceCount] == '\t')
    || (pCom->va.pszBuffer[uiEatSpaceCount] == '@'))
    {
        uiEatSpaceCount++;
    }
    uiLength = pCom->va.uiBufIdx - uiEatSpaceCount;
    pCom->va.uiBufIdx = (uint32_t)0;
    /* Test for valid command or just CR */
    if (uiLength >= 1)
    {
        /* If it is not an escape sequence */
        if (pCom->va.escapeSequence == ESC_NO_ESCAPE)
        {
            /* Copy to last command buffer */
            strcpy(pCom->va.pszCommand, pCom->va.pszBuffer);
        }
        else
        {
            /* Cancel escape sequence */
            pCom->va.escapeSequence = ESC_NO_ESCAPE;
        }
        /* Execute the command function */
        while (iCount < pCom->iNumTables)
        {
            errorCode = conExecute(pCom, pCom->ppFn[iCount], &bfValidCommand);
            if (bfValidCommand == false)
            {
                iCount++;
            }
            else
            {
                break;
            }
        }
        /* Test to see if command is valid */
        if (bfValidCommand == false)
        {
            fprintf(pCom->pOut, "\r\n\"%s\" Unknown command\r\n", pCom->va.pszBuffer);
        }
    }

    return errorCode;
}
/******************************************************************************
End of function conParseCommand
******************************************************************************/


/******************************************************************************
 Function Name: read_input_num
 Description:
 Arguments:     IN  iArgCount The number of arguments in the argument list
 IN  ppszArgument - The argument list
 IN  pCom - Pointer to the command object
 Return value:  CMD_OK for success
 ******************************************************************************/
void read_input_num(PCOMSET pCom, uint32_t *out)
{
    uint8_t cdata;
    uint8_t limit = 10;

    cdata = fgetc(pCom->pIn);
    fputc(cdata,pCom->pOut);
    fflush(pCom->pOut);

    (*out) = 0;

    while((limit--) && ((cdata >= '0') && (cdata <= '9')))
    {
        (*out) *= 10;
        (*out) += (cdata -'0');
        cdata = fgetc(pCom->pIn);
        fputc(cdata,pCom->pOut);
        fflush(pCom->pOut);
    }

    fprintf(pCom->pOut,"\r\n");
}
/******************************************************************************
* End of function read_input_num
******************************************************************************/

/******************************************************************************
 Function Name: read_input_hex
 Description:
 Arguments:     IN  pCom - Pointer to the command object
 Return value:  CMD_OK for success
 ******************************************************************************/
void read_input_hex(PCOMSET pCom, uint32_t *out)
{
    uint8_t cdata;
    uint8_t val;
    uint8_t limit = 9;

    cdata = fgetc(pCom->pIn);
    fputc(cdata,pCom->pOut);
    fflush(pCom->pOut);

    (*out) = 0;

    while((limit--) )
    {
        /* decode input */
        if((cdata >= '0') && (cdata <= '9'))
        {
            val = (cdata -'0');
        }
        else if((cdata >= 'a') && (cdata <= 'f'))
        {
            val = (10 + (cdata -'a'));
        }
        else if((cdata >= 'A') && (cdata <= 'F'))
        {
            val = (10 +(cdata -'A'));
        }
        else
        {
            limit = 0;
            val   = 0;
        }

        if(limit > 0)
        {
            (*out) *= 16;
            (*out) += val;
            cdata = fgetc(pCom->pIn);
        }
        fputc(cdata,pCom->pOut);
        fflush(pCom->pOut);
    }

    fprintf(pCom->pOut,"\r\n");
}
/******************************************************************************
* End of function read_input_hex
******************************************************************************/

/******************************************************************************
 Function Name: read_input_char
 Description:
 Arguments:     IN  iArgCount The number of arguments in the argument list
 IN  ppszArgument - The argument list
 IN  pCom - Pointer to the command object
 Return value:  CMD_OK for success
 ******************************************************************************/
void read_input_char(PCOMSET pCom, uint32_t limit, uint8_t *out)
{
    uint8_t cdata;

    cdata = fgetc(pCom->pIn);
    fputc(cdata,pCom->pOut);
    fflush(pCom->pOut);

    while((limit--) && ('\r' != cdata))
    {
        (*out++) = cdata;
        cdata = fgetc(pCom->pIn);
        fputc(cdata,pCom->pOut);
        fflush(pCom->pOut);
    }
    (*out) = '\0';

    fprintf(pCom->pOut,"\r\n");
}
/******************************************************************************
* End of function read_input_char
******************************************************************************/

/******************************************************************************
End  Of File
******************************************************************************/
