/******************************************************************************
 * DISCLAIMER
 * This software is supplied by Renesas Electronics Corporation and is only
 * intended for use with Renesas products. No other uses are authorized. This
 * software is owned by Renesas Electronics Corporation and is protected under
 * all applicable laws, including copyright laws.
 * THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
 * THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
 * LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
 * AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
 * TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
 * ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
 * FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
 * ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
 * BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 * Renesas reserves the right, without notice, to make changes to this software
 * and to discontinue the availability of this software. By using this software,
 * you agree to the additional terms and conditions found by accessing the
 * following link:
 * http://www.renesas.com/disclaimer
 * Copyright (C) 2014 Renesas Electronics Corporation. All rights reserved.
 *****************************************************************************/
/******************************************************************************
 * File Name    : rzDmaTest.c
 * Device(s)    : __#device#__
 * Tool-Chain   : __#tool_chain#__
 * OS           : __#OS_Name#__
 * H/W Platform : __#platform#__
 * $Rev         : 0 $
 * $Date        : 0#$
 * Version      : 1.0 <- Optional as long as history is shown below
 * Description  : This module solves all the world's problems
 *****************************************************************************/
/******************************************************************************
 * History     : DD.MM.YYYY Version Description
 *             : __#dd#__.__#mm#__.__#yyyy#__ __#ver_maj#__.__#ver_min#__ First Release
 *****************************************************************************/

/******************************************************************************
 WARNING!  IN ACCORDANCE WITH THE USER LICENCE THIS CODE MUST NOT BE CONVEYED
 OR REDISTRIBUTED IN COMBINATION WITH ANY SOFTWARE LICENSED UNDER TERMS THE
 SAME AS OR SIMILAR TO THE GNU GENERAL PUBLIC LICENCE
 ******************************************************************************/

/******************************************************************************
 Includes   <System Includes> , "Project Includes"
 *****************************************************************************/

#include <string.h>
#include "rzDmaTest.h"
#include "hwArmCache.h"
#include "iodefine.h"
#include "rza_io_regrw.h"
#include "trace.h"
#include "devdrv_intc.h"
#include "r_event.h"
/******************************************************************************
 Macro definitions
 *****************************************************************************/

/******************************************************************************
 Typedef definitions
 *****************************************************************************/

/******************************************************************************
 Exported global variables (to be accessed by other files)
 *****************************************************************************/

/******************************************************************************
 Private global variables and functions
 *****************************************************************************/

static bool_t rzDmaTransfer (void *pvDest, void *pvSrc, size_t stLength);
static void rzFillTestData (uint32_t *pulFileData, size_t stLength);
static bool_t rzCheckTestData (uint32_t *pulTestData, size_t stLength);

static PEVENT gpevTransferComplete = NULL;
static uint32_t gulDataStartValue = 0UL;

/******************************************************************************
 Imported global variables and functions (from other files)
 *****************************************************************************/

/**********************************************************************************
 * Function Name: rzDmaTest
 * Description  : Function to test a DMA memory to memory transfer. It fills the source
 memory with known data, performs a DMA transfer and then compares the
 source and destination memory
 * Arguments    : OUT pvDest - Pointer to the destination memory
 * Arguments    : IN  pvSrc - Pointer to the source memory
 * Arguments    : IN  stLength - The length of memory to transfer
 * Arguments    : IN  bfMemcpy - true to use memcpy C function
.* Return value:  The status of the test
 **********************************************************************************/
RZDMAE rzDmaTest (void *pvDest, void *pvSrc, size_t stLength, bool_t bfMemcpy)
{
    bool_t bfClearResult, bfInvalidateResult;
    void *pvDestCheck = pvDest;
    size_t stLengthCheck = stLength;

    /* The software stacks guarantee that any data to be transferred by the DMA
     will be aligned to four byte boundaries and length */
    if ((size_t) pvDest & (sizeof(uint32_t) - 1))
    {
        return RZ_DMA_DEST_MEMORY_ADDRESS_ERROR;
    }

    if ((size_t) pvSrc & (sizeof(uint32_t) - 1))
    {
        return RZ_DMA_SRC_MEMORY_ADDRESS_ERROR;
    }
    if (stLength % sizeof(uint32_t))
    {
        return RZ_DMA_MEMORY_LENGTH_ERROR;
    }

    /* 1. Fill the source memory with known data */
    rzFillTestData((uint32_t*) pvSrc, stLength / sizeof(uint32_t));

    /* 2. Transfer the memory with the DMA */
    do
    {
        void *pvPhysicalSrc, *pvPhysicalDest;
        size_t stSrcLength, stDestLength;

        /* Make sure that the source memory is written back to physical memory
         and get the address of the physical memory and segment length */
        bfClearResult = cacheClear(&pvPhysicalSrc, &stSrcLength, pvSrc, stLength);

        /* Make sure that the destination memory is not in cache memory */
        bfInvalidateResult = cacheInvalidate(&pvPhysicalDest, &stDestLength, pvDest, stLength);

        /* If the page boundaries are the same then this can be done with one DMA
         transfer */
        if (stDestLength == stSrcLength)
        {
            if (bfMemcpy)
            {
                printf("rzDmaTest: memcpy 0x%p -> 0x%p length %lu\r\n", pvSrc, pvDest, stDestLength);
                memcpy(pvDest, pvSrc, stDestLength);
            }
            else
            {
                printf("rzDmaTest: DMA 0x%p -> 0x%p length %lu\r\n", pvPhysicalSrc, pvPhysicalDest, stDestLength);
                rzDmaTransfer(pvPhysicalDest, pvPhysicalSrc, stDestLength);
            }

            /* Add on the transfer length to the source and destination pointers */
            pvDest = (void*) ((uint8_t*) pvDest + stDestLength);
            pvSrc = (void*) ((uint8_t*) pvSrc + stDestLength);

            /* Subtract the transfer length from the length */
            stLength -= stDestLength;
        }
        else
        {
            /* Calculate the shortest length */
            size_t stShortestLength = (stDestLength < stSrcLength) ? stDestLength : stSrcLength;

            /* Transfer only the shortest length */
            if (bfMemcpy)
            {
                printf("rzDmaTest: memcpy 0x%p -> 0x%p length %lu\r\n", pvSrc, pvDest, stShortestLength);
                memcpy(pvDest, pvSrc, stShortestLength);
            }
            else
            {
                printf("rzDmaTest: DMA 0x%p -> 0x%p length %lu\r\n", pvPhysicalSrc, pvPhysicalDest, stShortestLength);
                rzDmaTransfer(pvPhysicalDest, pvPhysicalSrc, stShortestLength);
            }

            /* Add on the length to the source and destination pointers */
            pvDest = (void*) ((uint8_t*) pvDest + stShortestLength);
            pvSrc = (void*) ((uint8_t*) pvSrc + stShortestLength);

            /* Subtract the shortest length from the length */
            stLength -= stShortestLength;
        }
    } while (stLength);

    /* 3. Check that the destination memory has the correct contents */
    if (rzCheckTestData(pvDestCheck, stLengthCheck / sizeof(uint32_t)))
    {
        /* Change the start value for the next test */
        gulDataStartValue++;
        return RZ_DMA_OK;
    }
    else
    {
        /* Change the start value for the next test */
        gulDataStartValue++;
        return RZ_DMA_MEMORY_COMPARE_ERROR;
    }
}
/**********************************************************************************
 End of function  rzDmaTest
 ***********************************************************************************/

/***********************************************************************************
 Private Functions
 ***********************************************************************************/

/******************************************************************************
 * Function Name: rzTransferComplete
 * Description  : Function to fill the file data buffer with known data
 * Arguments    : IN  uiParameter - Generic
 * Return Value : none
 ******************************************************************************/
static void rzTransferComplete (uint32_t uiParameter)
{
    eventSet(gpevTransferComplete);
}
/******************************************************************************
 End of function  rzTransferComplete
 ******************************************************************************/

/**********************************************************************************
 * Function Name: rzDmaTransfer
 * Description  : Function to perform a DMA transfer
 * Arguments    : OUT pvDest - Pointer to the desitnation memory
 * Arguments    : IN  pvSrc - Pointer to the source memory
 * Arguments    : IN  stLength - The length of memory to transfer
 * Return Value :  true for success
 **********************************************************************************/
static bool_t rzDmaTransfer (void *pvDest, void *pvSrc, size_t stLength)
{
    bool_t bfResult = false;
    if (0 == eventCreate(&gpevTransferComplete, 1))
    {
        /* Set Source Start Address */
        DMAC.N0SA_3 = (uint32_t) pvSrc;

        /* Set Destination Start Address */
        DMAC.N0DA_3 = (uint32_t) pvDest;

        /* Set Transfer Size */
        DMAC.N0TB_3 = stLength;
#if 0
        /* Set Count direction of the source address Auto increment */
        DMAC3.CHCFG.BIT.DAD = 0;

        /* Set Count direction of the destination address Auto increment */
        DMAC3.CHCFG.BIT.SAD = 0;

        /* Set Source Transfer Size 128 bits */
        DMAC3.CHCFG.BIT.DDS = 7;

        /* Set Destination Transfer Size 128 bits */
        DMAC3.CHCFG.BIT.SDS = 7;

        /* Set Channel Status Register DMA Mode to 0 (Register Mode) */
        DMAC3.CHCFG.BIT.DMS = 0;

        /* Set Channel Status Register Set to 0 (Select Executes the Next0 Register Set) */
        DMAC3.CHCFG.BIT.RSEL = 0;

        /* Set Channel Status Register Sweep Buffer Enable to 0 (Stops the DMA transfer without sweeping the buffer) */
        DMAC3.CHCFG.BIT.SBE = 0;

        /* Set Channel Status Register DMA Interrupt Mask to 1 to mask the interrupt */
        DMAC3.CHCFG.BIT.DEM = 0;

        /* Set Enabled Register Set Enable to 0 (Does not continue DMA transfers) */
        DMAC3.CHCFG.BIT.REN = 0;

        /* Set Register Select Switch to 0 (Does not invert RSEL automatically after a DMA transaction) */
        DMAC3.CHCFG.BIT.RSW = 0;

        /* Set block transfer mode */
        DMAC3.CHCFG.BIT.TM = 0;

        /* AM  : Set ACK Mode DMAACK not to be output Required for STG - see DMAC3.CHCTRL.BIT.STG = 1 below*/
        DMAC3.CHCFG.BIT.AM = 4;

        /* SEL : Set SEL setting */
        DMAC3.CHCFG.BIT.SEL = 3;

        /* LVL, HIEN, LOWN, REQD : Initial Value */
        DMAC3.CHCFG.BIT.LVL = 0;
        DMAC3.CHCFG.BIT.HIEN = 0;
        DMAC3.CHCFG.BIT.LOEN = 0;
        DMAC3.CHCFG.BIT.REQD = 0;
        Trace("DMAC3.CHCFG.LONG = 0x%.8lX\r\n", DMAC3.CHCFG.LONG);
#else
        DMAC.CHCFG_3 = 0x00477403;
#endif
        /* MID, RID : Initial Value */
        DMAC.DMARS1 = 0;

        /* ? */
        DMAC.CHITVL_3 = 0;
        DMAC.CHEXT_3 = 0;

        /* PR : Set Round Robin mode */
#if 0
        DMAC07.DCTRL.LONG = 0;
        DMAC07.DCTRL.BIT.PR = 1;
        Trace("DMAC07.DCTRL.LONG = 0x%.8lX\r\n", DMAC07.DCTRL.LONG);
#else
        DMAC.DCTRL_0_7 = 0x00000001;
#endif
        /* Set DMAC channel 3 function */
        R_INTC_RegistIntFunc(INTC_ID_DMAINT3, rzTransferComplete);

        /* Set DMAC channel 3 interrupt priority */
        R_INTC_SetPriority(INTC_ID_DMAINT3, 26);

        /* Enable DMAC channel 3 interrupt */
        R_INTC_Enable(INTC_ID_DMAINT3);

        /* If the DMA channel is not active */
        if ((0 == rza_io_reg_read_32(&DMAC.CHSTAT_3,
                                      DMAC3_CHSTAT_n_EN_SHIFT,
                                      DMAC3_CHSTAT_n_EN)) &&
            (0 == rza_io_reg_read_32(&DMAC.CHSTAT_3,
                                      DMAC3_CHSTAT_n_TACT_SHIFT,
                                      DMAC3_CHSTAT_n_TACT)))
        {
            volatile uint32_t ui_dummy = 0UL;

            /* Issue a software reset */
            rza_io_reg_write_32(&DMAC.CHCTRL_3,
                                1,
                                DMAC3_CHCTRL_n_SWRST_SHIFT,
                                DMAC3_CHCTRL_n_SWRST);
            ui_dummy = rza_io_reg_read_32(&DMAC.CHCTRL_3,
                                    DMAC3_CHCTRL_n_SWRST_SHIFT,
                                    DMAC3_CHCTRL_n_SWRST);

            /* Enable the channel */
            rza_io_reg_write_32(&DMAC.CHCTRL_3,
                                1,
                                DMAC3_CHCTRL_n_SETEN_SHIFT,
                                DMAC3_CHCTRL_n_SETEN);

            /* Software trigger */
            rza_io_reg_write_32(&DMAC.CHCTRL_3,
                                1,
                                DMAC3_CHCTRL_n_STG_SHIFT,
                                DMAC3_CHCTRL_n_STG);

            /* Wait for the interrupt to signal completion */
            eventWait(&gpevTransferComplete, 1, true);

            /* Transfer has completed successfully */
            bfResult = true;
        }

        /* Free the event */
        eventDestroy(&gpevTransferComplete, 1);
        gpevTransferComplete = NULL;
    }
    return bfResult;
}
/**********************************************************************************
 End of function  rzDmaTransfer
 ***********************************************************************************/

/******************************************************************************
 * Function Name: rzFillTestData
 * Description  : Function to fill the file data buffer with known data
 * Arguments    : OUT pulTestData - Pointer to the file data
 * Arguments    : IN  stLength - The length of the file in longs
 * Return Value :  none
 ******************************************************************************/
static void rzFillTestData (uint32_t *pulTestData, size_t stLength)
{
    uint32_t ulData = gulDataStartValue;
    while (stLength--)
    {
        *pulTestData++ = ulData++;
    }
}
/******************************************************************************
 End of function  rzFillTestData
 ******************************************************************************/

/******************************************************************************
 * Function Name: rzCheckTestData
 * Description  : Function to check the file data
 * Arguments    : IN  pulTestData - Pointer to the file data
 * Arguments    : IN  stLength - The length of the file in longs
 * Return Value :  true if the data matches
 ******************************************************************************/
static bool_t rzCheckTestData (uint32_t *pulTestData, size_t stLength)
{
    uint32_t dwData = gulDataStartValue;
    int iCount = 0;
    while (stLength--)
    {
        if (*pulTestData != dwData)
        {
            printf("rzCheckTestData: 0x%p got 0x%.8lX expecting 0x%.8lX\r\n", pulTestData, *pulTestData, dwData);
            return false;
        }
        pulTestData++;
        dwData++;
        iCount++;
    }
    return true;
}
/******************************************************************************
 End of function  rzCheckTestData
 ******************************************************************************/

/***********************************************************************************
 End  Of File
 ***********************************************************************************/
