/******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized.
* This software is owned by Renesas Electronics Corporation and is  protected
* under all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES
* REGARDING THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY,
* INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR  A
* PARTICULAR PURPOSE AND NON-INFRINGEMENT.  ALL SUCH WARRANTIES ARE  EXPRESSLY
* DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE  LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES
* FOR ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS
* AFFILIATES HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this
* software and to discontinue the availability of this software.
* By using this software, you agree to the additional terms and
* conditions found by accessing the following link:
* http://www.renesas.com/disclaimer
*******************************************************************************
* Copyright (C) 2010 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************
* File Name    : devlink.h
* Version      : 1.01
* Description  : Functions to link devices into the C standard IO run time
*                library
*******************************************************************************
* History : DD.MM.YYYY Version Description
*         : 05.08.2010 1.00    First Release
*         : 14.12.2010 1.01    Added devGetFromList
******************************************************************************/

/******************************************************************************
  WARNING!  IN ACCORDANCE WITH THE USER LICENCE THIS CODE MUST NOT BE CONVEYED
  OR REDISTRIBUTED IN COMBINATION WITH ANY SOFTWARE LICENSED UNDER TERMS THE
  SAME AS OR SIMILAR TO THE GNU GENERAL PUBLIC LICENCE
******************************************************************************/

#ifndef DEVLINK_H_INCLUDED
#define DEVLINK_H_INCLUDED

/******************************************************************************
Includes   <System Includes> , "Project Includes"
******************************************************************************/

#include <stdio.h>
#include "control.h"
#include "r_typedefs.h"

/******************************************************************************
Macro definitions
******************************************************************************/

/* Define the maximum string size for the device information strings */
#define DEVICE_MAX_STRING_SIZE      128

/******************************************************************************
Typedef definitions
******************************************************************************/

/* Define the structure of the information that describes a device */
typedef struct _DEVINFO
{
    /* The device symbolic link name */
    char      pszDeviceLinkName[DEVICE_MAX_STRING_SIZE];

    /* The manufacturer string */
    char      pszManufacturer[DEVICE_MAX_STRING_SIZE];

    /* The product string */
    char      pszProduct[DEVICE_MAX_STRING_SIZE];

    /* The serial number string */
    char      pszSerialNumber[DEVICE_MAX_STRING_SIZE];

    /* The device vendor ID */
    uint16_t  wVID;

    /* The device product ID */
    uint16_t  wPID;

    /* The device version in BCD */
    uint16_t  wbcdVersion;

    /* The device class code */
    uint8_t   byDeviceClass;

    /* The device sub class code */
    uint8_t   byDeviceSubClass;

    /* The device protocol code */
    uint8_t   byProtocolCode;
} DEVINFO,
*PDEVINFO;

/* Define the structure of a device which can be loaded at run time */
typedef struct _DYNDEV
{
    /* Pointer to the next on the list */
    struct _DYNDEV *pNext;

    /* Pointer to the device link name */
    int8_t    *pszDeviceLinkName;

    /* Flag to show if the device is allocated */
    _Bool   bfAllocated;

    /* Pointer to functions that open, close, read, write and control the
       device */
    struct _DEVICE *pDeviceDriver;

    /* The information that describes the device */
    DEVINFO Information;
} DYNDEV,
*PDYNDEV;

typedef struct _DEVICE *PDEVICE;

/* Define the structure of the table to hold a pointer to the device driver
   functions, pointer to any driver specific data, pointer to the file and
   access options */
typedef struct _STREAM
{
    PDEVICE pDeviceDriver;
    char    *pszStreamName;
    void    *pExtension;
#if defined( __RX )
    long    iFileNumber;
    long    iFileMode;
    long    iFileFlag;
#else
    int     iFileNumber;
    int     iFileMode;
    int     iFileFlag;
#endif
} STREAM,
*PSTREAM;

/* Define the structure of the functions to access the device driver*/
typedef struct _DEVICE
{
    /* The driver name */
    const char   *const pszDeviceName;

    /* The driver open function */
    int (*const open)(PSTREAM pStream);

    /* The driver close function */
    void (*const close)(PSTREAM pStream);

    /* The driver read function */
    int (*const read)(PSTREAM pStream, uint8_t *pbyBuffer, uint32_t uiCount);

    /* The driver write function */
    int (*const write)(PSTREAM pStream, uint8_t*pbyBuffer, uint32_t uiCount);

    /* A driver control function to handle all other driver specific
       functions */
    int (*const control)(PSTREAM pStream, CTLCODE ctlCode, void *pCtlStruct);
} DEVICE;

/******************************************************************************
Constant Data
******************************************************************************/

extern const DEVICE gNoDriver;

/******************************************************************************
Function Prototypes
******************************************************************************/

#ifdef __cplusplus
extern "C" {
#endif

/******************************************************************************
Function Name: devInitDynamicList
Description:   Function to initialise the dynamic device list
Arguments:     none
Return value:  none
******************************************************************************/

extern  void devInitDynamicList(void);

/******************************************************************************
Function Name: devGetPointer
Description:   Function to get a pointer to the open, close, read, write and
               control functions
Arguments:     IN  pszLinkName - Pointer to the link name
               OUT pszStreamName - Pointer to the stream name
Return value:  Pointer to the device or NULL on error
******************************************************************************/

extern  PDEVICE devGetPointer(char *pszLinkName, char **pszStreamName);

/******************************************************************************
Function Name: devAdd
Description:   Function to add a device to the device list
Arguments:     IN  pszLinkName - Pointer to the device link name, this is used
                                 to open the device
               IN  pDeviceDriver - Pointer to the functions to open, close,
                                   read, write and control the device
               IN  pInformation - Pointer to the information that describes the
                                  device
Return value:  true for success
******************************************************************************/

extern  _Bool devAdd(char    *pszLinkName,
                     PDEVICE  pDeviceDriver,
                     PDEVINFO pInformation);

/******************************************************************************
Function Name: devRemove
Description:   Function to remove a device from the list
Arguments:     IN  pszLinkName - Pointer to the device link name, this is used
                                 to open the device
Return value:  true for success
******************************************************************************/

extern  _Bool devRemove(char *pszLinkName);

/******************************************************************************
Function Name: devList
Description:   Function to list the link names and details of the devices
               attached
Arguments:     IN  pFile - Pointer to the file stream to print the details to
Return value:  The number of devices attached
******************************************************************************/

extern  int devList(FILE *pFile);

/*****************************************************************************
Function Name: devInfo
Description:   Function get the information about a dynamic device
Arguments:     OUT pDynDev - Pointer to the destination dynamic device info
               IN  iIndex - The index of the device
Return value:  true if a device was found
*****************************************************************************/

extern  _Bool devInfo(PDYNDEV pDynDev, int iIndex);

/******************************************************************************
Function Name: nodevOpen
Description:   Function open stub
Arguments:     IN  pStream - Pointer to the file stream
Return value:  0 for success otherwise -1
******************************************************************************/

extern  int nodevOpen(PSTREAM pStream);

/******************************************************************************
Function Name: nodevClose
Description:   Function close stub
Arguments:     IN  pStream - Pointer to the file stream
Return value:  none
******************************************************************************/

extern  void nodevClose(PSTREAM pStream);

/******************************************************************************
Function Name: nodevIO
Description:   Function to handle read or write when the driver has no function
Arguments:     IN  pStream - Pointer to the file stream
               IN  pbyBuffer - Pointer to the destination memory
               IN  uiCount - The number of bytes to read
Return value:  0 for success -1 on error
******************************************************************************/

extern  int nodevIO(PSTREAM pStream, uint8_t *pbyBuffer, uint32_t uiCount);

/******************************************************************************
Function Name: nodevControl
Description:   Function to handle custom controls when the driver has none
Arguments:     IN  pStream - Pointer to the file stream
               IN  ctlCode - The custom control code
               IN  pCtlStruct - Pointer to the custorm control structure
Return value:  0 for success -1 on error
******************************************************************************/

extern  int nodevControl(PSTREAM pStream, CTLCODE ctlCode, void *pCtlStruct);

/******************************************************************************
Function Name: devGetFromList
Description:   Function to check the device list for a device
Arguments:     IN  pszLinkName - Pointer to the device link name
Return value:  Pointer to the device or NULL if not found
******************************************************************************/

extern  PDYNDEV devGetFromList(char *pszLinkName);

#ifdef __cplusplus
}
#endif

#endif /* DEVLINK_H_INCLUDED */

/******************************************************************************
End  Of File
******************************************************************************/
