/******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
*******************************************************************************
* Copyright (C) 2010 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************
* File Name    : devlink.c
* Version      : 1.05
* Description  : Functions to link devices into the C standard IO run time
*                library
******************************************************************************
* History : DD.MM.YYYY Version Description
*         : 04.02.2010 1.00    First Release
*         : 10.06.2010 1.01    Updated type definitions
*         : 10.11.2010 1.02    Comment correction
*         : 14.12.2010 1.03    Added FTDI and made devGetFromList public
*         : 19.01.2011 1.04    Added MCP2200 Device Driver (CDClass)
*         : 17.02.2011 1.05    Modified to Device Link Table
******************************************************************************/

/******************************************************************************
  WARNING!  IN ACCORDANCE WITH THE USER LICENCE THIS CODE MUST NOT BE CONVEYED
  OR REDISTRIBUTED IN COMBINATION WITH ANY SOFTWARE LICENSED UNDER TERMS THE
  SAME AS OR SIMILAR TO THE GNU GENERAL PUBLIC LICENCE
******************************************************************************/

/******************************************************************************
Includes   <System Includes> , "Project Includes"
******************************************************************************/

#include <string.h>
#include <stdio.h>
#include "devlink.h"
#include "r_sysif.h"
#include "drvTimer.h"
#include "drvLed.h"

/* Reference all required device drivers */
#include "hwscif2.h"
#include "drvRTC.h"
#include "hwR1EX24016.h"
#include "hwLCDPMod.h"


/******************************************************************************
Macro Definitions
******************************************************************************/

/* Define the maximum number of devices that are expected to be dynamically
   attached. The USB on chip peripheral can only support 10 devices */
#define DEVICE_LINK_MAX_DEVICES     10

/* This macro clears all data set for a dynamic device on removal */
#define devFree(pDynDev)            memset(pDynDev, 0, sizeof(DYNDEV))

/******************************************************************************
Imported global variables and functions (from other files)
******************************************************************************/

/* Functions provided by lowsrc.c */
extern void closeAll(int8_t *pszStreamName);
extern _Bool _OPEN(PDEVICE pDeviceDriver, char *pszLinkName);

/******************************************************************************
Exported global variables and functions (to be accessed by other files)
******************************************************************************/

static PDYNDEV devAllocDevice(void);
static PDEVICE devGetClassDevice(PDYNDEV *ppDeviceListEntry,
                                 char *pszClassLinkName);

int nodevOpen(PSTREAM pStream);
void nodevClose(PSTREAM pStream);
int nodevIO(PSTREAM pStream, uint8_t *pbyBuffer, uint32_t uiCount);
int nodevControl(PSTREAM pStream, CTLCODE ctlCode, void *pCtlStruct);

/******************************************************************************
Private global variables and functions
******************************************************************************/

/* Define a driver function table for no device */
const DEVICE gNoDriver =
{
    "No Device Driver",
    nodevOpen,
    nodevClose,
    nodevIO,
    nodevIO,
    nodevControl
};

/* Define the static "Mount Table" in which a device is associated with a
   link name */
static const struct _MOUNTTABLE
{
    const char    *pszStreamName;
    const PDEVICE  pDeviceDriver;

} gMountTable[] =
{
    "stdin",  (PDEVICE)&gScif2Driver,
    "stdout", (PDEVICE)&gScif2Driver,
    "stderr", (PDEVICE)&gScif2Driver,
    "swtimer",(PDEVICE)&gTimerDriver,
    "led",    (PDEVICE)&gLedDriver,
    "lcdpmod",(PDEVICE)&gLCDPModDriver,
    "rtc",    (PDEVICE)&gRtcDriver,
    "eeprom", (PDEVICE)&gR1EX24016Driver,
    /* TODO: Add other devices in here */
};

/* The memory for the dynamic devices */
static DYNDEV gpDevices[DEVICE_LINK_MAX_DEVICES];

/* The dynamic device list pointer */
static PDYNDEV gpDynamicDeviceList = NULL;

/* The mutex object to protect the list in a multi tasking environment */
static void *gpevDeviceListLock = NULL;

/******************************************************************************
Public Functions
******************************************************************************/

/******************************************************************************
* Function Name: devInitDynamicList
* Description  : Function to initialise the dynamic device list
* Arguments    : none
* Return Value : none
******************************************************************************/
void devInitDynamicList(void)
{
    gpDynamicDeviceList = NULL;
    memset(gpDevices, 0, sizeof(gpDevices));
}
/******************************************************************************
End of function  devInitDynamicList
******************************************************************************/

/******************************************************************************
* Function Name: devGetPointer
* Description  : Function to get a pointer to the open, close, read, write and
*                control functions
* Arguments    : IN  pszLinkName - Pointer to the link name
*                OUT pszStreamName - Pointer to the device link name
* Return Value : Pointer to the device or NULL on error
******************************************************************************/
PDEVICE devGetPointer(char *pszLinkName, char **pszStreamName)
{
    int iDevice = sizeof(gMountTable) / sizeof(struct _MOUNTTABLE);

    /* First, search through all the static devices in the mount table */
    while (iDevice--)
    {
        if (strcmp(pszLinkName, gMountTable[iDevice].pszStreamName) == 0)
        {
            *pszStreamName = (char*)gMountTable[iDevice].pszStreamName;
            return gMountTable[iDevice].pDeviceDriver;
        }
    }
    return NULL;
}
/******************************************************************************
End of function devGetPointer
******************************************************************************/

/******************************************************************************
* Function Name: devAdd
* Description  : Function to add a device to the device list
* Arguments    : IN  pszLinkName - Pointer to the device link name, this is used
*                                  to open the device
*                IN  pDeviceDriver - Pointer to the functions to open, close,
*                                    read, write and control the device
*                IN  pInformation - Pointer to the information that describes the
*                                   device
* Return Value : true for success
******************************************************************************/
_Bool devAdd(char *pszLinkName, PDEVICE pDeviceDriver, PDEVINFO pInformation)
{
    PDYNDEV pNewDevice;

    /* Allocate the memory */
    pNewDevice = devAllocDevice();
    if (pNewDevice)
    {
        PDYNDEV *ppEndOfList = &gpDynamicDeviceList;

        /* Set the information */
        pNewDevice->pszDeviceLinkName = pszLinkName;
        pNewDevice->pDeviceDriver = pDeviceDriver;
        pNewDevice->Information = *pInformation;

        /* ACQUIRE MUTEX LIST LOCK */
        sysLock(&gpevDeviceListLock);

        /* Find the end of the list */
        while (*ppEndOfList)
        {
            ppEndOfList = &(*ppEndOfList)->pNext;
        }

        /* Add the to the list */
        if (ppEndOfList)
        {
            /* Add to the End of the list */
            *ppEndOfList = pNewDevice;
        }
        else
        {
            /* Add to the top of the list */
            pNewDevice->pNext = gpDynamicDeviceList;
            gpDynamicDeviceList = pNewDevice;
        }

        /* RELEASE MUTEX LIST LOCK */
        sysUnlock(&gpevDeviceListLock, 0);

        return true;
    }
    return false;
}
/******************************************************************************
End of function  devAdd
******************************************************************************/

/******************************************************************************
* Function Name: devRemove
* Description  : Function to remove a device from the list
* Arguments    : IN  pszLinkName - Pointer to the device link name, this is
*                                  used to open the device
* Return Value : true for success
******************************************************************************/
_Bool devRemove(char *pszLinkName)
{
    PDYNDEV pDynamicDevice;
    _Bool   bfResult;
    _Bool   bfRemoved = false;

    /* One device can support more than one interface */
    do {
        bfResult = false;

        /* See if the device exists */
        pDynamicDevice = devGetFromList(pszLinkName);
        if (pDynamicDevice)
        {
            /* Look for the device on the list */
            PDYNDEV *ppDynamicDeviceList = &gpDynamicDeviceList;

            /* ACQUIRE MUTEX LIST LOCK */
            sysLock(&gpevDeviceListLock);

            while ((*ppDynamicDeviceList)
            &&     (pDynamicDevice != *ppDynamicDeviceList))
            {
                ppDynamicDeviceList = &(*ppDynamicDeviceList)->pNext;
            }

            /* If it is found */
            if (*ppDynamicDeviceList)
            {
                /* Close the file stream that use it */
                closeAll(pszLinkName);

                /* Remove from the device list */
                *ppDynamicDeviceList = pDynamicDevice->pNext;

                /* Free the device */
                devFree(pDynamicDevice);

                bfResult = true;
                bfRemoved = true;
            }

            /* RELEASE MUTEX LIST LOCK */
            sysUnlock(&gpevDeviceListLock, 0);
        }

     /* Remove more than one device with the same link name */
    } while (bfResult);

    return bfRemoved;
}
/******************************************************************************
End of function  devRemove
******************************************************************************/

/******************************************************************************
* Function Name: devList
* Description  : Function to list the link names and details of the devices
*                attached
* Arguments    : IN  pFile - Pointer to the file stream to print the details to
* Return Value : The number of devices attached
******************************************************************************/
int devList(FILE *pFile)
{
    int iDeviceCount = 0;
    PDYNDEV pDynamicDevice = gpDynamicDeviceList;

    /* ACQUIRE MUTEX LIST LOCK */
    sysLock(&gpevDeviceListLock);
    while (pDynamicDevice)
    {
        iDeviceCount++;
        fprintf(pFile, "Link name: %s\r\n",pDynamicDevice->pszDeviceLinkName);
        if (strlen(pDynamicDevice->Information.pszManufacturer))
        {
            fprintf(pFile,
                    "Manufacturer: %s\r\n",
                    pDynamicDevice->Information.pszManufacturer);
        }
        else
        {
            fprintf(pFile, "Manufacturer: Unknown\r\n");
        }
        if (strlen(pDynamicDevice->Information.pszProduct))
        {
            fprintf(pFile,
                    "Product: %s\r\n",
                    pDynamicDevice->Information.pszProduct);
        }
        else
        {
            fprintf(pFile, "Product: Unknown\r\n");
        }
        if (strlen(pDynamicDevice->Information.pszSerialNumber))
        {
            fprintf(pFile,
                    "Serial Number: %s\r\n",
                    pDynamicDevice->Information.pszSerialNumber);
        }
        else
        {
            fprintf(pFile, "Serial Number: Unknown\r\n");
        }
        fprintf(pFile, "Vendor ID: 0x%.4X\r\n",
                       pDynamicDevice->Information.wVID);
        fprintf(pFile, "Product ID: 0x%.4X\r\n",
                       pDynamicDevice->Information.wPID);
        fprintf(pFile, "Device Class: 0x%.2X\r\n",
                       pDynamicDevice->Information.byDeviceClass);
        fprintf(pFile, "Device Sub Class: 0x%.2X\r\n",
                       pDynamicDevice->Information.byDeviceSubClass);
        fprintf(pFile, "Device Protocol: 0x%.2X\r\n",
                       pDynamicDevice->Information.byProtocolCode);
        if (pDynamicDevice->pDeviceDriver)
        {
            fprintf(pFile,
                    "Device Driver \"%s\" available for this device\r\n\r\n",
                    pDynamicDevice->pDeviceDriver->pszDeviceName);
        }
        else
        {
            fprintf(pFile, "There is no device driver available\r\n\r\n");
        }

        /* Check the next device */
        pDynamicDevice = pDynamicDevice->pNext;
    }

    /* RELEASE MUTEX LIST LOCK */
    sysUnlock(&gpevDeviceListLock, 0);
    return iDeviceCount;
}
/******************************************************************************
End of function  devList
******************************************************************************/

/*****************************************************************************
Function Name: devInfo
Description:   Function get the information about a dynamic device
Arguments:     OUT pDynDev - Pointer to the destination dynamic device info
               IN  iIndex - The index of the device
Return value:  true if a device was found
*****************************************************************************/
_Bool devInfo(PDYNDEV pDynDev, int iIndex)
{
    _Bool bfResult = false;
    int iDeviceCount = 0;
    PDYNDEV pDynamicDevice = gpDynamicDeviceList;

    /* ACQUIRE MUTEX LIST LOCK */
    sysLock(&gpevDeviceListLock);
    while (pDynamicDevice)
    {
        if (iIndex == iDeviceCount)
        {
            *pDynDev = *pDynamicDevice;
            bfResult = true;
            break;
        }
        else
        {
            /* Check the next device */
            iDeviceCount++;
            pDynamicDevice = pDynamicDevice->pNext;
        }
    }

    /* RELEASE MUTEX LIST LOCK */
    sysUnlock(&gpevDeviceListLock, 0);
    return bfResult;
}
/*****************************************************************************
End of function  devInfo
******************************************************************************/

/******************************************************************************
* Function Name: nodevOpen
* Description  : Function open stub
* Arguments    : IN  pStream - Pointer to the file stream
* Return Value : 0 for success otherwise -1
******************************************************************************/
int nodevOpen(PSTREAM pStream)
{
    (void)pStream;
    return -1;
}
/******************************************************************************
End of function  nodevOpen
******************************************************************************/

/******************************************************************************
* Function Name: nodevClose
* Description  : Function close stub
* Arguments    : IN  pStream - Pointer to the file stream
* Return Value : none
******************************************************************************/
void nodevClose(PSTREAM pStream)
{
    (void)pStream;
}
/******************************************************************************
End of function  nodevClose
******************************************************************************/

/******************************************************************************
* Function Name: nodevIO
* Description  : Function read / write stub
* Arguments    : IN  pStream - Pointer to the file stream
*                IN  pbyBuffer - Pointer to the destination memory
*                IN  uiCount - The number of bytes to read
* Return Value : 0 for success -1 on error
******************************************************************************/
int nodevIO(PSTREAM pStream, uint8_t *pbyBuffer, uint32_t uiCount)
{
    (void)pStream;
    (void)pbyBuffer;
    (void)uiCount;
    return -1;
}
/******************************************************************************
End of function  nodevIO
******************************************************************************/

/******************************************************************************
* Function Name: nodevControl
* Description  : Function to control stub
* Arguments    : IN  pStream - Pointer to the file stream
*                IN  ctlCode - The custom control code
*                IN  pCtlStruct - Pointer to the custorm control structure
* Return Value : 0 for success -1 on error
******************************************************************************/
int nodevControl(PSTREAM pStream, CTLCODE ctlCode, void *pCtlStruct)
{
    (void)pStream;
    (void)ctlCode;
    (void)pCtlStruct;
    return -1;
}
/******************************************************************************
End of function  nodevControl
******************************************************************************/

/******************************************************************************
Function Name: devGetFromList
Description:   Function to check the device list for a device
Arguments:     IN  pszLinkName - Pointer to the device link name
Return value:  Pointer to the device or NULL if not found
******************************************************************************/
PDYNDEV devGetFromList(char *pszLinkName)
{
    PDYNDEV pDynamicDevice = gpDynamicDeviceList;

    /* Now search through all the dynamic devices in the device list */
    while (pDynamicDevice)
    {
        if (strcmp(pDynamicDevice->pszDeviceLinkName, pszLinkName))
        {
            pDynamicDevice = pDynamicDevice->pNext;
        }
        else
        {
            return pDynamicDevice;
        }
    }
    return NULL;
}
/******************************************************************************
End of function  devGetFromList
******************************************************************************/

/******************************************************************************
Private global variables and functions
******************************************************************************/

/******************************************************************************
* Function Name: devAllocDevice
* Description  : Function to allocate a dynamic device
* Arguments    : none
* Return Value : Pointer to the device or NULL on error
******************************************************************************/
static PDYNDEV devAllocDevice(void)
{
    int iIndex;
    for (iIndex = 0; iIndex < DEVICE_LINK_MAX_DEVICES; iIndex++)
    {
        if (!gpDevices[iIndex].bfAllocated)
        {
            gpDevices[iIndex].bfAllocated = true;
            return &gpDevices[iIndex];
        }
    }
    return NULL;
}
/******************************************************************************
End of function  devAllocDevice
******************************************************************************/

/******************************************************************************
Function Name: devGetClassDevice
Description:   Function to get a device
Arguments:     OUT ppDeviceListEntry - Pointer to the entry of the device on
                                       the list
               IN  pszClassLinkName - Pointer to the class link name
Return value:  Pointer to the first available device driver
               NULL is returned if none avaialble
******************************************************************************/
static PDEVICE devGetClassDevice(PDYNDEV *ppDeviceListEntry,
                                 char    *pszClassLinkName)
{
    PDYNDEV pDynamicDevice = gpDynamicDeviceList;

    /* Now search through all the dynamic devices in the device list */
    while (pDynamicDevice)
    {
        if ((_OPEN(pDynamicDevice->pDeviceDriver,
                   pDynamicDevice->pszDeviceLinkName))
        ||  (strcmp(pDynamicDevice->pDeviceDriver->pszDeviceName,
                    pszClassLinkName)))
        {
            pDynamicDevice = pDynamicDevice->pNext;
        }
        else
        {
            if (ppDeviceListEntry)
            {
                *ppDeviceListEntry = pDynamicDevice;
            }
            return pDynamicDevice->pDeviceDriver;
        }
    }
    return NULL;
}
/******************************************************************************
End of function  devGetClassDevice
******************************************************************************/

/******************************************************************************
End  Of File
******************************************************************************/
