/******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
*******************************************************************************
* Copyright (C) 2011 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************
* File Name    : r_sys_initiolib.c
* Version      : 1.01
* Description  : low level ANSI IO functions
*******************************************************************************
* History      : DD.MM.YYYY Version Description
*              : 04.02.2010 1.00    First Release
*              : 10.06.2010 1.01    Updated type definitions
******************************************************************************/

/******************************************************************************
  WARNING!  IN ACCORDANCE WITH THE USER LICENCE THIS CODE MUST NOT BE CONVEYED
  OR REDISTRIBUTED IN COMBINATION WITH ANY SOFTWARE LICENSED UNDER TERMS THE
  SAME AS OR SIMILAR TO THE GNU GENERAL PUBLIC LICENCE
******************************************************************************/

/******************************************************************************
Includes   <System Includes> , "Project Includes"
******************************************************************************/

#include <string.h>
#include <stdio.h>
#include <stddef.h>
#include <reent.h>
#include <fcntl.h>
#include "devLink.h"
#include "r_sysIf.h"
#include "swi.h"
#include "r_sys_initiolib.h"

/******************************************************************************
Macro definitions
******************************************************************************/


/* The fixed file descriptors required by the run time library */

/* Standard input (console) */
#define STDIN                       (0)

/* Standard output (console) */
#define STDOUT                      (1)

/* Standard error output (console) */
#define STDERR                      (2)

/* The first file descriptor available to the user */
#define FIRST_FILE                  (3)

/******************************************************************************
Imported global variables and functions
******************************************************************************/

extern void initialise_monitor_handles(void);

/******************************************************************************
Private global variables and functions
******************************************************************************/

static STREAM filestream[IOSTREAM];
static int_t get_first_free_file(const char *pszFileName);
static char *get_device_name(char *pszFileName);

/* The string that is used to differentiate a device name from a file name */
static const char * const psz_dev_ident = DEVICE_INDENTIFIER;

/******************************************************************************
Constant Data
******************************************************************************/

#if defined( __RX )
/* The number of files for input/output files */
const long g_nfiles = IOSTREAM;
#else
/* The number of files for input/output files */
const int_t g_nfiles = IOSTREAM;
#endif

/******************************************************************************
Exported global variables and functions (to be accessed by other files)
******************************************************************************/

/* Device file descriptors for global reference */
int_t g_irtc = -1;
int_t g_itimer = -1;
int_t g_iled = -1;

/******************************************************************************
* Function Name: low_open
* Description  : ANSI library low level open function
* Arguments    : IN  pszFileName - Pointer to the file name
*                IN  iFileFlag - Passed on to open function
* Return Value : File number or -1 on error
******************************************************************************/
int low_open(char *pszFileName, int iFileFlag)
{
    /* Get the fist available file number */
    int_t file_number;
    sysWaitAccess();
    file_number = get_first_free_file(pszFileName);
    if (file_number >= 0)
    {
        /* First look for a device */
        char *pszlink_name = get_device_name(pszFileName);

        /* Initialise the driver pointer */
        filestream[file_number].pDeviceDriver = &gNoDriver;

        /* Check to see if this is a device driver not a file */
        if (pszlink_name)
        {
            char    *pszstream_name;
            PDEVICE pdevice_driver = devGetPointer(pszlink_name, &pszstream_name);

            if (pdevice_driver)
            {
                filestream[file_number].pszStreamName = pszstream_name;
                filestream[file_number].pDeviceDriver = pdevice_driver;
            }
        }

        /* If we found a device or a file then open it */
        if ((filestream[file_number].pDeviceDriver)
        &&  (&gNoDriver != filestream[file_number].pDeviceDriver))
        {
            PSTREAM pstream = &filestream[file_number];
            int_t iresult;

            pstream->iFileNumber = file_number;
            if (0 == iFileFlag)
            {
                /* Always read only */
                pstream->iFileMode = __SRD;
            }

            /* Check for read and write */
            if (iFileFlag & O_RDWR)
            {
                pstream->iFileMode = (__SRD | __SWR);
            }

            /* Check for write only */
            else if (iFileFlag & (O_WRONLY))
            {
                pstream->iFileMode = __SWR;
            }

            else
            {
                /* Read Only not supported*/
            }

            /* Check for create & add write */
            if (iFileFlag & O_CREAT)
            {
                pstream->iFileMode |= __SWR;
            }
            pstream->iFileFlag = iFileFlag;

            iresult = pstream->pDeviceDriver->open(pstream);
            if (iresult)
            {
                memset(pstream, 0, sizeof(STREAM));

                /* Device failed to open */
                sysReleaseAccess();
                return (-1);
            }
            sysReleaseAccess();
            return file_number;
        }
        else
        {
            filestream[file_number].pDeviceDriver = NULL;
        }
    }

    /* File not available */
    sysReleaseAccess();
    return (-1);
}
/******************************************************************************
End of function low_open
******************************************************************************/

/******************************************************************************
* Function Name: low_close
* Description  : ANSI library low level close function
* Arguments    : IN  iFileNumber - The file number to close
* Return Value : 0 for success, -1 on error
******************************************************************************/
int low_close(int iFileNumber)
{
    /* File number range check */
    if ((iFileNumber < 0) || ((FOPEN_MAX - 1) < iFileNumber))
    {
        return (-1);
    }

    /* File open check */
    if (filestream[iFileNumber].pDeviceDriver)
    {
        PSTREAM pstream = &filestream[iFileNumber];

        /* Close the stream */
        pstream->pDeviceDriver->close(pstream);
        memset(pstream, 0, sizeof(STREAM));
        return 0;
    }
    return -1;
}
/******************************************************************************
End of function low_close
******************************************************************************/

/******************************************************************************
* Function Name: low_read
* Description  : ANSI library low level read function
* Arguments    : IN  iFileNumber - The file index
*                IN  pbyBuffer - The pointer to the buffer
*                IN  iCount - The number of characters to read
* Return Value : Number of characters read, -1 on error
******************************************************************************/
int low_read(int iFileNumber, uint8_t *pbyBuffer, uint32_t iCount)
{
    /* File number range check */
    if ((iFileNumber < 0) || ((FOPEN_MAX - 1) < iFileNumber))
    {
        return (-1);
    }

    /* File open check */
    if (filestream[iFileNumber].pDeviceDriver)
    {
        PSTREAM pstream = &filestream[iFileNumber];

        /* File access rights check */
        if (pstream->iFileMode & __SRD)
        {
            return pstream->pDeviceDriver->read(pstream, pbyBuffer, iCount);
        }
    }
    return (-1);
}
/******************************************************************************
End of function low_read
******************************************************************************/

/******************************************************************************
* Function Name: low_write
* Description  : ANSI library low level read function
* Arguments    : IN  iFileNumber - The file index
*                IN  pbyBuffer - The pointer to the buffer
*                IN  iCount - The number of characters to write
* Return Value : Number of characters written, -1 on error
******************************************************************************/
int low_write(int iFileNumber, uint8_t *pbyBuffer, uint32_t iCount)
{
    /* File number range check */
    if ((iFileNumber < 0) || ((FOPEN_MAX - 1) < iFileNumber))
    {
        return (-1);
    }

    /* File open check */
    if (filestream[iFileNumber].pDeviceDriver)
    {
        PSTREAM pstream = &filestream[iFileNumber];

        /* File access rights check */
        if (pstream->iFileMode & __SWR)
        {
            return pstream->pDeviceDriver->write(pstream, (uint8_t * const)pbyBuffer, iCount);
        }
    }
    return (-1);
}
/******************************************************************************
End of function low_write
******************************************************************************/

/******************************************************************************
* Function Name: low_lseek
* Description  : ANSI library low level seek function
* Arguments    : IN  iFileNumber - The file index
*                IN  lOffset - The offset
*                IN  iBase - The base
* Return Value : The offset in bytes to the current file position or -1 on error
******************************************************************************/
long low_lseek(int iFileNumber, long lOffset, int iBase)
{
    /* File number range check */
    if ((iFileNumber < 0) || ((FOPEN_MAX - 1) < iFileNumber))
    {
        return (-1L);
    }

    /* File open check */
    if (filestream[iFileNumber].pDeviceDriver)
    {
        PSTREAM pstream = &filestream[iFileNumber];
        FILESEEK file_seek;
        file_seek.lOffset = lOffset;
        file_seek.iBase = iBase;
        file_seek.lResult = 0L;
        if ((pstream->pDeviceDriver->control(pstream,
                                             CTL_FILE_SEEK, &file_seek))
        &&  (0L == file_seek.lResult))
        {
            file_seek.lResult = (-1L);
        }
        return file_seek.lResult;
    }
    return -1L;
}
/******************************************************************************
End of function low_lseek
******************************************************************************/

/******************************************************************************
* Function Name: control (NOT ANSI)
* Description  : Function to perform a device control provided by a control code.
*                This is not a standard ANSI function. It is provided as an extension
*                to the ANSI architecture.
* Arguments    : IN  iFileNumber - The file number to control
*                IN  ctlCode - The device specific control code
*                IN  pCtlStruct - Pointer to the control structure
* Return Value : 0 for success -1 on error
******************************************************************************/
int control(int iFileNumber, CTLCODE ctlCode, void *pCtlStruct)
{
    /* File number range check */
    if ((iFileNumber < 0) || ((FOPEN_MAX - 1) < iFileNumber))
    {
        return -1L;
    }

    /* File open check */
    if (filestream[iFileNumber].pDeviceDriver)
    {
        PSTREAM pstream = &filestream[iFileNumber];
        return pstream->pDeviceDriver->control(pstream, ctlCode, pCtlStruct);
    }
    return -1;
}
/******************************************************************************
End of function  control
******************************************************************************/

/*****************************************************************************
Function Name: filePointerToDescriptor
Description:   Function to convert a file pointer into a file descriptor
Arguments:     IN  pFile - Pointer to the file stream;
Return value:  The file descriptor
*****************************************************************************/
int filePointerToDescriptor(FILE *pFile)
{
    if (pFile)
    {
        struct __sFILE *pinternal = (struct __sFILE *)pFile;
        return (int)pinternal->_file;
    }
    else
    {
        return -1;
    }
}
/*****************************************************************************
End of function  filePointerToDescriptor
******************************************************************************/

/******************************************************************************
* Function Name: closeAll
* Description  : Function to close all files of a given name
* Arguments    : IN  pszStreamName - Pointer to the stream name to close
* Return Value : none
******************************************************************************/
void closeAll(char *pszStreamName)
{
    int_t iindex = FOPEN_MAX;

    /* For each file stream */
    while (iindex--)
    {
        /* If the device is open */
        if ((filestream[iindex].pDeviceDriver)

        /* Check the link name */
        &&  (strcmp(pszStreamName, filestream[iindex].pszStreamName) == 0))
        {
            low_close(filestream[iindex].iFileNumber);
        }
    }
}
/******************************************************************************
End of function  closeAll
******************************************************************************/

/******************************************************************************
* Function Name: _OPEN
* Description  : Function to find out if a device driver is open
* Arguments    : IN  pDeviceDriver - Pointer to the device driver
*                IN  pszLinkName - Pointer to the link name
* Return Value : true if the device is open
******************************************************************************/
_Bool _OPEN(PDEVICE pDeviceDriver, char *pszLinkName)
{
    int_t iindex = FOPEN_MAX;

    /* For each file stream */
    while (iindex--)
    {
        if (filestream[iindex].pDeviceDriver == pDeviceDriver)
        {
            if (strcmp(filestream[iindex].pszStreamName, pszLinkName) == 0)
            {
                return true;
            }
        }
    }
    return false;
}
/******************************************************************************
End of function  _OPEN
******************************************************************************/

/*****************************************************************************
Function Name: fileUsage
Description:   Function to calculate the file usage
Arguments:     OUT pstInUse - Pointer to the number of files in use
               OUT pstTotal - Pointer to the maximum number of files that can
                              be open
Return value:  none
*****************************************************************************/
void fileUsage(size_t *pstInUse, size_t *pstTotal)
{
    int_t   ifile_number;
    size_t  stcount = 0;
    for (ifile_number = FIRST_FILE; ifile_number < IOSTREAM; ifile_number++)
    {
        if (NULL == filestream[ifile_number].pDeviceDriver)
        {
            stcount++;
        }
    }
    if (pstTotal)
    {
        (*pstTotal) = ((size_t)IOSTREAM);
    }
    if (pstInUse)
    {
        (*pstInUse) = ((size_t)IOSTREAM - stcount);
    }
}
/*****************************************************************************
End of function  fileUsage
******************************************************************************/

/******************************************************************************
Private global variables and functions
******************************************************************************/

/******************************************************************************
* Function Name: get_first_free_file
* Description  : Function to get the first free file number
* Arguments    : IN pszFileName - Pointer to the file name. This is to reserve
                                  the file descriptors for the standard streams
                                    defined in stdio.h, stdin, stdout and stderr
* Return Value : File number or -1 on error
******************************************************************************/
static int get_first_free_file(const char *pszFileName)
{
    /* Reserve the file descriptors for stdin stdout and stderr as defined by
       the run time library */
    if (strcmp(pszFileName, DEVICE_INDENTIFIER "stdin") == 0)
    {
        return STDIN;
    }
    else if (strcmp(pszFileName, DEVICE_INDENTIFIER "stdout") == 0)
    {
        return STDOUT;
    }
    else if (strcmp(pszFileName, DEVICE_INDENTIFIER "stderr") == 0)
    {
        return STDERR;
    }
    else
    {
        /* Allocate all other file descriptors dynamically */
        int_t ifile_number;
        for (ifile_number = FIRST_FILE; ifile_number < FOPEN_MAX; ifile_number++)
        {
            if (NULL == filestream[ifile_number].pDeviceDriver)
            {
                return ifile_number;
            }
        }
    }
    return -1;
}
/******************************************************************************
End of function  get_first_free_file
******************************************************************************/

/******************************************************************************
* Function Name: get_device_name
* Description  : Function to see if the file name supplied is for a device
* Arguments    : IN  pszFileName - Pointer to the file name
* Return Value : Pointer to the device link name or NULL on failure
******************************************************************************/
static char *get_device_name(char *pszFileName)
{
    if (strstr(pszFileName, psz_dev_ident) == pszFileName)
    {
        return (pszFileName += (sizeof(psz_dev_ident)));
    }
    return NULL;
}
/******************************************************************************
End of function  get_device_name
******************************************************************************/

/******************************************************************************
* Function Name: R_SYS_InitIolib
* Description  : ANSI library low level initialisation function
* Arguments    : none
* Return Value : none
******************************************************************************/
void R_SYS_InitIolib(void)
{
    /* Initialise the ANSI IO stream tables */
    memset(filestream, 0, sizeof(filestream));

    /* Initialise the list of dynamic devices */
    devInitDynamicList();

    /* Call the NewLib IO initialisation function */
    initialise_monitor_handles();

    /* Open the software timer */
    g_itimer = open(DEVICE_INDENTIFIER "swtimer", 0);

    /* Open the real time clock */
    g_irtc = open(DEVICE_INDENTIFIER "rtc", 0);

    /* Open the LED interface */
    g_iled = open(DEVICE_INDENTIFIER "led", 0);

}
/******************************************************************************
End of function R_SYS_InitIolib
******************************************************************************/

/******************************************************************************
End  Of File
******************************************************************************/
