/******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
*******************************************************************************
* Copyright (C) 2011 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************
* File Name    : malloc.h
* Version      : 1.0
* Device(s)    : Renesas
* Tool-Chain   : Renesas SH2A V9+
* OS           : None
* H/W Platform : SH2A
* Description  : Replacement for the standard C run time library memory
                 management functions to make them "task" and multiple memory
                 type aware
*******************************************************************************
* History      : DD.MM.YYYY Ver. Description
*              : 01.08.2009 1.00 First Release
*              : 14.12.2020 1.01 Added SRAM Heap
******************************************************************************/

/******************************************************************************
  WARNING!  IN ACCORDANCE WITH THE USER LICENCE THIS CODE MUST NOT BE CONVEYED
  OR REDISTRIBUTED IN COMBINATION WITH ANY SOFTWARE LICENSED UNDER TERMS THE
  SAME AS OR SIMILAR TO THE GNU GENERAL PUBLIC LICENCE
******************************************************************************/

#ifndef MALLOC_H_INCLUDED
#define MALLOC_H_INCLUDED

/******************************************************************************
Includes   <System Includes> , "Project Includes"
******************************************************************************/

#include "r_typedefs.h"

/******************************************************************************
Macro definitions
******************************************************************************/

/* Define the memory that is used for malloc, calloc, realloc and free */
#define MALLOC_MEMORY_TYPE          HEAP_SRAM

/******************************************************************************
Typedef definitions
******************************************************************************/
typedef enum _MEMTYPE
{
    HEAP_RTOS_STACK = 0,
    HEAP_WEB_SERVER,
    HEAP_LWIP_PRIMARY,
    HEAP_SRAM,
    HEAP_SDRAM,
    NUMBER_OF_HEAPS
} MEMTYPE;

typedef struct _HEAPUSAGE
{
    /* The total heap size */
    size_t  stTotalHeap;

    /* The total free heap */
    size_t  stTotalFree;

    /* The total allocated */
    size_t  stTotalAllocated;

    /* The allocation overhead */
    size_t  stOverhead;

    /* The biggest allocation possible */
    size_t  stLargestFreeSegment;

    /* The Peak allocation since last reset */
    size_t stPeakAllocated;
} HEAPUSAGE,
*PHEAPUSAGE;


extern void vApplicationMallocFailedHook (void);


/*****************************************************************************
Public Functions
******************************************************************************/

#ifdef __cplusplus
extern "C" {
#endif

/*****************************************************************************
Function Name: exMalloc
Description:   Function to allocate memory from memory of different attributes
Arguments:     IN  memType - The heap attribute or memory type
               IN  stLength - The length to allocate
Return value:  Pointer to the memory or NULL if not enough free
*****************************************************************************/

extern  void *exMalloc(MEMTYPE memType, size_t stLength);

/*****************************************************************************
Function Name: exCalloc
Description:   Function to free memory from memory of different attributes
Arguments:     IN  memType - The heap attribute or memory type
               IN  stNumber - The number of objects
               IN  stLength - The length to allocate
Return value:  Pointer to the memory or NULL if not enough free
*****************************************************************************/

extern  void *exCalloc(MEMTYPE memType, size_t stNumber, size_t stLength);

/*****************************************************************************
Function Name: exRealloc
Description:   Function to reallocate memory from memory of different
               attributes
Arguments:     IN  memType - The heap attribute or memory type
               IN  pvReAlloc - Pointer to the memory to block to change the
                               size of
               IN  stLength - The new length of the block
Return value:  Pointer to the re-allocated memory or NULL if not enough free
*****************************************************************************/

extern  void *exRealloc(MEMTYPE memType, void *pvReAlloc, size_t stLength);

/*****************************************************************************
Function Name: exFree
Description:   Function to free memory from memory of different attributes
Arguments:     IN  memType - The heap attribute or memory type
               IN  pvFree - Pointer to the memory to free
Return value:  none
*****************************************************************************/

extern  void exFree(MEMTYPE memType, void *pvFree);

/*****************************************************************************
Function Name: exFreeByTaskID
Description:   Function to free the memory by TaskID
Arguments:     IN  pvTaskHandle - The ID of the task to free
Return value:  The quantity of memory freed
*****************************************************************************/

extern  size_t exFreeByTaskID(void *pvTaskHandle);

/*****************************************************************************
Function Name: exMallocMutex
Description:   Function to return true when the malloc list is in use
Arguments:     none
Return value:  true when the malloc list is in use
*****************************************************************************/

extern  _Bool exMallocMutex(void);

/*****************************************************************************
Function Name: heapUsage
Description:   Function to get the heap usage information
Arguments:     OUT pUsage - Pointer to the destination usage information
               IN  memType - The memory type
Return value:  none
*****************************************************************************/

extern  void heapUsage(PHEAPUSAGE pUsage, MEMTYPE memType);

/*****************************************************************************
Function Name: heapSearchObject
Description:   Function to search the heap for an object. This is because lwIP
               leaks memory and a special function is required to debug it
Arguments:     IN  memType - The heap to search
               IN  stSize - The size of the object to look for
               IN/OUT pstOffset - The pointer to offset into the heap
Return value:  Pointer to the first object on the heap after the offset
               argument of the described size or NULL
*****************************************************************************/

extern void *heapSearchObject(MEMTYPE memType, size_t stSize, size_t *pstOffset);

#ifdef __cplusplus
}
#endif

#endif /* MALLOC_H_INCLUDED */

/******************************************************************************
End  Of File
******************************************************************************/
