/******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only 
* intended for use with Renesas products. No other uses are authorized. This 
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS 
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE 
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the 
* following link:
* http://www.renesas.com/disclaimer
*******************************************************************************
* Copyright (C) 2011 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************
* File Name    : mabMalloc.c
* Version      : 1.02
* Description  : Object oriented version of malloc and free with memory blocks
                 tagged with a "task handle"

  This is required for several reasons:

  1. The memory list managed by malloc,calloc,realloc must be protected by a
     mutex event to prevent multiple tasks breaking the list.
  2. There is more than one type of memory in an embedded system. In this case
     there is internal RAM which is small and fast, SDRAM which is slow and
     vast and 1M Byte of internal static RAM!
  3. The memory allocated by a task needs to be identified so "leaks" can be
     cleaned up when the task stops.

*******************************************************************************
* History      : DD.MM.YYYY Version Description
*              : 04.02.2010 1.00    First Release
*              : 19.01.2011 1.01    Added ASSERT to memory allocation
*              : 18.03.2011 1.02    Added Allocation storage record updaters
******************************************************************************/

/******************************************************************************
  WARNING!  IN ACCORDANCE WITH THE USER LICENCE THIS CODE MUST NOT BE CONVEYED
  OR REDISTRIBUTED IN COMBINATION WITH ANY SOFTWARE LICENSED UNDER TERMS THE
  SAME AS OR SIMILAR TO THE GNU GENERAL PUBLIC LICENCE
******************************************************************************/

/******************************************************************************
Includes   <System Includes> , "Project Includes"
******************************************************************************/

#include <string.h>

/*****************************************************************************
User Includes
******************************************************************************/

#include "mabMalloc.h"
#include "trace.h"

/*****************************************************************************
Function Prototypes
******************************************************************************/

static void mabInsertMBL(PMBL pMBL, PMBL pNewMBL);

/*****************************************************************************
Public Functions
******************************************************************************/
#ifdef _USE_IRAM_
#pragma section IRAM
#endif
/******************************************************************************
Function Name: mabMalloc
Description:   Function to allocate a block of memory
Arguments:     IN  pMAB - Pointer to the memory allocation block
               IN  pvTaskHandle - The ID of the task that requires the memory
               IN  stLength - The length of memory to allocate
Return value:  Pointer to the allocated memory or NULL if no memory free
******************************************************************************/
void *mabMalloc(PMAB pMAB, void *pvTaskHandle, size_t stLength)
{
    void *pvReturn = NULL;
    if ((pMAB)
    &&  (stLength))
    {
        PMBL    pMBL;
        /* Align to the desired number of bytes */
        stLength += (sizeof(UMBL) - 1);
        stLength &= ~(sizeof(UMBL) - 1);
        /* Check for room at the top of the heap */
        if ((stLength + sizeof(UMBL))
        <=  (size_t)(pMAB->pbyHeapEnd - pMAB->pbyHeap))
        {
            /* Allocate memory from here */
            pMBL = (PMBL)pMAB->pbyHeap;
            /* Set the ID so the block is in use */
            pMBL->pvTaskHandle = pvTaskHandle;
            /* Calculate the new heap end */
            pMAB->pbyHeap += (stLength + sizeof(UMBL));
            /* Set the next pointer to the end of the heap */
            pMBL->pNext = (PMBL)pMAB->pbyHeap;
            /* Set the allocation address */
            pvReturn = (void*)(pMAB->pbyHeap - stLength);
        }
        else /* Otherwise search for a free block big enough */
        {
            /* Search the entire heap */
            pMBL = (PMBL)pMAB->pbyHeapStart;
            /* Until we reach the end */
            while ((uint8_t *)pMBL < pMAB->pbyHeap)
            {
               /* Check for a free block with enough space to allocate from */
                if ((pMBL->pvTaskHandle == MAB_INVALID_TASK_HANDLE)
                &&  ((((uint8_t *)pMBL->pNext - (uint8_t *)pMBL) - sizeof(UMBL))
                >=  stLength))
                {
                    /* Calculate the allocation address */
                    pvReturn = (void*)((uint8_t *)pMBL + sizeof(UMBL));
                    /* Set the ID so the block is in use */
                    pMBL->pvTaskHandle = pvTaskHandle;
                    /* Insert a header for any free space at the end */
                    mabInsertMBL(pMBL, (PMBL)((uint8_t *)pvReturn + stLength));
                    break;
                }
                /* Advance to the next block */
                pMBL = (PMBL)pMBL->pNext;
            }
        }
        /* Update Running total of allocated memory & peak allocation */
        if (pvReturn)
        {
            pMAB->uiAllocated += stLength;
            pMAB->uiPeakAllocated = (pMAB->uiAllocated > pMAB->uiPeakAllocated)
                                  ? pMAB->uiAllocated
                                  : pMAB->uiPeakAllocated;
        }
    }
    /* This assert is a pain because the lwIP storage allocator tries to get
       memory from a small (and fast) heap first, when it fails it then tries
       the big (and slow) heap. The result is lots of memory allocation failure
       asserts */
#ifdef _MALLOC_CHECK_
    ASSERT(pvReturn);
#endif
    return pvReturn;
}
/******************************************************************************
End of function  mabMalloc
******************************************************************************/

/******************************************************************************
Function Name: mabFree
Description:   Function to free a block of memory
Arguments:     IN  pMAB - Pointer to the memory allocation block
               IN  pvFree - Pointer to the start of a memory block to free
Return value:  none
******************************************************************************/
void mabFree(PMAB pMAB, void *pvFree)
{
    if ((pMAB)
    &&  (pvFree))
    {
        PMBL   pPrev;
        PMBL   pSearch = (PMBL)pMAB->pbyHeapStart;
        pPrev = (PMBL)pMAB->pbyHeapStart;
        /* Search the entire heap in use */
        while ((uint8_t *)pSearch < pMAB->pbyHeap)
        {
            /* check for pointer match */
            if (pvFree == ((uint8_t *)pSearch + sizeof(UMBL)))
            {
                /* Update the allocation log */
                pMAB->uiAllocated -= (((uint8_t *)pSearch->pNext
                                   - (uint8_t *)pSearch)- sizeof(UMBL));
                /* If the block is in use - we don't care if the ID matches */
                if (pSearch->pvTaskHandle != MAB_INVALID_TASK_HANDLE)
                {
                    /* Check the end of the allocated heap if there is no next 
                       block then set the next to the current */
                    PMBL pNext = ((uint8_t *)pSearch->pNext < pMAB->pbyHeap)
                               ? (PMBL)pSearch->pNext : pSearch;
                    /* If the previous block is not in use */
                    if (pPrev->pvTaskHandle == MAB_INVALID_TASK_HANDLE)
                    {
                        /* Join all three if the next one is  in use othewize
                           just join the previous and the current */
                        pPrev->pNext = (pNext->pvTaskHandle != MAB_INVALID_TASK_HANDLE)
                                     ? pSearch->pNext : pNext->pNext;
                        /* The current pointer is now the joined to the
                           previous block */
                        pSearch = pPrev;
                    }
                    else
                    {
                        /* If the next one is not in use then */
                        if (pNext->pvTaskHandle == MAB_INVALID_TASK_HANDLE)
                        {
                            /* Joint the next one to the current */
                            pSearch->pNext = pNext->pNext;
                        }
                        /* Mark as unused */
                        pSearch->pvTaskHandle = MAB_INVALID_TASK_HANDLE;
                    }
                    /* If we have joined the last block on the heap then */
                    if ((uint8_t *)pSearch->pNext == pMAB->pbyHeap)
                    {
                        /* Update the heap end pointer */
                        pMAB->pbyHeap = (uint8_t *)pSearch;
                    }
                }
                /* Block was made free */
                break;
            }
            /* Move to the next one on the list */
            pPrev = pSearch;
            pSearch = pSearch->pNext;
        }
    }
}
/******************************************************************************
End of function  mabFree
******************************************************************************/

/*****************************************************************************
Function Name: mabFreeByTaskID
Description:   Function to free memory by the ID of the task that allocated it
Arguments:     IN  pMAB - Pointer to the memory allocation block
               IN  pvTaskHandle - The task ID
Return value:  The quantity of memory freed
*****************************************************************************/
size_t mabFreeByTaskID(PMAB pMAB, void *pvTaskHandle)
{
    size_t stFree = 0;
    if ((pMAB)
    &&  (pvTaskHandle != MAB_INVALID_TASK_HANDLE))
    {
        PMBL   pPrev = (PMBL)pMAB->pbyHeapStart;
        PMBL   pSearch = (PMBL)pMAB->pbyHeapStart;
        /* Search the entire heap in use */
        while ((uint8_t *)pSearch < pMAB->pbyHeap)
        {
            /* If the block is marked with the ID */
            if (pSearch->pvTaskHandle == pvTaskHandle)
            {
                PMBL pNext;
                /* Add on the size of the memory to the total */
                stFree += (size_t)
                          (((uint8_t *)pSearch->pNext
                        - (uint8_t *)pSearch) - sizeof(UMBL));
                /* Check the end of the allocated heap  if there is no next
                   block then  set the next to the current */
                pNext = ((uint8_t *)pSearch->pNext < pMAB->pbyHeap)
                      ? (PMBL)pSearch->pNext : pSearch;
                /* If the previous block is not in use */
                if (pPrev->pvTaskHandle == MAB_INVALID_TASK_HANDLE)
                {
                    /* Join all three if the next one is  in use othewise just
                       join the previous and the current */
                    pPrev->pNext = (pNext->pvTaskHandle != MAB_INVALID_TASK_HANDLE)
                                 ? pSearch->pNext : pNext->pNext;
                    /* The current block is now the joined to the previous */
                    pSearch = pPrev;
                }
                else
                {
                    /* If the next one is not in use then */
                    if (pNext->pvTaskHandle == MAB_INVALID_TASK_HANDLE)
                    {
                        /* Joint the next one to the current */
                        pSearch->pNext = pNext->pNext;
                    }
                    /* Mark as unused */
                    pSearch->pvTaskHandle = MAB_INVALID_TASK_HANDLE;
                }
                /* If we have joined the last block on the heap then */
                if ((uint8_t *)pSearch->pNext == pMAB->pbyHeap)
                {
                    /* Update the heap end pointer */
                    pMAB->pbyHeap = (uint8_t *)pSearch;
                }
            }
            /* Move to the next one on the list */
            pPrev = pSearch;
            pSearch = pSearch->pNext;
        }
    }
    /* Update the allocated log */
    pMAB->uiAllocated -= stFree;
    return stFree;
}
/*****************************************************************************
End of function  mabFreeByTaskID
******************************************************************************/

/******************************************************************************
Function Name: mabRealloc
Description:   Function to change the size of the allocated memory
Arguments:     IN  pMAB - Pointer to the memory allocation block
               IN  pvReAlloc - Pointer to the memory to re-allocate
               IN  stLength - The new length of the block
Return value:  Pointer to the re-allocated memory or NULL if not enough free
******************************************************************************/
void *mabRealloc(PMAB pMAB, void *pvReAlloc, size_t stLength)
{
    if ((pMAB) && (pvReAlloc) && (stLength))
    {
        PMBL    pSearch = (PMBL)pMAB->pbyHeapStart;
        /* Align to the desired number of bytes */
        stLength += (sizeof(UMBL) - 1);
        stLength &= ~(sizeof(UMBL) - 1);
        /* Search the heap */
        while ((uint8_t *)pSearch < pMAB->pbyHeap)
        {
            /* Search for the entry */
            if ((uint8_t *)pvReAlloc == ((uint8_t *)pSearch + sizeof(UMBL)))
            {
                /* Set the current pointer */
                PMBL   pCurr = (PMBL)pSearch;
                /* Set a pointer to the next block */
                PMBL   pNext = (PMBL)pCurr->pNext;
                /* Check to see if the block is in use */
                if (pCurr->pvTaskHandle != MAB_INVALID_TASK_HANDLE)
                {
                    size_t  stAllocated = (size_t)(((uint8_t *)pCurr->pNext
                                        - (uint8_t *)pvReAlloc) - sizeof(UMBL));
                    /* Test to see if it is being made smaller */
                    if ((size_t)((uint8_t *)pCurr->pNext 
                    -            (uint8_t *)pvReAlloc) > stLength)
                    {
                        {
                            size_t  stFreed = stAllocated - stLength;
                            /* Update the allocation log */
                            pMAB->uiAllocated -= stFreed;
                        }
                        /* It is getting smaller so if it is the last block */
                        if ((uint8_t *)pCurr->pNext == pMAB->pbyHeap)
                        {
                            /* Update the heap pointer */
                            pMAB->pbyHeap = ((uint8_t *)pvReAlloc + stLength);
                            /* Update the next pointer */
                            pCurr->pNext = (PMBL)pMAB->pbyHeap;
                            
                        }
                        /* If the next block is not in use */
                        else if (pNext->pvTaskHandle == MAB_INVALID_TASK_HANDLE)
                        {
                            /* Set the next pointer to the new length */
                            pCurr->pNext = (PMBL)((uint8_t *)pvReAlloc + stLength);
                            /* Adjust the next pointer to reflect the size
                               reduction */
                            pCurr = pCurr->pNext;
                            pCurr->pvTaskHandle = MAB_INVALID_TASK_HANDLE;
                            pCurr->pNext = pNext->pNext;
                        }
                        else
                        {
                            /* Insert a new block */
                            mabInsertMBL(pCurr,
                                         (PMBL)((uint8_t *)pvReAlloc + stLength));
                        }
                    }
                    /* Test to see if it is being made bigger */
                    else if ((size_t)((uint8_t *)pCurr->pNext
                         -            (uint8_t *)pvReAlloc) 
                         <   stLength)
                    {
                        uint8_t *   pbyNew;
                        /* If the last one on the heap */
                        if ((uint8_t *)pCurr->pNext == pMAB->pbyHeap)
                        {
                            /* Check to see if it will fit */
                            if (((uint8_t *)pvReAlloc + stLength) 
                            <=  pMAB->pbyHeapEnd)
                            {
                                {
                                    /* Update the allocation log */
                                    pMAB->uiAllocated -= stAllocated;
                                    pMAB->uiAllocated += stLength;
                                    pMAB->uiPeakAllocated = (pMAB->uiAllocated
                                         > pMAB->uiPeakAllocated)
                                         ? pMAB->uiAllocated
                                         : pMAB->uiPeakAllocated;
                                }
                                /* Update the heap end */
                                pMAB->pbyHeap = (uint8_t *)((uint8_t*)pvReAlloc
                                              + stLength);
                                /* Set the new end pointer */
                                pCurr->pNext = (PMBL)pMAB->pbyHeap;
                            }
                            else
                            {
                                /* Can't make it any bigger! */
                                return NULL;
                            }
                        }
                        /* Test to see if the next one is free and together
                           satisfy the requirement */
                        else if ((pNext->pvTaskHandle == MAB_INVALID_TASK_HANDLE)
                             && ((size_t)((uint8_t *)pNext->pNext
                             -   (uint8_t *)pvReAlloc)
                             >   stLength))
                        {
                            {
                                /* Update the allocation log */
                                pMAB->uiAllocated -= stAllocated;
                                pMAB->uiAllocated += stLength;
                                pMAB->uiPeakAllocated = (pMAB->uiAllocated
                                     > pMAB->uiPeakAllocated)
                                     ? pMAB->uiAllocated
                                     : pMAB->uiPeakAllocated;
                            }
                            /* Join the blocks together */
                            pCurr->pNext = pNext->pNext;
                            /* Insert a new block if required */
                            mabInsertMBL(pCurr,
                                         (PMBL)((uint8_t *)pvReAlloc + stLength));
                        }
                        /* Last resort is to allocate some more memory */
                        else if ((pbyNew = (uint8_t *)mabMalloc(pMAB,
                                                                xTaskGetCurrentTaskHandle(),
                                                                stLength)))
                        {
                            /* Copy the data across */
                            memcpy(pbyNew,
                                   pvReAlloc,
                                   ((size_t)pCurr->pNext - (size_t)pvReAlloc));
                            /* Free the old block */
                            mabFree(pMAB, pvReAlloc);
                            /* Assign the new address */
                            pvReAlloc = pbyNew;
                        }
                        else
                        {
                            return NULL;
                        }
                    }
                }
                else
                {
                    return NULL;
                }
                return pvReAlloc;
            }
            /* Advance to next item on the list */
            pSearch = pSearch->pNext;
        }
    }
    return NULL;
}
/******************************************************************************
End of function  mabRealloc
******************************************************************************/

/*****************************************************************************
Private Functions
******************************************************************************/

/******************************************************************************
Function Name: mabInsertMBL
Description:   Function to insert a Memory Block List structure
Arguments:     IN  pMBL - Pointer to the header insertion point
               IN  pNewMBL - Pointer to the location of the pbyNew header
Return value:  none
******************************************************************************/
static void mabInsertMBL(PMBL pMBL, PMBL pNewMBL)
{
    /* If there is enough room for the header */
    if (((size_t)pMBL->pNext - (size_t)pNewMBL) > sizeof(UMBL))
    {
        /* Keep a copy of the next block */
        PMBL    pNext = pMBL->pNext;
        /* Insert the new header at this location */
        pMBL->pNext = pNewMBL;
        /* Initialise the task ID */
        pNewMBL->pvTaskHandle = MAB_INVALID_TASK_HANDLE;
        /* Set next pointer of the new header */
        pNewMBL->pNext = pNext;
    }
}
/******************************************************************************
End of function  mabInsertMBL
******************************************************************************/

/******************************************************************************
End  Of File
******************************************************************************/
