/******************************************************************************
 * DISCLAIMER
 * This software is supplied by Renesas Electronics Corporation and is only
 * intended for use with Renesas products. No other uses are authorized. This
 * software is owned by Renesas Electronics Corporation and is protected under
 * all applicable laws, including copyright laws.
 * THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
 * THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
 * LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
 * AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
 * TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
 * ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
 * FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
 * ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
 * BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 * Renesas reserves the right, without notice, to make changes to this software
 * and to discontinue the availability of this software. By using this software,
 * you agree to the additional terms and conditions found by accessing the
 * following link:
 * http://www.renesas.com/disclaimer
 *******************************************************************************
 * Copyright (C) 2011 Renesas Electronics Corporation. All rights reserved.
 *******************************************************************************
 * File Name    : malloc.c
 * Version      : 1.02
 * Description  : Replacement for the standard C run time library memory
 management functions to make them "task" and multiple memory
 type aware
 *******************************************************************************
 * History      : DD.MM.YYYY Ver. Description
 *              : 01.08.2009 1.00 MAB First Release
 *              : 14.12.2010 1.01 Configured internal Heap
 *              : 18.03.2011 1.02 Added paek allocated settings
 ******************************************************************************/

/******************************************************************************
 WARNING!  IN ACCORDANCE WITH THE USER LICENCE THIS CODE MUST NOT BE CONVEYED
 OR REDISTRIBUTED IN COMBINATION WITH ANY SOFTWARE LICENSED UNDER TERMS THE
 SAME AS OR SIMILAR TO THE GNU GENERAL PUBLIC LICENCE
 ******************************************************************************/

/*****************************************************************************
 System Includes
 ******************************************************************************/

#include <string.h>

/*****************************************************************************
 User Includes
 ******************************************************************************/

/* Ensure local malloc is used */
#include "..\inc\malloc.h"

#include "mabMalloc.h"
#include "trace.h"
#include "hwArmCache.h"

/******************************************************************************
 Function Macros
 ******************************************************************************/

/* Comment this line out to turn ON module trace in this file */
#undef _TRACE_ON_

#ifndef _TRACE_ON_
#undef TRACE
#define TRACE(x)
#endif

/******************************************************************************
 Macro definitions
 ******************************************************************************/

#define MALLOC_SDRAM_START          (&_sdram_heap_start)
#define MALLOC_SDRAM_END            (&_sdram_heap_end)
#define MALLOC_SRAM_START           (&_sram_heap_start)
#define MALLOC_SRAM_END             (&_sram_heap_end)


#define WEB_SERVER_HEAP_SIZE        (1024U * 256U)
#define LWIP_PRIMARY_HEAP_SIZE      (1024U * 512U)
#define RTOS_STACK_SIZE             (1024U * 128U)

#define HEAP_RTOS_STACK_START       (MALLOC_SRAM_START)
#define HEAP_RTOS_STACK_END         (HEAP_RTOS_STACK_START + RTOS_STACK_SIZE)
#define HEAP_WEB_SERVER_START       (HEAP_RTOS_STACK_END)
#define HEAP_WEB_SERVER_END         (HEAP_WEB_SERVER_START + WEB_SERVER_HEAP_SIZE)
#define HEAP_LWIP_PRIMARY_START     (HEAP_WEB_SERVER_END)
#define HEAP_LWIP_PRIMARY_END       (HEAP_LWIP_PRIMARY_START + LWIP_PRIMARY_HEAP_SIZE)

/*****************************************************************************
 Imported global variables and functions (from other files)
 ******************************************************************************/

/* Addresses provide by the linker */
extern uint8_t _sdram_heap_start;
extern uint8_t _sdram_heap_end;
extern uint8_t _sram_heap_start;
extern uint8_t _sram_heap_end;

/*****************************************************************************
 Global Variables
 ******************************************************************************/

/* The heap management object - Have as many heaps as you like!! */
static MAB gp_heap[NUMBER_OF_HEAPS] =
{
    {   /* HEAP_RTOS_STACK - In internal SRAM */
        MALLOC_SRAM_START,
        MALLOC_SRAM_START,
        HEAP_RTOS_STACK_END,
        0UL,
        0UL
    },

    {   /* HEAP_WEB_SERVER - In internal SRAM */
        HEAP_WEB_SERVER_START,
        HEAP_WEB_SERVER_START,
        HEAP_WEB_SERVER_END,
        0UL,
        0UL
    },

    {   /* HEAP_LWIP_PRIMARY - In internal SRAM */
           HEAP_LWIP_PRIMARY_START,
           HEAP_LWIP_PRIMARY_START,
           HEAP_LWIP_PRIMARY_END,
        0UL,
        0UL
    },

    {   /* HEAP_SRAM remaining SRAM - In internal SRAM */
        HEAP_LWIP_PRIMARY_END,
        HEAP_LWIP_PRIMARY_END,
        MALLOC_SRAM_END,
        0UL,
        0UL
    },

    {   /* HEAP_SDRAM - In external SDRAM */
        MALLOC_SDRAM_START,
        MALLOC_SDRAM_START,
        MALLOC_SDRAM_END,
        0UL,
        0UL
    }
};

/*****************************************************************************
 Function Prototypes
 ******************************************************************************/

static void mabInformation (PMABINFO pMabInfo, PMAB pMAB);

/*****************************************************************************
 Public Functions
 ******************************************************************************/

/*****************************************************************************
 Function Name: exMalloc
 Description:   Function to allocate memory from memory of different attributes
 Arguments:     IN  memType - The heap attribute or memory type
 IN  stLength - The length to allocate
 Return value:  Pointer to the memory or NULL if not enough free
 *****************************************************************************/
void *exMalloc (MEMTYPE memType, size_t stLength)
{
    void * pvReturn = NULL;
    if ((memType < NUMBER_OF_HEAPS) && (stLength))
    {
        void *pvTaskHandle;
        vTaskSuspendAll();
        pvTaskHandle = xTaskGetCurrentTaskHandle();
        pvReturn = mabMalloc(&gp_heap[memType], pvTaskHandle, stLength);
        if ((!pvReturn)

        /* lwIP has a pool of fast memory. When it is used it all it then
           uses the slower SDRAM so don't count this as a fatal error */
        && (memType != HEAP_LWIP_PRIMARY))
        {
            vApplicationMallocFailedHook();
        }

        /* The first 8 bytes have the pointer to the next memory address
         * and the FreeRTOS task handle. Make sure that this is written
         * to physical memory in case the application invalidates the
         * cache */
        if (pvReturn)
        {
            cacheClear(NULL, NULL, pvReturn, sizeof(UMBL));
        }
        xTaskResumeAll();
    }
    return pvReturn;
}
/*****************************************************************************
 End of function  exMalloc
 ******************************************************************************/

/*****************************************************************************
 Function Name: exCalloc
 Description:   Function to free memory from memory of different attributes
 Arguments:     IN  memType - The heap attribute or memory type
 IN  stNumber - The number of objects
 IN  stLength - The length to allocate
 Return value:  Pointer to the memory or NULL if not enough free
 *****************************************************************************/
void *exCalloc (MEMTYPE memType, size_t stNumber, size_t stLength)
{
    size_t stSize = (size_t) (stNumber * stLength);
    void * pvReturn = exMalloc(memType, stSize);
    if (pvReturn)
    {
        memset(pvReturn, 0, stSize);
    }
    return pvReturn;
}
/*****************************************************************************
 End of function  exCalloc
 ******************************************************************************/

/*****************************************************************************
 Function Name: exRealloc
 Description:   Function to reallocate memory from memory of different attributes
 Arguments:     IN  memType - The heap attribute or memory type
 IN  pvReAlloc - Pointer to the memory to block to change the size of
 IN  stLength - The new length of the block
 Return value:  Pointer to the re-allocated memory or NULL if not enough free
 *****************************************************************************/
void *exRealloc (MEMTYPE memType, void *pvReAlloc, size_t stLength)
{
    void * pvReturn = NULL;
    if ((memType < NUMBER_OF_HEAPS) && (stLength))
    {
        vTaskSuspendAll();
        pvReturn = mabRealloc(&gp_heap[memType], pvReAlloc, stLength);

        /* The first 8 bytes have the pointer to the next memory address
         * and the FreeRTOS task handle. Make sure that this is written
         * to physical memory in case the application invalidates the
         * cache */
        if (pvReturn)
        {
            cacheClear(NULL, NULL, pvReturn, sizeof(UMBL));
        }
        xTaskResumeAll();
    }
    return pvReturn;
}
/*****************************************************************************
 End of function  exRealloc
 ******************************************************************************/

/*****************************************************************************
 Function Name: exFree
 Description:   Function to free memory from memory of different attributes
 Arguments:     IN  memType - The heap attribute or memory type
 IN  pvFree - Pointer to the memory to free
 Return value:  none
 *****************************************************************************/
void exFree (MEMTYPE memType, void *pvFree)
{
    if (memType < NUMBER_OF_HEAPS)
    {
        vTaskSuspendAll();
        mabFree(&gp_heap[memType], pvFree);
        xTaskResumeAll();
    }
}
/*****************************************************************************
 End of function  exFree
 ******************************************************************************/

/*****************************************************************************
 Function Name: exFreeByTaskID
 Description:   Function to free the memory by TaskID
 Arguments:     IN  pvTaskHandle - The ID of the task to free
 Return value:  The quantity of memory freed
 *****************************************************************************/
size_t exFreeByTaskID (void *pvTaskHandle)
{
    size_t stReturn = 0;
    int iHeap = NUMBER_OF_HEAPS;
    while (iHeap--)
    {
        stReturn += mabFreeByTaskID(&gp_heap[iHeap], pvTaskHandle);
    }
    return stReturn;
}
/*****************************************************************************
 End of function  exFreeByTaskID
 ******************************************************************************/

/*****************************************************************************
 Function Name: malloc
 Description:   Replacement of the C run time library function
 Arguments:     IN  stLength - The length to allocate
 Return value:  Pointer to the memory or NULL if not enough free
 *****************************************************************************/
void *malloc (size_t stLength)
{
    return exMalloc(MALLOC_MEMORY_TYPE, stLength);
}
/*****************************************************************************
 End of function  malloc
 ******************************************************************************/

/*****************************************************************************
 Function Name: calloc
 Description:   Replacement of the C run time library function
 Arguments:     IN  stNumber - The number of objects
 IN  stLength - The length to allocate
 Return value:  Pointer to the memory or NULL if not enough free
 *****************************************************************************/
void *calloc (size_t stNumber, size_t stLength)
{
    return exCalloc(MALLOC_MEMORY_TYPE, stNumber, stLength);
}
/*****************************************************************************
 End of function  calloc
 ******************************************************************************/

/*****************************************************************************
 Function Name: realloc
 Description:   Replacement of the C run time library function
 Arguments:     IN  pvReAlloc - Pointer to the memory to re-allocate
 IN  stLength - The new length of the memory
 Return value:  Pointer to the re-allocated memory or NULL if not enough free
 *****************************************************************************/
void *realloc (void *pvReAlloc, size_t stLength)
{
    return exRealloc(MALLOC_MEMORY_TYPE, pvReAlloc, stLength);
}
/*****************************************************************************
 End of function  realloc
 ******************************************************************************/

/*****************************************************************************
 Function Name: free
 Description:   Replacement of the C run time library function
 Arguments:     IN  pvFree - Pointer to the memory to free
 Return value:  none
 *****************************************************************************/
void free (void *pvFree)
{
    exFree(MALLOC_MEMORY_TYPE, pvFree);
}
/*****************************************************************************
 End of function  free
 ******************************************************************************/

/*****************************************************************************
 Function Name: heapUsage
 Description:   Function to get the heap usage information
 Arguments:     OUT pUsage - Pointer to the destination usage information
 IN  memType - The heap attribute or memory type
 Return value:  none
 *****************************************************************************/
void heapUsage (PHEAPUSAGE pUsage, MEMTYPE memType)
{
    /* Initialise the structure */
    memset(pUsage, 0, sizeof(HEAPUSAGE));
    if (memType < NUMBER_OF_HEAPS)
    {
        MABINFO mabInfo;
        vTaskSuspendAll();

        /* Get the information */
        mabInformation(&mabInfo, &gp_heap[memType]);

        /* Set the details */
        pUsage->stTotalHeap = mabInfo.stSize;
        pUsage->stTotalFree = mabInfo.stTotalFree;
        pUsage->stTotalAllocated = mabInfo.stTotalAllocated;
        pUsage->stOverhead = mabInfo.stOverhead;
        pUsage->stLargestFreeSegment = mabInfo.stLargestFreeSegment;
        pUsage->stPeakAllocated = mabInfo.stPeakAllocated;
        xTaskResumeAll();
    }
}
/*****************************************************************************
 End of function  heapUsage
 ******************************************************************************/

/*****************************************************************************
 Function Name: heapSearchObject
 Description:   Function to search the heap for an object. This is because lwIP
 leaks memory and a special function is required to debug it
 Arguments:     IN  memType - The heap attribute or memory type
 IN  stSize - The size of the object to look for
 IN/OUT pstOffset - The pointer to offset into the heap
 Return value:  Pointer to the first object on the heap after the offset
 argument of the described size or NULL
 *****************************************************************************/
void *heapSearchObject (MEMTYPE memType, size_t stSize, size_t *pstOffset)
{
    stSize += sizeof(size_t);
    if ((memType < NUMBER_OF_HEAPS) && (pstOffset))
    {
        /* Get a pointer to the heap */
        PMAB pMAB = &gp_heap[memType];

        /* Get a pointer to the heap memory allocation block list */
        PMBL pMBL = (PMBL) pMAB->pbyHeapStart;

        /* Search the entire heap */
        while ((uint8_t *) pMBL < pMAB->pbyHeap)
        {
            /* Check for search start offset */
            if ((((uint8_t *) pMBL) - pMAB->pbyHeapStart) >= *pstOffset)
            {
                /* Check to see if the allocation is in use */
                if (MAB_INVALID_TASK_HANDLE != pMBL->pvTaskHandle)
                {
                    /* Calculate the size of this entry */
                    size_t stLength;
                    if ((size_t)((uint8_t *)pMBL->pNext -
                                 (uint8_t *)pMBL) > sizeof(UMBL))
                    {
                        stLength = (size_t)
                                   (((uint8_t *)pMBL->pNext
                                 - (uint8_t *)pMBL) - sizeof(UMBL));
                        /* Check for a size match */
                        if (stLength == stSize)
                        {
                            /* Update the offset */
                            *pstOffset = (((uint8_t *)pMBL->pNext)
                                       - pMAB->pbyHeapStart);
                            /* Return the pointer to the ojbect */
                            return (void*) ((uint8_t *) pMBL + sizeof(UMBL));
                        }
                    }
                }
            }

            /* Advance to the next block */
            pMBL = (PMBL) pMBL->pNext;
        }
    }
    return NULL;
}
/*****************************************************************************
 End of function  heapSearchObject
 ******************************************************************************/

/*****************************************************************************
 Private Functions
 ******************************************************************************/

/*****************************************************************************
 Function Name: mabInformation
 Description:   Function to scan a memory allocation block and return the
 usage information
 Arguments:     OUT pMabInfo - Pointer to the information structure to fill
 IN  pMAB - Pointer to the memory allocation block to scan
 Return value:  none
 *****************************************************************************/
static void mabInformation (PMABINFO pMabInfo, PMAB pMAB)
{
    PMBL pMBL;

    /* Initialise the structure */
    memset(pMabInfo, 0, sizeof(MABINFO));

    /* Set the total heap size */
    pMabInfo->stSize = (size_t) (pMAB->pbyHeapEnd - pMAB->pbyHeapStart);

    /* Initialise total free */
    pMabInfo->stTotalFree = (size_t) (pMAB->pbyHeapEnd - pMAB->pbyHeap);

    /* Start this as the biggest free segment */
    if (pMabInfo->stTotalFree > sizeof(UMBL))
    {
        pMabInfo->stLargestFreeSegment = pMabInfo->stTotalFree - sizeof(UMBL);
    }

    /* Add on the overhead of this MAB */
    pMabInfo->stOverhead = sizeof(MAB);

    /* Search the entire allocated heap */
    pMBL = (PMBL) pMAB->pbyHeapStart;

    /* Until we reach the end */
    while ((uint8_t *) pMBL < pMAB->pbyHeap)
    {
        /* Calculate the size of this entry */
        size_t stLength;
        if ((size_t) ((uint8_t *) pMBL->pNext - (uint8_t *) pMBL) > sizeof(UMBL))
        {
            stLength = (size_t)
                       (((uint8_t *)pMBL->pNext - (uint8_t *)pMBL) - sizeof(UMBL));
        }
        else
        {
            stLength = 0;
        }

        /* Check to see if the allocation is in use */
        if (MAB_INVALID_TASK_HANDLE == pMBL->pvTaskHandle)
        {
            /* Add on to the total free */
            pMabInfo->stTotalFree += stLength;

            /* Check to update the biggest free segment */
            if (pMabInfo->stLargestFreeSegment < stLength)
            {
                pMabInfo->stLargestFreeSegment = stLength;
            }
        }
        else
        {
            /* Add on the total allocated */
            pMabInfo->stTotalAllocated += stLength;

            /* Check to update the largest allocation */
            if (pMabInfo->stLargestAllocation < stLength)
            {
                pMabInfo->stLargestAllocation = stLength;
            }
        }
        TRACE(("0x%p L%lu Task %d\r\n", ((uint8_t *)pMBL + sizeof(UMBL)),
                                        stLength,
                                        pMBL->pvTaskHandle));

        /* Advance to the next block */
        pMBL = (PMBL) pMBL->pNext;

        /* Sum the overhead */
        pMabInfo->stOverhead += sizeof(UMBL);
    }

    /* Reset the Total allocation for this MAB */
    pMAB->uiAllocated = pMabInfo->stTotalAllocated;

    /* Update the Peak allocation for this MAB */
    pMabInfo->stPeakAllocated = pMAB->uiPeakAllocated;

    /* Reset the Peak allocation for this MAB */
    pMAB->uiPeakAllocated = pMAB->uiAllocated;
}
/*****************************************************************************
 End of function  mabInformation
 ******************************************************************************/

/******************************************************************************
 End  Of File
 ******************************************************************************/
