/******************************************************************************
* DISCLAIMER

* This software is supplied by Renesas Electronics Corporation and is only 
* intended for use with Renesas products. No other uses are authorized.

* This software is owned by Renesas Electronics Corporation and is protected under 
* all applicable laws, including copyright laws.

* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES 
* REGARDING THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, 
* INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A 
* PARTICULAR PURPOSE AND NON-INFRINGEMENT.  ALL SUCH WARRANTIES ARE EXPRESSLY 
* DISCLAIMED.

* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS 
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE 
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES 
* FOR ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS 
* AFFILIATES HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.

* Renesas reserves the right, without notice, to make changes to this 
* software and to discontinue the availability of this software.  
* By using this software, you agree to the additional terms and 
* conditions found by accessing the following link:
* http://www.renesas.com/disclaimer
******************************************************************************/
/* Copyright (C) 2011 Renesas Electronics Corporation. All rights reserved.  */
/******************************************************************************
* File Name    : drvRIIC.c
* Version      : 1.00
* Device       : RZA1H
* Tool-Chain   : GNU
* H/W Platform : RSK2+RZA1H
* Description  : Device driver for RIIC in master operation
******************************************************************************
* History : DD.MM.YYYY Version Description
*         : dd.mm.yyyy 1.00    First Release
******************************************************************************/

/******************************************************************************
Includes   <System Includes> , "Project Includes"
******************************************************************************/
#include <stdlib.h>
#include "drvRIIC.h"
#include "r_task.h"
#include "Trace.h"

/******************************************************************************
Defines
******************************************************************************/

#define RIIC_TIME_OUT_MS                (600)
#define RIIC_TRANSFER_TIME(l)           (RIIC_TIME_OUT_MS + l)

/******************************************************************************
Private Function Prototypes
******************************************************************************/
static void riic_init(PRIICDRV pRIICD);

/******************************************************************************
Exported global variables and functions (to be accessed by other files)
******************************************************************************/

/******************************************************************************
* Function Name: r_drvRIIC_Open
* Description  : Function to open the RIIC interface
* Arguments    : OUT pRIICD - Pointer to the RIIC driver data
*                IN  clockSelection - The clock selection
*                IN  ulBitRateLow - The low clock cycle value
*                IN  ulBitRateHigh - The high clock cycle value
* Return Value : 0 for success or -1 on error
******************************************************************************/
int32_t r_drvRIIC_Open(PRIICDRV    pRIICD,
                       volatile struct st_riic_n *pRIIC,
                       RIICCKS     clockSelection,
                       uint32_t    ulBitRateLow,
                       uint32_t    ulBitRateHigh)
{
    memset(pRIICD, 0, sizeof(RIICDRV));

    /* Initialise the variables */
    pRIICD->pRIIC = pRIIC;
    pRIICD->clockSelection = clockSelection;
    pRIICD->ulBitRateLow = ulBitRateLow;
    pRIICD->ulBitRateHigh = ulBitRateHigh;

    /* Create the signals */
    pRIICD->itemp = eventCreate(pRIICD->ppevSignals, RIIC_NUM_SIGNALS);
    if (pRIICD->itemp)
    {
        eventDestroy(pRIICD->ppevSignals, RIIC_NUM_SIGNALS - pRIICD->itemp);
        memset(pRIICD, 0, sizeof(RIICDRV));
        return -1;
    }
    riic_init(pRIICD);
    return 0;
}
/******************************************************************************
End of function  r_drvRIIC_Open
******************************************************************************/

/******************************************************************************
* Function Name: r_drvRIIC_Close
* Description  : Function to close the RIIC interface
* Arguments    : IN  pRIICD - Pointer to the RIIC driver data
* Return Value : none
******************************************************************************/
void r_drvRIIC_Close(PRIICDRV pRIICD)
{
    eventDestroy(pRIICD->ppevSignals, RIIC_NUM_SIGNALS);
    riic_init(pRIICD);
    memset(pRIICD, 0, sizeof(RIICDRV));
}
/******************************************************************************
End of function  r_drvRIIC_Close
******************************************************************************/

/******************************************************************************
* Function Name: r_drvRIIC_TxStart
* Description  : Function to transmit the start condition
* Arguments    : IN  pRIICD - Pointer to the RIIC driver data
* Return Value : 0 for success or -1 on error
******************************************************************************/
int32_t r_drvRIIC_TxStart(PRIICDRV pRIICD)
{
    pRIICD->itemp = -1;

    /* Initialise the interface */
    riic_init(pRIICD);

    /* Check that the bus is in the correct state */
    if (!pRIICD->pRIIC->RIICnICCR2.BIT.BBSY)
    {
        /* Reset the events */
        eventReset(pRIICD->ppevSignals[RIIC_START]);

        /* Enable the start and time-out interrupts */
        pRIICD->pRIIC->RIICnICIER.BIT.STIE = 1;
        pRIICD->pRIIC->RIICnICIER.BIT.TMOIE = 1;

        /* Wait for the start condition to be issued or a time-out */
        pRIICD->ppSignals[0] = pRIICD->ppevSignals[RIIC_START];
        pRIICD->ppSignals[1] = timerStartEvent(SINGLE_SHOT_TIMER, RIIC_TIME_OUT_MS);

        /* Set the start condition */
        pRIICD->pRIIC->RIICnICCR2.BIT.ST = 1;

        /* Wait for the start to complete */
        if (0 == eventWait(pRIICD->ppSignals, 2, true))
        {
            /* Check the NACK flag for successful start */
            if (!pRIICD->pRIIC->RIICnICSR2.BIT.NACKF)
            {
                pRIICD->itemp = 0;
            }
        }
        timerStopEvent(pRIICD->ppSignals[1]);
    }
    else
    {
        TRACE(("r_drvRIIC_TxStart: bus busy\r\n"));
    }
    pRIICD->pRIIC->RIICnICIER.BIT.STIE = 0;
    pRIICD->pRIIC->RIICnICIER.BIT.TMOIE = 0;
    return pRIICD->itemp;
}
/******************************************************************************
End of function  r_drvRIIC_TxStart
******************************************************************************/

/******************************************************************************
* Function Name: int32_t r_drvRIIC_TxData
* Description  : Function to transmit data
* Arguments    : IN  pRIICD - Pointer to the RIIC driver data
*                IN  pvData - Pointer to the data to transmit
*                IN  stLength - The length of data to transmit
* Return Value : 0 for success or -1 on error
******************************************************************************/
int32_t r_drvRIIC_TxData(PRIICDRV pRIICD,
                         const void *pvData,
                         size_t stLength)
{
    pRIICD->itemp = -1;

    /* It should be busy as the start condition should have been sent */
    if ((pRIICD->pRIIC->RIICnICCR2.BIT.BBSY)
    &&  (stLength))
    {
        /* Reset the events */
        eventReset(pRIICD->ppevSignals[RIIC_TRANSMISSION_COMPLETE]);
        eventReset(pRIICD->ppevSignals[RIIC_RECEPTION_COMPLETE]);

        /* Take a copy of the data pointer & counter */
        pRIICD->pbyData = (uint8_t*)pvData;
        pRIICD->stCount = stLength;
        pRIICD->bfTransmit = true;

        /* Because the interrupt is only vectored when the TDRE is set
           the first byte must be written to the data register now */
        r_drvRIIC_TI(pRIICD);

        /* Enable the time-out and reception interrupts */
        pRIICD->pRIIC->RIICnICIER.BIT.TMOIE = 1;
        pRIICD->pRIIC->RIICnICIER.BIT.RIE = 1;
        pRIICD->ppSignals[0] = pRIICD->ppevSignals[RIIC_TRANSMISSION_COMPLETE];
        pRIICD->ppSignals[1] = pRIICD->ppevSignals[RIIC_RECEPTION_COMPLETE];
        pRIICD->ppSignals[2] = timerStartEvent(SINGLE_SHOT_TIMER, RIIC_TRANSFER_TIME(stLength));

        /* Wait for the write to complete */
        switch (eventWait(pRIICD->ppSignals, 3, true))
        {
            case 1:
                /* The peripheral has entered master receive mode
                   and is now ready to receive data.
                   Fall through is intended */
            case 0:
            {
                /* If all of the data has been transmitted */
                if (!pRIICD->stCount)
                {
                    pRIICD->itemp = 0;
                }
                break;
            }
            default:
                /* Time out */
                break;
        }
        timerStopEvent(pRIICD->ppSignals[2]);
        pRIICD->pRIIC->RIICnICIER.BIT.RIE = 0;
        pRIICD->pRIIC->RIICnICIER.BIT.TEIE = 0;
        pRIICD->pRIIC->RIICnICIER.BIT.TIE = 0;
        pRIICD->pRIIC->RIICnICIER.BIT.TMOIE = 0;
    }
    else
    {
        TRACE(("r_drvRIIC_TxData: bus busy\r\n"));
    }
    return pRIICD->itemp;
}
/******************************************************************************
End of function  int32_t r_drvRIIC_TxData
******************************************************************************/

/******************************************************************************
* Function Name: int32_t r_drvRIIC_RxData
* Description  : Function to receive data
* Arguments    : IN  pRIICD - Pointer to the RIIC driver data
*                IN  pvData - Pointer to the received data buffer
*                IN  stLength - The length of data to be received
* Return Value : 0 for success or -1 on error
******************************************************************************/
int32_t r_drvRIIC_RxData(PRIICDRV   pRIICD,
                         void       *pvData,
                         size_t     stLength)
{
    pRIICD->itemp = -1;

    /* It should be busy as the start condition should have been sent */
    if ((pRIICD->pRIIC->RIICnICCR2.BIT.BBSY)
    &&  (stLength))
    {
        /* Reset the events */
        eventReset(pRIICD->ppevSignals[RIIC_RECEPTION_COMPLETE]);

        /* Take a copy of the data pointer & counter */
        pRIICD->pbyData = (uint8_t*)pvData;
        pRIICD->stCount = stLength;
        pRIICD->bfTransmit = false;

        /* Enable the receive data full and time-out interrupts */
        pRIICD->pRIIC->RIICnICIER.BIT.RIE = 1;
        pRIICD->pRIIC->RIICnICIER.BIT.TMOIE = 1;

        pRIICD->ppSignals[0] = pRIICD->ppevSignals[RIIC_RECEPTION_COMPLETE];
        pRIICD->ppSignals[1] = timerStartEvent(SINGLE_SHOT_TIMER, RIIC_TRANSFER_TIME(stLength));

        /* If there is only one byte to send */
        if (1U == pRIICD->stCount)
        {
            /* Send NACK on last byte */
            pRIICD->pRIIC->RIICnICMR3.BIT.ACKBT = 1;
        }
        else
        {
            /* Send ACK on first byte */
            pRIICD->pRIIC->RIICnICMR3.BIT.ACKBT = 0;
        }
        /* Start reception with a dummy read */
        {
            volatile uint8_t byDummy
                = pRIICD->pRIIC->RIICnICDRR.BIT.ICDRR;
        }
        /* Wait for the write to complete */
        if (0 == eventWait(pRIICD->ppSignals, 2, true))
        {
            /* If all of the data has been received */
            if (!pRIICD->stCount)
            {
                pRIICD->itemp = 0;
            }
        }
        timerStopEvent(pRIICD->ppSignals[1]);
        pRIICD->pRIIC->RIICnICIER.BIT.RIE = 0;
        pRIICD->pRIIC->RIICnICIER.BIT.TMOIE = 0;
    }
    return pRIICD->itemp;
}
/******************************************************************************
End of function  int32_t r_drvRIIC_RxData
******************************************************************************/

/******************************************************************************
* Function Name: r_drvRIIC_TxStop
* Description  : Function to send the stop condition
* Arguments    : IN  pRIICD - Pointer to the RIIC driver data
* Return Value : 0 for success or -1 on error
******************************************************************************/
int32_t r_drvRIIC_TxStop(PRIICDRV pRIICD)
{
    pRIICD->itemp = -1;

    if (!pRIICD->pRIIC->RIICnICSR2.BIT.STOP)
    {
        /* Reset the events */
        eventReset(pRIICD->ppevSignals[RIIC_STOP]);

        /* Enable the stop and time-out interrupts */
        pRIICD->pRIIC->RIICnICIER.BIT.SPIE = 1;
        pRIICD->pRIIC->RIICnICIER.BIT.TMOIE = 1;

        pRIICD->ppSignals[0] = pRIICD->ppevSignals[RIIC_STOP];
        pRIICD->ppSignals[1] = timerStartEvent(SINGLE_SHOT_TIMER, RIIC_TIME_OUT_MS);

        /* Issue the stop condition */
        pRIICD->pRIIC->RIICnICCR2.BIT.SP = 1;

        /* Check for reception */
        if (false == pRIICD->bfTransmit)
        {
            volatile uint8_t byDummy = 0;

            UNUSED_VARIABLE(byDummy);

            byDummy = pRIICD->pRIIC->RIICnICDRR.BIT.ICDRR;
        }

        /* Wait for the stop to complete */
        if (0 == eventWait(pRIICD->ppSignals, 2, true))
        {
            pRIICD->itemp = 0;
        }
        timerStopEvent(pRIICD->ppSignals[1]);
        pRIICD->pRIIC->RIICnICIER.BIT.SPIE = 0;
        pRIICD->pRIIC->RIICnICIER.BIT.TMOIE = 0;
    }
    return pRIICD->itemp;
}
/******************************************************************************
End of function  r_drvRIIC_TxStop
******************************************************************************/

/******************************************************************************
* Function Name: r_drvRIIC_TI
* Description  : Interrupt handler for TI
* Arguments    : IN  pRIICD - Pointer to the RIIC driver data
* Return Value : none
******************************************************************************/
void r_drvRIIC_TI(PRIICDRV pRIICD)
{
    /* Disable this interrupt */
    pRIICD->pRIIC->RIICnICIER.BIT.TIE = 0;
    if (pRIICD->stCount)
    {
        if (pRIICD->pRIIC->RIICnICSR2.BIT.TDRE)
        {
            /* Once the data for transmission are written to RIICnDRT,
               the TDRE flag is automatically cleared to 0 */
            pRIICD->pRIIC->RIICnICDRT.BIT.ICDRS = *pRIICD->pbyData++;
            pRIICD->stCount--;

            /* Enable the transmit end interrupt */
            pRIICD->pRIIC->RIICnICIER.BIT.TEIE = 1;
        }
        else
        {
            /* Enable the interrupt */
            pRIICD->pRIIC->RIICnICIER.BIT.TIE = 1;
        }
    }
}
/******************************************************************************
End of function  r_drvRIIC_TI
******************************************************************************/

/******************************************************************************
* Function Name: r_drvRIIC_TEI
* Description  : Interrupt handler for TEI
* Arguments    : IN  pRIICD - Pointer to the RIIC driver data
* Return Value : none
******************************************************************************/
void r_drvRIIC_TEI(PRIICDRV pRIICD)
{
    /* Disable the transmit end interrupt */
    pRIICD->pRIIC->RIICnICIER.BIT.TEIE = 0;

    /* If the device has acknowledged the data */
    if ((!pRIICD->pRIIC->RIICnICSR2.BIT.NACKF)

    /* and there is more data to transmit */
    &&  (pRIICD->stCount))
    {
        /* If the transmit register is empty */
        if (pRIICD->pRIIC->RIICnICSR2.BIT.TDRE)
        {
            /* Send the next byte of data */
            r_drvRIIC_TI(pRIICD);
        }
        else
        {
            /* Enable the transmit empty interrupt to send the next
               byte when the hardware is ready */
            pRIICD->pRIIC->RIICnICIER.BIT.TIE = 1;
        }
    }
    else
    {
        /* Device has not acknowledged the data so stop transmission */
        eventSet(pRIICD->ppevSignals[RIIC_TRANSMISSION_COMPLETE]);
    }
}
/******************************************************************************
End of function  r_drvRIIC_TEI
******************************************************************************/

/******************************************************************************
* Function Name: r_drvRIIC_RI
* Description  : Interrupt handler for RI
* Arguments    : IN  pRIICD - Pointer to the RIIC driver data
* Return Value : none
******************************************************************************/
void r_drvRIIC_RI(PRIICDRV pRIICD)
{
    /* Check for transmit mode */
    if (pRIICD->bfTransmit)
    {
        pRIICD->pRIIC->RIICnICIER.BIT.RIE = 0;
        eventSet(pRIICD->ppevSignals[RIIC_RECEPTION_COMPLETE]);
    }
    else
    {
        pRIICD->stCount--;

        /* If there is only one byte to receive */
        if (1U == pRIICD->stCount)
        {
            /* Send NACK on last byte */
            pRIICD->pRIIC->RIICnICMR3.BIT.ACKBT = 1;
        }
        else
        {
            /* Send ACK on all other bytes */
            pRIICD->pRIIC->RIICnICMR3.BIT.ACKBT = 0;
        }

        /* Read the data from the register */
        *pRIICD->pbyData++ = pRIICD->pRIIC->RIICnICDRR.BIT.ICDRR;

        /* If there is more data to go */
        if (0UL == pRIICD->stCount)
        {
            /* This is the end of reception */
            pRIICD->pRIIC->RIICnICIER.BIT.RIE = 0;
            eventSet(pRIICD->ppevSignals[RIIC_RECEPTION_COMPLETE]);
        }
    }
}
/******************************************************************************
End of function  r_drvRIIC_RI
******************************************************************************/

/******************************************************************************
* Function Name: r_drvRIIC_SPI
* Description  : Interrupt handler for SPI
* Arguments    : IN  pRIICD - Pointer to the RIIC driver data
* Return Value : none
******************************************************************************/
void r_drvRIIC_SPI(PRIICDRV pRIICD)
{
    eventSet(pRIICD->ppevSignals[RIIC_STOP]);
    do
    {
        pRIICD->pRIIC->RIICnICSR2.BIT.STOP = 0;
    } while (pRIICD->pRIIC->RIICnICSR2.BIT.STOP);
}
/******************************************************************************
End of function  r_drvRIIC_SPI
******************************************************************************/

/******************************************************************************
* Function Name: r_drvRIIC_STI
* Description  : Interrupt handler for STI
* Arguments    : IN  pRIICD - Pointer to the RIIC driver data
* Return Value : none
******************************************************************************/
void r_drvRIIC_STI(PRIICDRV pRIICD)
{
    eventSet(pRIICD->ppevSignals[RIIC_START]);
    do
    {
        pRIICD->pRIIC->RIICnICSR2.BIT.START = 0;
    } while (pRIICD->pRIIC->RIICnICSR2.BIT.START);
}
/******************************************************************************
End of function  r_drvRIIC_STI
******************************************************************************/

/******************************************************************************
* Function Name: r_drvRIIC_NAK
* Description  : Interrupt handler for NAK
* Arguments    : IN  pRIICD - Pointer to the RIIC driver data
* Return Value : none
******************************************************************************/
void r_drvRIIC_NAK(PRIICDRV pRIICD)
{


}
/******************************************************************************
End of function  r_drvRIIC_NAK
******************************************************************************/

/******************************************************************************
* Function Name: r_drvRIIC_AL
* Description  : Interrupt handler for AL
* Arguments    : IN  pRIICD - Pointer to the RIIC driver data
* Return Value : none
******************************************************************************/
void r_drvRIIC_AL(PRIICDRV pRIICD)
{


}
/******************************************************************************
End of function  r_drvRIIC_AL
******************************************************************************/

/******************************************************************************
* Function Name: r_drvRIIC_TMO
* Description  : Interrupt handler for TMO
* Arguments    : IN  pRIICD - Pointer to the RIIC driver data
* Return Value : none
******************************************************************************/
void r_drvRIIC_TMO(PRIICDRV pRIICD)
{


}
/******************************************************************************
End of function  r_drvRIIC_TMO
******************************************************************************/

/******************************************************************************
Private global variables and functions
******************************************************************************/

/******************************************************************************
* Function Name: riic_init
* Description  : Function to initialise the hardware
* Arguments    : IN  pRIICD - Pointer to the RIIC driver data
* Return Value : none
******************************************************************************/
static void riic_init(PRIICDRV pRIICD)
{
    /* Clear the RIICn reset bit */
    pRIICD->pRIIC->RIICnICCR1.BIT.ICE = 0;

    /* Set I2C Bus Interface Internal Reset */
    pRIICD->pRIIC->RIICnICCR1.BIT.IICRST = 1;

    /* Set the RIICn reset bit */
    pRIICD->pRIIC->RIICnICCR1.BIT.ICE = 1;

    /* Set Internal Reference Clock Selection */
    pRIICD->pRIIC->RIICnICMR1.BIT.CKS = pRIICD->clockSelection;

    /* Set Bit Rate Low-Level Period */
    pRIICD->pRIIC->RIICnICBRL.BIT.BRL = pRIICD->ulBitRateLow;

    /* Set Bit Rate High-Level Period */
    pRIICD->pRIIC->RIICnICBRH.BIT.BRH = pRIICD->ulBitRateHigh;

    /* Set NACK Reception Transfer Suspension Enable */
    pRIICD->pRIIC->RIICnICFER.BIT.NACKE = 1;

    /* Disable all interrupts */
    pRIICD->pRIIC->RIICnICIER.LONG = 0;

    /* Clear I2C Bus Interface Internal Reset */
    pRIICD->pRIIC->RIICnICCR1.BIT.IICRST = 0;
}
/******************************************************************************
End of function  riic_init
******************************************************************************/

/******************************************************************************
End  Of File
*******************************************************************************/
