/******************************************************************************
  Copyright (C) 2014 Renesas Electronics Corporation, All Rights Reserved.
*******************************************************************************
* File Name    : drvRIIC.h
* Version      : 1.00
* Description  : Device driver for RIIC in master operation
******************************************************************************
* History : DD.MM.YYYY Version Description
*         : dd.mm.yyyy 1.00    First Release
******************************************************************************/

#ifndef DRVRIIC_H_INCLUDED
#define DRVRIIC_H_INCLUDED

#ifdef __cplusplus
extern "C" {
#endif

/******************************************************************************
System includes / User includes
******************************************************************************/
#include "r_typedefs.h"
#include "r_task.h"
#include "iodefine.h"

/******************************************************************************
Defines
******************************************************************************/

/******************************************************************************
Constant Macros
******************************************************************************/

/******************************************************************************
Function Macros
******************************************************************************/

/******************************************************************************
Enumerated Types
******************************************************************************/

/* Clock selection */
typedef enum _RIICCKS
{
    RIIC_CKS_DIVISION_1 = 0,
    RIIC_CKS_DIVISION_2,
    RIIC_CKS_DIVISION_4,
    RIIC_CKS_DIVISION_8,
    RIIC_CKS_DIVISION_16,
    RIIC_CKS_DIVISION_32,
    RIIC_CKS_DIVISION_64,
    RIIC_CKS_DIVISION_128
} RIICCKS;

typedef enum _RIICSIG
{
    RIIC_START = 0,
    RIIC_TRANSMISSION_COMPLETE,
    RIIC_RECEPTION_COMPLETE,
    RIIC_ACK,
    RIIC_NACK,
    RIIC_STOP,
    RIIC_NUM_SIGNALS
} RIICSIG;

/******************************************************************************
Typedefs
******************************************************************************/

typedef struct _RIICDRV
{
    /* Pointer to the hardware */
    volatile struct st_riic_n * pRIIC;
    RIICCKS         clockSelection;
    uint32_t        ulBitRateLow;
    uint32_t        ulBitRateHigh;
    uint8_t         byPort1Map;
    uint8_t         byPort2Map;
    uint8_t         *pbyData;
    size_t          stCount;
    PEVENT          ppevSignals[RIIC_NUM_SIGNALS];
    PEVENT          ppSignals[3]; // re-entrant signals
    int32_t         itemp;
    _Bool           bfTransmit;

} RIICDRV,
*PRIICDRV;

/******************************************************************************
Constant Data
******************************************************************************/

/******************************************************************************
Function Prototypes
******************************************************************************/

/******************************************************************************
External Variables
******************************************************************************/

/******************************************************************************
Global Variables
******************************************************************************/

/******************************************************************************
Public Functions
******************************************************************************/

/******************************************************************************
* Function Name: r_drvRIIC_Open
* Description  : Function to open the RIIC interface
* Arguments    : OUT pRIICD - Pointer to the RIIC driver data
*                IN  clockSelection - The clock selection
*                IN  ulBitRateLow - The low clock cycle value
*                IN  ulBitRateHigh - The high clock cycle value
* Return Value : 0 for success or -1 on error
******************************************************************************/

extern  int32_t r_drvRIIC_Open(PRIICDRV    pRIICD,
                               volatile struct st_riic_n *pRIIC,
                               RIICCKS     clockSelection,
                               uint32_t    ulBitRateLow,
                               uint32_t    ulBitRateHigh);

/******************************************************************************
* Function Name: r_drvRIIC_Close
* Description  : Function to close the RIIC interface
* Arguments    : IN  pRIICD - Pointer to the RIIC driver data
* Return Value : none
******************************************************************************/

extern  void r_drvRIIC_Close(PRIICDRV pRIICD);

/******************************************************************************
* Function Name: r_drvRIIC_TxStart
* Description  : Function to transmit the start condition
* Arguments    : IN  pRIICD - Pointer to the RIIC driver data
* Return Value : 0 for success or -1 on error
******************************************************************************/

extern  int32_t r_drvRIIC_TxStart(PRIICDRV pRIICD);

/******************************************************************************
* Function Name: int32_t r_drvRIIC_TxData
* Description  : Function to transmit data
* Arguments    : IN  pRIICD - Pointer to the RIIC driver data
*                IN  pvData - Pointer to the data to transmit
*                IN  stLength - The length of data to transmit
* Return Value : 0 for success or -1 on error
******************************************************************************/

extern  int32_t r_drvRIIC_TxData(PRIICDRV pRIICD,
                                 const void *pvData,
                                 size_t stLength);

/******************************************************************************
* Function Name: int32_t r_drvRIIC_RxData
* Description  : Function to receive data
* Arguments    : IN  pRIICD - Pointer to the RIIC driver data
*                IN  pvData - Pointer to the data buffer
*                IN  stLength - The length of data to receive
* Return Value : 0 for success or -1 on error
******************************************************************************/

extern  int32_t r_drvRIIC_RxData(PRIICDRV pRIICD,
                                 void   *pvData,
                                 size_t stLength);

/******************************************************************************
* Function Name: r_drvRIIC_TxStop
* Description  : Function to send the stop condition
* Arguments    : IN  pRIICD - Pointer to the RIIC driver data
* Return Value : 0 for success or -1 on error
******************************************************************************/

extern  int32_t r_drvRIIC_TxStop(PRIICDRV pRIICD);

/******************************************************************************
* Function Name: r_drvRIIC_TEI
* Description  : Interrupt handler for TEI
* Arguments    : IN  pRIICD - Pointer to the RIIC driver data
* Return Value : none
******************************************************************************/

extern  void r_drvRIIC_TEI(PRIICDRV pRIICD);

/******************************************************************************
* Function Name: r_drvRIIC_RI
* Description  : Interrupt handler for RI
* Arguments    : IN  pRIICD - Pointer to the RIIC driver data
* Return Value : none
******************************************************************************/

extern  void r_drvRIIC_RI(PRIICDRV pRIICD);

/******************************************************************************
* Function Name: r_drvRIIC_TI
* Description  : Interrupt handler for TI
* Arguments    : IN  pRIICD - Pointer to the RIIC driver data
* Return Value : none
******************************************************************************/

extern  void r_drvRIIC_TI(PRIICDRV pRIICD);

/******************************************************************************
* Function Name: r_drvRIIC_SPI
* Description  : Interrupt handler for SPI
* Arguments    : IN  pRIICD - Pointer to the RIIC driver data
* Return Value : none
******************************************************************************/

extern  void r_drvRIIC_SPI(PRIICDRV pRIICD);

/******************************************************************************
* Function Name: r_drvRIIC_STI
* Description  : Interrupt handler for STI
* Arguments    : IN  pRIICD - Pointer to the RIIC driver data
* Return Value : none
******************************************************************************/

extern  void r_drvRIIC_STI(PRIICDRV pRIICD);

/******************************************************************************
* Function Name: r_drvRIIC_NAK
* Description  : Interrupt handler for NAK
* Arguments    : IN  pRIICD - Pointer to the RIIC driver data
* Return Value : none
******************************************************************************/

extern  void r_drvRIIC_NAK(PRIICDRV pRIICD);

/******************************************************************************
* Function Name: r_drvRIIC_AL
* Description  : Interrupt handler for AL
* Arguments    : IN  pRIICD - Pointer to the RIIC driver data
* Return Value : none
******************************************************************************/

extern  void r_drvRIIC_AL(PRIICDRV pRIICD);

/******************************************************************************
* Function Name: r_drvRIIC_TMO
* Description  : Interrupt handler for TMO
* Arguments    : IN  pRIICD - Pointer to the RIIC driver data
* Return Value : none
******************************************************************************/

extern  void r_drvRIIC_TMO(PRIICDRV pRIICD);

#ifdef __cplusplus
}
#endif

#endif                              /* DRVRIIC_H_INCLUDED */

/*****************************************************************************
End  Of File
******************************************************************************/
