/******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only 
* intended for use with Renesas products. No other uses are authorized. This 
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS 
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE 
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the 
* following link:
* http://www.renesas.com/disclaimer
*******************************************************************************
* Copyright (C) 2012 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************
* File Name    : drvLed.c
* Version      : 1.00
* Device(s)    : Renesas
* Tool-Chain   : GNU
* OS           : FreeRTOS
* H/W Platform : RZA1
* Description  : LED device driver
*******************************************************************************
* History      : DD.MM.YYYY Version Description
*              : 04.02.2010 1.00    First Release
******************************************************************************/

/******************************************************************************
  WARNING!  IN ACCORDANCE WITH THE USER LICENCE THIS CODE MUST NOT BE CONVEYED
  OR REDISTRIBUTED IN COMBINATION WITH ANY SOFTWARE LICENSED UNDER TERMS THE
  SAME AS OR SIMILAR TO THE GNU GENERAL PUBLIC LICENCE
******************************************************************************/

/******************************************************************************
Includes   <System Includes> , "Project Includes"
******************************************************************************/

#include "r_typedefs.h"
#include "iodefine.h"
#include "drvLed.h"
#include "trace.h"
#include "r_task.h"
#include "trace.h"
#include "hwRIIC3.h"

/******************************************************************************
Macro definitions
******************************************************************************/

/* Comment this line out to turn ON module trace in this file */
#undef _TRACE_ON_

#ifndef _TRACE_ON_
#undef TRACE
#define TRACE(x)
#endif

#define PX_CMD_READ_REG                     (0x00u)
#define PX_CMD_WRITE_OUT_REG                (0x01u)
#define PX_CMD_POLARITY_REG                 (0x02u)
#define PX_CMD_CONFIG_REG                   (0x03u)
#define PX_CMD_READ                         (1)
#define PX_CMD_WRITE                        (0)

#define RZA1H_RSK_I2C_PX_IO1                (0x40)
#define RZA1H_RSK_I2C_PX_IO2                (0x42)

/******************************************************************************
Enumerated Types
******************************************************************************/

/******************************************************************************
Typedefs
******************************************************************************/

typedef void (*IFP)(uint32_t);

/*****************************************************************************
Function Prototypes
******************************************************************************/

static int ledOpen(PSTREAM pStream);
static void ledClose(PSTREAM pStream);
static int ledControl(PSTREAM pStream, CTLCODE ctlCode, void *pCtlStruct);
static int32_t ledConfigurePortExpanders(void);
static int32_t ledUpdate(int32_t iLedMap);

/*****************************************************************************
Constant Data
******************************************************************************/

/* Define the driver function table for this */
const DEVICE gLedDriver =
{
    "LED Device Driver",
    ledOpen,
    ledClose,
    nodevIO,
    nodevIO,
    ledControl
};

/******************************************************************************
Private global variables and functions
******************************************************************************/

static struct _DRVLED
{
    _Bool   bfOpen;
    uint8_t byPort0Map;
    uint8_t byPort1Map;
    int32_t iLedMap;
}
gLedDrv =
{
    false,
};

/******************************************************************************
Function Name: ledOpen
Description:   Function to open the system timer device driver
Arguments:     IN  pStream - Pointer to the file stream
Return value:  0 for success otherwise -1
******************************************************************************/
static int ledOpen(PSTREAM pStream)
{
    (void)pStream;
    if (!gLedDrv.bfOpen)
    {
        ledConfigurePortExpanders();
        gLedDrv.bfOpen = true;
        return 0;
    }
    return -1;
}
/******************************************************************************
End of function  ledOpen
******************************************************************************/

/******************************************************************************
Function Name: ledClose
Description:   Function to close the standard IN driver. All open devices
               connected to the led are closed.
Arguments:     IN  pStream - Pointer to the file stream
Return value:  none
******************************************************************************/
static void ledClose(PSTREAM pStream)
{
    (void)pStream;
    if (gLedDrv.bfOpen)
    {

        gLedDrv.bfOpen = false;
    }
}
/******************************************************************************
End of function  ledClose
******************************************************************************/

/******************************************************************************
Function Name: ledControl
Description:   Function to handle custom controls for the standard IN device
Arguments:     IN  pStream - Pointer to the file stream
               IN  ctlCode - The custom control code
               IN  pCtlStruct - Pointer to the custom control structure
Return value:  0 for success -1 on error
******************************************************************************/
static int ledControl(PSTREAM pStream, CTLCODE ctlCode, void *pCtlStruct)
{
    static  PEVENT pevLedMutex = NULL;
    int iResult = -1;
    /* Avoid unused parameter compiler warning */
    (void)pStream;
    eventWaitMutex(&pevLedMutex, EV_WAIT_INFINITE);
    if (pCtlStruct)
    {
        switch (ctlCode)
        {
            case CTL_SET_LED_ON:
            {
                int32_t iLedMap = (*(int32_t*)pCtlStruct);
                gLedDrv.iLedMap |= iLedMap;
                iResult = ledUpdate(gLedDrv.iLedMap);
                break;
            }
            case CTL_SET_LED_OFF:
            {
                int32_t iLedMap = (*(int32_t*)pCtlStruct);
                gLedDrv.iLedMap &= ~iLedMap;
                iResult = ledUpdate(gLedDrv.iLedMap);
                break;
            }
            case CTL_SET_LED_TOGGLE:
            {
                int32_t iLedMap = (*(int32_t*)pCtlStruct);
                gLedDrv.iLedMap ^= iLedMap;
                iResult = ledUpdate(gLedDrv.iLedMap);
                break;
            }
            case CTL_SET_LED_MAP:
            {
                gLedDrv.iLedMap = (*(int32_t*)pCtlStruct);
                iResult = ledUpdate(gLedDrv.iLedMap);
                break;
            }
            case CTL_GET_LED_MAP:
            {
                if (pCtlStruct)
                {
                    (*(int32_t*)pCtlStruct) = gLedDrv.iLedMap;
                    iResult = 0;
                }
                break;
            }
            default:
            {
                TRACE(("drvLed: Unknown control code\r\n"));
                break;
            }
        }
    }
    eventReleaseMutex(&pevLedMutex);
    return iResult;
}
/******************************************************************************
End of function  ledControl
******************************************************************************/

/******************************************************************************
* Function Name: ledWritePortExpander
* Description  : Function to write to the port expander
* Arguments    : IN  pbyCommand - Pointer to the command to write
*                IN  stLength - The length of the command
* Return Value : 0 for success or -1 on error
******************************************************************************/
static int32_t ledWritePortExpander(uint8_t *pbyCommand, size_t stLength)
{
    int32_t iResult = -1;
    if (0 == r_RIIC3_TxStart())
    {
        if (0 == r_RIIC3_TxData(pbyCommand, stLength))
        {
            iResult = 0;
        }
        else
        {
            TRACE(("ledWritePortExpander: r_RIIC3_TxData failed\r\n"));
        }
    }
    if (r_RIIC3_TxStop())
    {
        TRACE(("ledWritePortExpander: r_RIIC3TxStop failed\r\n"));
        return -1;
    }
    taskSleep(1UL);
    return iResult;
}
/******************************************************************************
End of function  ledWritePortExpanderRegister
******************************************************************************/

/******************************************************************************
* Function Name: ledConfigurePortExpanders
* Description  : Function to configure the port expanders
* Arguments    : none
* Return Value : 0 for success or -1 on error
******************************************************************************/
static int32_t ledConfigurePortExpanders(void)
{
    int32_t iResult = -1;
    if (0 == r_RIIC3_Open())
    {
        uint8_t pbyCommand[3];
        /* Configure port expander 0 */
        pbyCommand[0] = RZA1H_RSK_I2C_PX_IO1 + PX_CMD_WRITE;
        pbyCommand[1] = PX_CMD_CONFIG_REG;
        /*
         * IO[0] = LED1             (output)
         * IO[1] = LED2             (output)
         * IO[2] = LED3             (output)
         * IO[3] = NOR_A25          (output)
         * IO[4] = PMOD1_RST        (output)
         * IO[5] = PMOD2_RST        (output)
         * IO[6] = SD_CONN_PWR_EN   (output)
         * IO[7] = SD_MMC_PWR_EN    (output)
         */
        pbyCommand[2] = 0x00;
        if (ledWritePortExpander(pbyCommand, sizeof(pbyCommand)))
        {
            TRACE(("ledConfigurePortExpanders: Failed to configure expander 0\r\n"));
        }
        /* Configure port expander 1 */
        pbyCommand[0] = RZA1H_RSK_I2C_PX_IO2 + PX_CMD_WRITE;
        pbyCommand[1] = PX_CMD_CONFIG_REG;
        /*
         * IO[0] = PX1_EN0          (output)
         * IO[1] = PX1_EN1          (output)
         * IO[2] = TFT_CS           (output)
         * IO[3] = PX1_EN3          (output)
         * IO[4] = USB_OVR_CURRENT  (input)
         * IO[5] = USB_PWR_ENA      (output)
         * IO[6] = USB_PWR_ENB      (output)
         * IO[7] = PX1_EN7          (output)
         */
        pbyCommand[2] = 0x10;
        if (ledWritePortExpander(pbyCommand, sizeof(pbyCommand)))
        {
            TRACE(("ledConfigurePortExpanders: Failed to configure expander 1\r\n"));
        }
        /* Switch all the LEDs off */
        gLedDrv.byPort0Map = 0xFF;
        gLedDrv.byPort1Map = 0xFF;
        r_RIIC3_Close();
    }
    else
    {
        TRACE(("ledConfigurePortExpanders: r_RIIC3_Open failed\r\n"));
    }
    return iResult;
}
/******************************************************************************
End of function  ledConfigurePortExpanders
******************************************************************************/

/******************************************************************************
* Function Name: ledUpdate
* Description  : Function to update the LEDs
* Arguments    : IN  byLedMap - a bit map of the LEDs
* Return Value : 0 for success or -1 on error
******************************************************************************/
static int32_t ledUpdate(int32_t iLedMap)
{
    int32_t iResult = -1;
    uint8_t byLedMap = (uint8_t)iLedMap;
    /* The LEDs are mapped in the bit positions 0..3.
     * LED's 1,2 and 3 are in bit positions 0,1 and 2 on the port expander
     * and LED0 is on a port pin. A logic 0 switches the LED on.
     * */
    /* First check for the LED on the port pin */
    if (byLedMap & LED0)
    {
        /* Switch the LED on the port pin on */
        PORT7.Pn.BIT.Pn1 = 0;
    }
    else
    {
        /* Switch the LED on the port pin off */
        PORT7.Pn.BIT.Pn1 = 1;
    }
    /* Shift right to place in the bit positions for the port expander */
    byLedMap >>= 1;
    /* Invert the bits */
    byLedMap ^= 0x07;
    /* Place into the port map */
    gLedDrv.byPort0Map = (gLedDrv.byPort0Map & 0xF8) | (byLedMap & 0x07);
    /* Open the RIIC3 interface */
    if (0 == r_RIIC3_Open())
    {
        uint8_t pbyCommand[3];
        /* Write to the output register for IO1 */
        pbyCommand[0] = RZA1H_RSK_I2C_PX_IO1 + PX_CMD_WRITE;
        pbyCommand[1] = PX_CMD_WRITE_OUT_REG;
        pbyCommand[2] = gLedDrv.byPort0Map;
        iResult = ledWritePortExpander(pbyCommand, sizeof(pbyCommand));
    }

    /* Close the RIIC3 interface */
    r_RIIC3_Close();
    taskSleep(10);

    return iResult;
}
/******************************************************************************
End of function  ledUpdate
******************************************************************************/

/******************************************************************************
End  Of File
******************************************************************************/
