/******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only 
* intended for use with Renesas products. No other uses are authorized. This 
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS 
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE 
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the 
* following link:
* http://www.renesas.com/disclaimer
*******************************************************************************
* Copyright (C) 2012 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************
* File Name    : drvRtc.c
* Version      : 1.0
* Description  : Software emulation of a real time clock
*******************************************************************************
* History      : DD.MM.YYYY Version Description
*              : 04.02.2010 1.00    First Release
*              : 10.06.2010 1.01    Updated type definitions
******************************************************************************/

/******************************************************************************
  WARNING!  IN ACCORDANCE WITH THE USER LICENCE THIS CODE MUST NOT BE CONVEYED
  OR REDISTRIBUTED IN COMBINATION WITH ANY SOFTWARE LICENSED UNDER TERMS THE
  SAME AS OR SIMILAR TO THE GNU GENERAL PUBLIC LICENCE
******************************************************************************/

/******************************************************************************
Includes   <System Includes> , "Project Includes"
******************************************************************************/

#include <string.h>
#include <stdio.h>
#include "drvRTC.h"
#include "r_timer.h"
#include "trace.h"
/* Adding this dependency ensures that this file is built every time because
   of the build counter */
#include "version.h"

/******************************************************************************
Macro definitions
******************************************************************************/

/* Comment this line out to turn ON module trace in this file */
#undef _TRACE_ON_

#ifndef _TRACE_ON_
#undef TRACE
#define TRACE(x)
#endif

/******************************************************************************
Private global variables and functions
******************************************************************************/

static int rtcOpen(PSTREAM pStream);
static void rtcClose(PSTREAM pStream);
static int rtcControl(PSTREAM pStream, CTLCODE ctlCode, void *pCtlStruct);
static _Bool rtcDateIsInvalid(int iDay, int iMonth, int iYear);
static DATE rtcGetDate(void);
static _Bool rtcSetDate(DATE Date);
static void rtcTick(void);

const DEVICE gRtcDriver =
{
    "Real Time Clock Emulation Driver",
    rtcOpen,
    rtcClose,
    nodevIO,
    nodevIO,
    rtcControl
};

static CBTMR    gRTC;
static int      giYear;
static int      giMonth = 1;
static int      giDay;
static int      giHour;
static int      giMinute;
static int      giSecond;

/******************************************************************************
Public Functions
******************************************************************************/

/******************************************************************************
Private Functions
******************************************************************************/

/******************************************************************************
* Function Name: rtcOpen
* Description  : Function to open the emulated RTC
* Arguments    : IN  pStream - Pointer to the file stream
* Return Value : 0 for success otherwise -1
******************************************************************************/
static int rtcOpen(PSTREAM pStream)
{
    static _Bool bfFirstOpen = true;
    /* Avoid unused parameter compiler warning */
    (void)pStream;
    if (bfFirstOpen)
    {
        char    pszMonth[16];
        char    chDelim;
        static const char * const ppszMonth[] =
        {
            "---", 
            "Jan", "Feb", "Mar", "Apr", "May", "Jun",
            "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"
        };
        size_t stCount = sizeof(ppszMonth) / sizeof(char*);
        /* Initialise the time from the build time of this file */
        sscanf(__TIME__, "%d%c%d%c%d", &giHour, &chDelim,
               &giMinute, &chDelim, &giSecond);
        sscanf(__DATE__, "%s%d%d", pszMonth, &giDay, &giYear);
        /* Look up the month */
        while (stCount--)
        {
            if (strstr(pszMonth, ppszMonth[stCount]))
            {
                giMonth = (int)stCount;
                break;
            }
        }
        bfFirstOpen = false;
    }
    /* Start a 1 second timer for the clock */
    timerStartCallBack(&gRTC, (PCFNTMR)rtcTick, NULL, 1000UL, PERIODIC_TIMER);
    return 0;
}
/******************************************************************************
End of function rtcOpen
******************************************************************************/

/******************************************************************************
* Function Name: rtcClose
* Description  : Function to close the SCIF
* Arguments    : IN  pStream - Pointer to the file stream
* Return Value : none
******************************************************************************/
static void rtcClose(PSTREAM pStream)
{
    /* Avoid unused parameter compiler warning */
    (void)pStream;
    /* Stop the timer */
    timerStopCallBack(&gRTC);
}
/******************************************************************************
End of function rtcClose
******************************************************************************/

/******************************************************************************
* Function Name: rtcControl
* Description  : Function to handle custom control functions for SCIF
* Arguments    : IN  pStream - Pointer to the file stream
*                IN  ctlCode - The custom control code
*                IN  pCtlStruct - Pointer to the custom control structure
* Return Value : 0 for success -1 on error
******************************************************************************/
static int rtcControl(PSTREAM pStream, CTLCODE ctlCode, void *pCtlStruct)
{
    /* Avoid unused parameter compiler warning */
    (void)pStream;
    switch (ctlCode)
    {
        case CTL_GET_DATE:
        {
            if (pCtlStruct)
            {
                /* Return the current date */
                *((PDATE)pCtlStruct) = rtcGetDate();
                return 0;
            }
            else
            {
                return -1;
            }
        }
        case CTL_SET_DATE:
        {
            if (pCtlStruct)
            {
                /* Set the date */
                if (rtcSetDate(*((PDATE)pCtlStruct)))
                {
                    return 0;
                }
            }
            return -1;
        }
        default:
        {
            TRACE(("drvRtc: Unknown control code\r\n"));
            break;
        }
    }
    return -1;
}
/******************************************************************************
End of function  rtcControl
******************************************************************************/

/******************************************************************************
* Function Name: rtcCalcWeekDay
* Description  : Function to calculate the week day for a given date from 2000
*                to 2099.
* Arguments    : IN  iDay - The day 1..31
*                IN  iMonth - The month 1..12
*                IN  iYear - The year 2009..2099
* Return Value : The weekday 0 = Sunday ... 6 = Sunday
******************************************************************************/
int rtcCalcWeekDay(int iDay, int iMonth, int iYear)
{
    if (iMonth < 3)
    {
        iMonth += 12;
        iYear -= 1;
    }
    return (iDay + (2 * iMonth) + (6 * (iMonth + 1) / 10) + iYear 
            + (iYear / 4)- (iYear / 100) + (iYear / 400)) % 7;
}
/******************************************************************************
End of function  rtcCalcWeekDay
******************************************************************************/

/******************************************************************************
* Function Name: rtcDateIsInvalid
* Description  : Function to check if a date is _not_ valid
* Arguments    : IN  iDay - The day 1..31
*                IN  iMonth - The month 1..12
*                IN  iYear - The year 2009..2099
* Return Value : TRUE if the date is invalid
******************************************************************************/
static _Bool rtcDateIsInvalid(int iDay, int iMonth, int iYear)
{
    int     iDaysInMonth;
    static const int piDaysInMonth[] = 
    {
        0, 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31
    };
    /* Check for valid day and month range */
    if ((iDay > 31) || (iDay < 1) || (iMonth > 12) ||(iMonth < 1))
    {
        return true;
    }
    /* Look up the number of days in this month and account for a leap year */
    if ((iMonth == 2) && (iYear % 4 == 0)
    &&  ((iYear % 100 != 0) || (iYear % 400 == 0)))
    {
        iDaysInMonth = piDaysInMonth[iMonth] + 1;
    }
    else
    {
        iDaysInMonth = piDaysInMonth[iMonth];
    }
    /* Check the number of days in the month */
    if (iDay > iDaysInMonth)
    {
        return true;
    }
    return false;
}
/******************************************************************************
End of function  rtcDateIsInvalid
******************************************************************************/

/******************************************************************************
* Function Name: rtcGetDate
* Description  : Function to get the current date
* Arguments    : none
* Return Value : The current date
******************************************************************************/
static DATE rtcGetDate(void)
{
    DATE    Date;
    Date.Field.WeekDay = rtcCalcWeekDay(giDay, giMonth, giYear);
    Date.Field.Year = giYear;
    Date.Field.Month = giMonth;
    Date.Field.Day = giDay;
    Date.Field.Hour = giHour;
    Date.Field.Minute = giMinute;
    Date.Field.Second = giSecond;
    return Date;
}
/******************************************************************************
End of function  rtcGetDate
******************************************************************************/

/******************************************************************************
* Function Name: rtcSetDate
* Description  : Function to set the date
* Arguments    : IN  Date - The date to set
* Return Value : TRUE if the date was set
******************************************************************************/
static _Bool rtcSetDate(DATE Date)
{
    /* Check for a valid date */
    if (rtcDateIsInvalid((int)Date.Field.Day,
                         (int)Date.Field.Month,
                         (int)Date.Field.Year))
    {
        return false;
    }
    /* Check for a valid time */
    if ((Date.Field.Hour > 23)
    ||  (Date.Field.Minute > 59)
    ||  (Date.Field.Second > 59))
    {
        return false;
    }
    /* Set the date */
    rtcClose(NULL);
    giYear = (int)Date.Field.Year;
    giMonth = (int)Date.Field.Month;
    giDay = (int)Date.Field.Day;
    giHour = (int)Date.Field.Hour;
    giMinute = (int)Date.Field.Minute;
    giSecond = (int)Date.Field.Second;
    rtcOpen(NULL);
    return true;
}
/******************************************************************************
End of function  rtcSetDate
******************************************************************************/

/******************************************************************************
* Function Name: rtcTick
* Description  : Function to increment the current time and date
* Arguments    : none
* Return Value : none
******************************************************************************/
static void rtcTick(void)
{
    giSecond++;
    if (giSecond >= 60)
    {
        giSecond = 0;
        giMinute++;
        if (giMinute >= 60)
        {
            giMinute = 0;
            giHour++;
            if (giHour >= 24)
            {
                giHour = 0;
                giDay++;
                if (rtcDateIsInvalid(giDay, giMonth, giYear))
                {
                    giDay = 1;
                    giMonth++;
                    if (rtcDateIsInvalid(giDay, giMonth, giYear))
                    {
                        giMonth = 1;
                        giYear++;
                    }
                }
            }
        }
    }
}
/******************************************************************************
End of function  rtcTick
******************************************************************************/

/******************************************************************************
End  Of File
******************************************************************************/
