/******************************************************************************
* DISCLAIMER

* This software is supplied by Renesas Electronics Corporation and is only 
* intended for use with Renesas products. No other uses are authorized.

* This software is owned by Renesas Electronics Corporation and is protected under 
* all applicable laws, including copyright laws.

* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES 
* REGARDING THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, 
* INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A 
* PARTICULAR PURPOSE AND NON-INFRINGEMENT.  ALL SUCH WARRANTIES ARE EXPRESSLY 
* DISCLAIMED.

* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS 
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE 
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES 
* FOR ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS 
* AFFILIATES HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.

* Renesas reserves the right, without notice, to make changes to this 
* software and to discontinue the availability of this software.  
* By using this software, you agree to the additional terms and 
* conditions found by accessing the following link:
* http://www.renesas.com/disclaimer
******************************************************************************/
/* Copyright (C) 2014 Renesas Electronics Corporation. All rights reserved.  */
/******************************************************************************
* File Name    : hwR1EX24016.c
* Version      : 1.00
* Device       : RZA1
* Tool-Chain   : GNU
* H/W Platform : RSK2+RZA1H
* Description  : Hardware interface and device driver for R1EX24016
******************************************************************************
* History : DD.MM.YYYY Version Description
*         : dd.mm.yyyy 1.00    First Release
******************************************************************************/

/******************************************************************************
Includes   <System Includes> , "Project Includes"
******************************************************************************/
#include <stdlib.h>
#include "hwR1EX24016.h"
#include "trace.h"
#include "r_task.h"
#include "hwRIIC3.h"
//#include "interruptPriority.h"

/******************************************************************************
Defines
******************************************************************************/

/* Define the device size in bytes - issue with IIC driver / peripheral means
 * that the size is reduced by two bytes because the device gets stuck in the transmit
 * state and holds SDA low preventing the host sending the stop condition.
 * This is probably a bug in the driver...
 * TODO: Look at EEROM and IIC driver
 * */
#define EE_MEMORY_SIZE              2032UL
#define EE_TIME_OUT_MS              1000
#define EE_COUNT_OUT                10
#define EE_CONTOL_READ              0xA1
#define EE_CONTOL_WRITE             0xA0
#define EE_WRITE_CYCLE_TIME         (5)
#define EE_PAGE_SIZE                (16)

/* Comment this line out to turn ON module trace in this file */
//#undef _TRACE_ON_

#ifndef _TRACE_ON_
#undef TRACE
#define TRACE(x)
#endif

/*****************************************************************************
Function Macros
******************************************************************************/

/* Define a macro to set the upper three bits of the addres */
#define EE_MAKE_CONTROL(a, t)       (uint8_t)(((a >> 7)  & 0xEUL) | t)

/******************************************************************************
Enumerated Types
******************************************************************************/

/******************************************************************************
Typedefs
******************************************************************************/

/*****************************************************************************
Function Prototypes
******************************************************************************/

static int romOpen(PSTREAM pStream);
static void romClose(PSTREAM pStream);
static int romRead(PSTREAM pStream, uint8_t *pbyBuffer, uint32_t uiCount);
static int romWrite(PSTREAM pStream, uint8_t *pbyBuffer, uint32_t uiCount);
static int romControl(PSTREAM pStream, CTLCODE ctlCode, void *pCtlStruct);
static _Bool eeWrite(size_t stAddress, uint8_t *pbyData, size_t stLength);
static _Bool eeRead(size_t stAddress, uint8_t *pbyData, size_t stLength);

/*****************************************************************************
Constant Data
******************************************************************************/

/* Define the driver function table for this device */
const DEVICE gR1EX24016Driver =
{
    "R1EX24016 Device Driver",
    romOpen,
    romClose,
    romRead,
    romWrite,
    romControl
};

/******************************************************************************
Imported global variables and functions (from other files)
******************************************************************************/

/******************************************************************************
Exported global variables and functions (to be accessed by other files)
******************************************************************************/

/******************************************************************************
Private global variables and functions
******************************************************************************/

/*****************************************************************************
* Function Name: romOpen
* Description  : Function to open the EEPROM
* Arguments    : IN  pStream - Pointer to the file stream
* Return Value : 0 for success otherwise -1
******************************************************************************/
static int romOpen(PSTREAM pStream)
{
    /* The extension pointer is used as the file index - set to zero */
    pStream->pExtension = NULL;
    return 0;
}
/*****************************************************************************
End of function  romOpen
******************************************************************************/

/*****************************************************************************
* Function Name: romClose
* Description  : Function to close the EEPROM
* Arguments    : IN  pStream - Pointer to the IOStream
* Return Value : none
******************************************************************************/
static void romClose(PSTREAM pStream)
{
    (void)pStream;
}
/*****************************************************************************
End of function  romClose
******************************************************************************/

/*****************************************************************************
* Function Name: romRead
* Description  : Function to read from the EEPROM
* Arguments    : IN  pStream - Pointer to the IOStream
*                IN  pbyBuffer - Pointer to the destination buffer
*                IN  uiCount - The number of bytes to read
* Return Value : The number of bytes read, or -1 on error
******************************************************************************/
static int romRead(PSTREAM pStream, uint8_t *pbyBuffer, uint32_t uiCount)
{
    size_t  stAddress = (size_t)pStream->pExtension;
    int     iResult;
    /* Check for the end of file */
    if (stAddress > EE_MEMORY_SIZE)
    {
        return EOF;
    }
    /* Limit the read to the end of memory */
    if ((stAddress + uiCount) > EE_MEMORY_SIZE)
    {
        uiCount = EE_MEMORY_SIZE - stAddress;
    }
    /* Open the interface to the EEPROM */
    if (0 == r_RIIC3_Open())
    {
        /* Read from the device */
        if (eeRead(stAddress, pbyBuffer, (size_t)uiCount))
        {
            stAddress += uiCount;
            pStream->pExtension = (void*)stAddress;
        }
        else
        {
            /* The transfer failed */
            uiCount = -1;
        }
        /* Close the interface to the EEPROM */
        r_RIIC3_Close();
    }
    return uiCount;
}
/*****************************************************************************
End of function  romRead
******************************************************************************/

/*****************************************************************************
* Function Name: romWrite
* Description  : Function to write to the EEPROM
* Arguments    : IN  pStream - Pointer to the IO stream
*                IN  pbyBuffer - Pointer to the destination buffer
*                IN  uiCount - The nuber of characters to write
* Return Value : The number of characters written or -1 on error
******************************************************************************/
static int romWrite(PSTREAM pStream, uint8_t *pbyBuffer, uint32_t uiCount)
{
    size_t  stAddress = (size_t)pStream->pExtension;
    /* Limit the write to the end of memory */
    if ((stAddress + uiCount) > EE_MEMORY_SIZE)
    {
        uiCount = EE_MEMORY_SIZE - stAddress;
    }
    /* Open the interface to the EEPROM */
    if (0 == r_RIIC3_Open())
    {
        /* Write to the device */
        if (eeWrite(stAddress, pbyBuffer, (size_t)uiCount))
        {
            stAddress += uiCount;
            pStream->pExtension = (void*)stAddress;
        }
        else
        {
            /* The transfer failed */
            uiCount = -1;
        }
        /* Close the interface to the EEPROM */
        r_RIIC3_Close();
    }
    return uiCount;
}
/*****************************************************************************
End of function  romWrite
******************************************************************************/

/*****************************************************************************
* Function Name: romControl
* Description  : Function to handle file specific controls
* Arguments    : IN  pStream - Pointer to the file stream
*                IN  ctlCode - The custom control code
*                IN  pCtlStruct - Pointer to the custom control structure
* Return Value : 0 for success and -1 on error
******************************************************************************/
static int romControl(PSTREAM pStream, CTLCODE ctlCode, void *pCtlStruct)
{
    switch (ctlCode)
    {
        case CTL_FILE_SEEK:
        {
            if (pCtlStruct)
            {
                PFILESEEK   pSeek = (PFILESEEK)pCtlStruct;
                if (pSeek->iBase == SEEK_SET)
                {
                    /* Don't set it if it is passed the end of storage */
                    if (pSeek->lOffset <= EE_MEMORY_SIZE)
                    {
                        /* Set the file file index to the desired offset */
                        pStream->pExtension = (void*)pSeek->lOffset;
                        pSeek->lResult = pSeek->lOffset;
                        return 0;
                    }
                }
                else if (pSeek->iBase == SEEK_CUR)
                {
                    /* Add on the offset to the current position */
                    size_t  stPosition = (size_t)pStream->pExtension;
                    stPosition += pSeek->lOffset;
                    if (stPosition <= EE_MEMORY_SIZE)
                    {
                        pStream->pExtension = (void*)stPosition;
                        pSeek->lResult = stPosition;
                        return 0;
                    }
                }
                else if (pSeek->iBase  == SEEK_END)
                {
                    size_t  stPosition = EE_MEMORY_SIZE + pSeek->lOffset;
                    if (stPosition <= EE_MEMORY_SIZE)
                    {
                        /* Set the file pointer to the end of memory */
                        pStream->pExtension = (void*)stPosition;
                        pSeek->lResult = stPosition;
                        return 0;
                    }
                }
            }
            break;
        }
        case CTL_FILE_SIZE:
        {
            if (pCtlStruct)
            {
                *((uint32_t *)pCtlStruct) = EE_MEMORY_SIZE;
                return 0;
            }
            break;
        }
        default:
        {
            return -1;
        }
    }
    return -1;
}
/*****************************************************************************
End of function  romControl
******************************************************************************/

/******************************************************************************
* Function Name: eeWriteAddress
* Description  : Function to write the access address to the device
* Arguments    : IN  stAddress - The address
* Return Value : true if the address was written to the device
******************************************************************************/
static _Bool eeWriteAddress(size_t stAddress)
{
    uint8_t pbyAddressCommand[2];
    /* Make the command */
    pbyAddressCommand[0] = EE_MAKE_CONTROL(stAddress, EE_CONTOL_WRITE);
    pbyAddressCommand[1] = (uint8_t)stAddress;
    /* Issue a start */
    if (0 == r_RIIC3_TxStart())
    {
        /* Write the control byte and the address */
        if (0 == r_RIIC3_TxData(&pbyAddressCommand, sizeof(pbyAddressCommand)))
        {
            return true;
        }
        else
        {
            TRACE(("EER_WriteAddress: Device not responding\r\n"));
        }
    }
    return false;
}
/******************************************************************************
End of function  eeWriteAddress
******************************************************************************/

/******************************************************************************
* Function Name: eeRead
* Description  : Function to read from the device
* Arguments    : IN  stAddress - The address to read from
                 OUT pbyData - Pointer to the destination memory
                 IN  stLength - The number of bytes to read
* Return Value : true if the read was successful
******************************************************************************/
static _Bool eeRead(size_t stAddress, uint8_t *pbyData, size_t stLength)
{
    _Bool   bfResult = false;
    uint8_t byControl = EE_MAKE_CONTROL(stAddress, EE_CONTOL_READ);
    /* Write the address */
    if (eeWriteAddress(stAddress))
    {
        /* Issue the start condition */
        if (0 == r_RIIC3_TxStart())
        {
            /* Write the control byte for a data read */
            if (0 == r_RIIC3_TxData(&byControl, sizeof(byControl)))
            {
                if (0 == r_RIIC3_RxData(pbyData, stLength))
                {
                    bfResult = true;
                }
                else
                {
                    TRACE(("eeRead: failed r_RIIC3_RxData\r\n"));
                }
            }
            else
            {
                TRACE(("eeRead: Control byte NACKED\r\n"));
            }
        }
        else
        {
            TRACE(("eeRead: r_RIIC3_TxStart failed after address write\r\n"));
        }
    }
    else
    {
        TRACE(("eeRead: failed eeWriteAddress\r\n"));
    }
    if (r_RIIC3_TxStop())
    {
        TRACE(("eeRead: failed r_RIIC3TxStop\r\n"));
        return false;
    }
    return bfResult;
}
/******************************************************************************
End of function  eeRead
******************************************************************************/

/*****************************************************************************
* Function Name: eeWritePage
* Description  : Function to write to a page to the EEPROM
* Arguments    : IN  stAddress - The address to write to (no offset)
*                IN  pbyData - Pointer to the data to write
*                IN  stLength - The length of data to write
* Return Value : true if the data was written successfully
******************************************************************************/
static _Bool eeWritePage(size_t stAddress, uint8_t *pbyData, size_t stLength)
{
    _Bool   bfResult = false;
    /* Write the address */
    if (eeWriteAddress(stAddress))
    {
        /* Write the page of data or less */
        if (0 == r_RIIC3_TxData(pbyData, stLength))
        {
            bfResult = true;
        }
        else
        {
            TRACE(("eeWritePage: failed r_RIIC3_TxData\r\n"));
        }
    }
    else
    {
        TRACE(("eeWritePage: failed eeWriteAddress\r\n"));
    }
    if (r_RIIC3_TxStop())
    {
        TRACE(("eeWritePage: failed r_RIIC3TxStop\r\n"));
        return false;
    }
    taskSleep(EE_WRITE_CYCLE_TIME);
    return bfResult;
}

/*****************************************************************************
* Function Name: eeWrite
* Description  : Function to write to the EEPROM
* Arguments    : IN  stAddress - The address to write to (no offset)
*                IN  pbyData - Pointer to the data to write
*                IN  stLength - The length of data to write
* Return Value : true if the data was written successfully
******************************************************************************/
static _Bool eeWrite(size_t stAddress, uint8_t *pbyData, size_t stLength)
{
    size_t stLengthWritten = 0UL;
    while (stLengthWritten < stLength)
    {
        /* Calculate the length to write */
        size_t  stLengthToWrite = stLength - stLengthWritten;
        /* Can only write a page on a page boundary */
        if ((stAddress % EE_PAGE_SIZE) == 0)
        {
            /* Make sure that it does not exceed the page size of the device */
            if (stLengthToWrite > EE_PAGE_SIZE)
            {
                stLengthToWrite = EE_PAGE_SIZE;
            }
        }
        else
        {
            stLengthToWrite = 1;
        }
        /* Write the page to the device */
        if (eeWritePage(stAddress, pbyData, stLengthToWrite))
        {
            /* Total up the number of bytes written */
            stLengthWritten += stLengthToWrite;
            /* Increment the pointer */
            pbyData += stLengthToWrite;
            /* Increment the address to write to */
            stAddress += stLengthToWrite;
        }
        else
        {
            TRACE(("eeWrite: failed at %lu bytes\r\n", stLengthWritten));
            return false;
        }
    }
    return true;
}
/*****************************************************************************
End of function  eeWrite
******************************************************************************/

/*****************************************************************************
End  Of File
******************************************************************************/
