/******************************************************************************
* DISCLAIMER

* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized.
* This software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES
* REGARDING THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY,
* INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
* PARTICULAR PURPOSE AND NON-INFRINGEMENT.  ALL SUCH WARRANTIES ARE EXPRESSLY
* DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES
* FOR ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS
* AFFILIATES HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this
* software and to discontinue the availability of this software.
* By using this software, you agree to the additional terms and
* conditions found by accessing the following link:
* http://www.renesas.com/disclaimer
******************************************************************************/
/* Copyright (C) 2011 Renesas Electronics Corporation. All rights reserved.  */
/******************************************************************************
* File Name    : hwRIIC0.c
* Version      : 1.00
* Device       : RZA1
* Tool-Chain   : GNU
* H/W Platform : RSK2+RZA1H
* Description  : Hardware interface for RIIC0
******************************************************************************
* History : DD.MM.YYYY Version Description
*         : dd.mm.yyyy 1.00    First Release
******************************************************************************/

/******************************************************************************
Includes   <System Includes> , "Project Includes"
******************************************************************************/
#include <stdlib.h>
#include "hwRIIC0.h"
#include "drvRIIC.h"
#include "devdrv_intc.h"
#include "dev_drv.h"
#include "rza_io_regrw.h"
#include "trace.h"
//#include "interruptPriority.h"

/******************************************************************************
Defines
******************************************************************************/
//TODO: Interrupt priority
#define RIIC0_INTERRUPT_PRIORITY    26//RIIC0_INTERRUPT_PRIORITY

/* IIC data write code */
#define SAMPLE_RIIC_RWCODE_W        (0)
/* IIC data read code  */
#define SAMPLE_RIIC_RWCODE_R        (1)
/* Sampling loop */
#define DELAY_TIME_1MS              (260000uL)
/* IIC channel attached to the audio Codec */
#define CODEC_IIC_CH                (3)


/* FocalTech CTP Controller  */
#define RZA1H_APP_I2C_FT5216    (0x70)
/* TI PanelBus� DIGITAL TRANSMITTER */
#define RZA1H_APP_I2C_TFP410    (0x78)


/* ==== Transmission mode ==== */
#define RIIC_TX_MODE_START      (0) /* Start condition issuance request   */
#define RIIC_TX_MODE_RESTART    (1) /* Restart condition issuance request */
#define RIIC_TX_MODE_STOP       (2) /* Stop condition issuance request    */



/******************************************************************************
Constant Macros
******************************************************************************/

/******************************************************************************
Function Macros
******************************************************************************/

/******************************************************************************
Typedef definitions
******************************************************************************/

typedef void (*IFP)(uint32_t);

/******************************************************************************
Enumerated Types
******************************************************************************/

/******************************************************************************
Macro definitions
******************************************************************************/

/*****************************************************************************
Function Prototypes
******************************************************************************/

static void r_RIIC0_TEI(void);
static void r_RIIC0_RI(void);
static void r_RIIC0_TI(void);
static void r_RIIC0_SPI(void);
static void r_RIIC0_STI(void);
static void r_RIIC0_NAK(void);
static void r_RIIC0_AL(void);
static void r_RIIC0_TMO(void);
static int32_t r_RIIC0_TxStart(void);
static int32_t r_RIIC0_TxData(const void *pvData, size_t stLength);
static int32_t r_RIIC0_RxData(void *pvData, size_t stLength);
static int32_t r_RIIC0_TxStop(void);

/******************************************************************************
Constant Data
******************************************************************************/

/******************************************************************************
Imported global variables and functions (from other files)
******************************************************************************/

/******************************************************************************
Private global variables
******************************************************************************/

/* The memory required by the RIIC device driver */
static RIICDRV  gRIIC0DRV;

/* A mutex event to protect from access by multiple tasks */
static PEVENT   gpRII0Mutex = NULL;


/******************************************************************************
Exported global variables and functions (to be accessed by other files)
******************************************************************************/


/******************************************************************************
* Function Name: WritePortExpander
* Description  : Function to write to the port expander
* Arguments    : IN  pbyCommand - Pointer to the command to write
*                IN  stLength - The length of the command
* Return Value : 0 for success or -1 on error
******************************************************************************/
static int32_t WritePortExpander(uint8_t *pbyCommand, size_t stLength)
{
    int32_t iResult = -1;
    if (0 == r_RIIC0_TxStart())
    {
        if (0 == r_RIIC0_TxData(pbyCommand, stLength))
        {
            iResult = 0;
        }
        else
        {
            TRACE(("WritePortExpander: r_RIIC0_TxData failed\r\n"));
        }
    }
    if (r_RIIC0_TxStop())
    {
        TRACE(("WritePortExpander: r_RIIC0_TxStop failed\r\n"));
        return -1;
    }
    taskSleep(1UL);
    return iResult;
}
/******************************************************************************
End of function  WritePortExpander
******************************************************************************/

/******************************************************************************
* Function Name: r_RIIC0_Open
* Description  : Function to open RIIC channel 0
* Arguments    : none
* Return Value : 0 for success or -1 on error
******************************************************************************/
int32_t r_RIIC0_Open(void)
{
    int32_t iResult = 0;
    volatile uint8_t byDummy = 0;

    UNUSED_VARIABLE(byDummy);

    /* Wait here for access to the hardware */
    eventWaitMutex(&gpRII0Mutex, EV_WAIT_INFINITE);

    /* Enable the clock to the module */
    rza_io_reg_write_8((uint8_t *)&CPG.STBCR9,
                      0,
                      CPG_STBCR9_MSTP97_SHIFT,
                      CPG_STBCR9_MSTP97);
    byDummy = CPG.STBCR9;

    /* Configure the pins */
    /* P1_0 for SCL0 function, 1st multiplex function, I/O control by peripheral function,
     * bidirectional mode enabled */
    PORT1.PIBCn.BIT.PIBCn0 = 0;
    PORT1.PBDCn.BIT.PBDCn0 = 1;
    PORT1.PMn.BIT.PMn0 = 1;
    PORT1.PMCn.BIT.PMCn0 = 0;
    PORT1.PIPCn.BIT.PIPCn0 = 1;
    PORT1.PBDCn.BIT.PBDCn0 = 1;
    PORT1.PFCn.BIT.PFCn0 = 0;
    PORT1.PFCEn.BIT.PFCEn0 = 0;
    PORT1.PFCAEn.BIT.PFCAEn0 = 0;
    PORT1.PIPCn.BIT.PIPCn0 = 1;
    PORT1.PMCn.BIT.PMCn0 = 1;

    /* P1_1 for SDA0 function, 1st multiplex function, I/O control by peripheral function,
     * bidirectional mode enabled */
    PORT1.PIBCn.BIT.PIBCn1 = 0;
    PORT1.PBDCn.BIT.PBDCn1 = 1;
    PORT1.PMn.BIT.PMn1 = 1;
    PORT1.PMCn.BIT.PMCn1 = 0;
    PORT1.PIPCn.BIT.PIPCn1 = 1;
    PORT1.PBDCn.BIT.PBDCn1 = 1;
    PORT1.PFCn.BIT.PFCn1 = 0;
    PORT1.PFCEn.BIT.PFCEn1 = 0;
    PORT1.PFCAEn.BIT.PFCAEn1 = 0;
    PORT1.PIPCn.BIT.PIPCn1 = 1;
    PORT1.PMCn.BIT.PMCn1 = 1;

    /* Open the driver */
    if (r_drvRIIC_Open(&gRIIC0DRV,
                       (volatile struct st_riic_n *)&RIIC0,
                       RIIC_CKS_DIVISION_8,
                       19U,
                       16U))
    {
        eventReleaseMutex(&gpRII0Mutex);
        iResult = -1;
    }
    else
    {
        /* Register interrupts */
        R_INTC_RegistIntFunc(INTC_ID_INTIICTEI0, (IFP)r_RIIC0_TEI);
        R_INTC_RegistIntFunc(INTC_ID_INTIICRI0, (IFP)r_RIIC0_RI);
        R_INTC_RegistIntFunc(INTC_ID_INTIICTI0, (IFP)r_RIIC0_TI);
        R_INTC_RegistIntFunc(INTC_ID_INTIICSPI0, (IFP)r_RIIC0_SPI);
        R_INTC_RegistIntFunc(INTC_ID_INTIICSTI0, (IFP)r_RIIC0_STI);
        R_INTC_RegistIntFunc(INTC_ID_INTIICNAKI0, (IFP)r_RIIC0_NAK);
        R_INTC_RegistIntFunc(INTC_ID_INTIICALI0, (IFP)r_RIIC0_AL);
        R_INTC_RegistIntFunc(INTC_ID_INTIICTMOI0, (IFP)r_RIIC0_TMO);

        /* Set interrupt priorities */
        R_INTC_SetPriority(INTC_ID_INTIICTEI0, RIIC0_INTERRUPT_PRIORITY);
        R_INTC_SetPriority(INTC_ID_INTIICRI0, RIIC0_INTERRUPT_PRIORITY);
        R_INTC_SetPriority(INTC_ID_INTIICTI0, RIIC0_INTERRUPT_PRIORITY);
        R_INTC_SetPriority(INTC_ID_INTIICSPI0, RIIC0_INTERRUPT_PRIORITY);
        R_INTC_SetPriority(INTC_ID_INTIICSTI0, RIIC0_INTERRUPT_PRIORITY);
        R_INTC_SetPriority(INTC_ID_INTIICNAKI0, RIIC0_INTERRUPT_PRIORITY);
        R_INTC_SetPriority(INTC_ID_INTIICALI0, RIIC0_INTERRUPT_PRIORITY);
        R_INTC_SetPriority(INTC_ID_INTIICTMOI0, RIIC0_INTERRUPT_PRIORITY);

        /* Enable interrupts */
        R_INTC_Enable(INTC_ID_INTIICTEI0);
        R_INTC_Enable(INTC_ID_INTIICRI0);
        R_INTC_Enable(INTC_ID_INTIICTI0);
        R_INTC_Enable(INTC_ID_INTIICSPI0);
        R_INTC_Enable(INTC_ID_INTIICSTI0);
        R_INTC_Enable(INTC_ID_INTIICNAKI0);
        R_INTC_Enable(INTC_ID_INTIICALI0);
        R_INTC_Enable(INTC_ID_INTIICTMOI0);

    }
    return iResult;
}
/******************************************************************************
End of function  r_RIIC0_Open
******************************************************************************/

/******************************************************************************
* Function Name: r_RIIC0_Close
* Description  : Function to close RIIC channel 0
* Arguments    : none
* Return Value : none
******************************************************************************/
void r_RIIC0_Close(void)
{
    volatile uint8_t byDummy = 0;

    UNUSED_VARIABLE(byDummy);

    /* Close the driver */
    r_drvRIIC_Close(&gRIIC0DRV);

    /* Disable interrupts */
    R_INTC_Disable(INTC_ID_INTIICTEI0);
    R_INTC_Disable(INTC_ID_INTIICRI0);
    R_INTC_Disable(INTC_ID_INTIICTI0);
    R_INTC_Disable(INTC_ID_INTIICSPI0);
    R_INTC_Disable(INTC_ID_INTIICSTI0);
    R_INTC_Disable(INTC_ID_INTIICNAKI0);
    R_INTC_Disable(INTC_ID_INTIICALI0);
    R_INTC_Disable(INTC_ID_INTIICTMOI0);

    /* Disable the clock to the module */
    rza_io_reg_write_8((uint8_t *)&CPG.STBCR9,
                      1,
                      CPG_STBCR9_MSTP97_SHIFT,
                      CPG_STBCR9_MSTP97);
    byDummy = CPG.STBCR9;

    /* Allow access to the hardware */
    eventReleaseMutex(&gpRII0Mutex);
}
/******************************************************************************
End of function  r_RIIC0_Close
******************************************************************************/

/******************************************************************************
* Function Name: r_RIIC0_TxStart
* Description  : Function to transmit the start condition
* Arguments    : none
* Return Value : 0 for success or -1 on error
******************************************************************************/
int32_t r_RIIC0_TxStart(void)
{
    return r_drvRIIC_TxStart(&gRIIC0DRV);
}
/******************************************************************************
End of function  r_RIIC0_TxStart
******************************************************************************/

/******************************************************************************
* Function Name: r_RIIC0_TxData
* Description  : Function to transmit data
* Arguments    : IN  pvData - Pointer to the data to transmit
*                IN  stLength - The length of data to transmit
* Return Value : 0 for success or -1 on error
******************************************************************************/
int32_t r_RIIC0_TxData(const void *pvData, size_t stLength)
{
    return r_drvRIIC_TxData(&gRIIC0DRV, pvData, stLength);
}
/******************************************************************************
End of function  r_RIIC0_TxData
******************************************************************************/

/******************************************************************************
* Function Name: r_RIIC0_RxData
* Description  : Function to receive data
* Arguments    : IN  pvData - Pointer to the data buffer
*                IN  stLength - The length of data to receive
* Return Value : 0 for success or -1 on error
******************************************************************************/
int32_t r_RIIC0_RxData(void *pvData, size_t stLength)
{
    return r_drvRIIC_RxData(&gRIIC0DRV, pvData, stLength);
}
/******************************************************************************
End of function  r_RIIC0_RxData
******************************************************************************/

/******************************************************************************
* Function Name: r_RIIC0_TxStop
* Description  : Function to send the stop condition
* Arguments    : none
* Return Value : 0 for success or -1 on error
******************************************************************************/
int32_t r_RIIC0_TxStop(void)
{
    return r_drvRIIC_TxStop(&gRIIC0DRV);
}
/******************************************************************************
End of function  r_RIIC0_TxStop
******************************************************************************/

/******************************************************************************
Private functions
******************************************************************************/

/******************************************************************************
* Function Name: r_RIIC0_TEI
* Description  : Interrupt handler for TEI
* Arguments    : none
* Return Value : none
******************************************************************************/
static void r_RIIC0_TEI(void)
{
    r_drvRIIC_TEI(&gRIIC0DRV);
}
/******************************************************************************
End of function  r_RIIC0_TEI
******************************************************************************/

/******************************************************************************
* Function Name: r_RIIC0_RI
* Description  : Interrupt handler for RI
* Arguments    : none
* Return Value : none
******************************************************************************/
static void r_RIIC0_RI(void)
{
    r_drvRIIC_RI(&gRIIC0DRV);
}
/******************************************************************************
End of function  r_RIIC0_RI
******************************************************************************/

/******************************************************************************
* Function Name: r_RIIC0_TI
* Description  : Interrupt handler for TI
* Arguments    : none
* Return Value : none
******************************************************************************/
static void r_RIIC0_TI(void)
{
    r_drvRIIC_TI(&gRIIC0DRV);
}
/******************************************************************************
End of function  r_RIIC0_TI
******************************************************************************/

/******************************************************************************
* Function Name: r_RIIC0_SPI
* Description  : Interrupt handler for SPI
* Arguments    : none
* Return Value : none
******************************************************************************/
static void r_RIIC0_SPI(void)
{
    r_drvRIIC_SPI(&gRIIC0DRV);
}
/******************************************************************************
End of function  r_RIIC0_SPI
******************************************************************************/

/******************************************************************************
* Function Name: r_RIIC0_STI
* Description  : Interrupt handler for STI
* Arguments    : none
* Return Value : none
******************************************************************************/
static void r_RIIC0_STI(void)
{
    r_drvRIIC_STI(&gRIIC0DRV);
}
/******************************************************************************
End of function  r_RIIC0_STI
******************************************************************************/

/******************************************************************************
* Function Name: r_RIIC0_NAK
* Description  : Interrupt handler for NAK
* Arguments    : none
* Return Value : none
******************************************************************************/
static void r_RIIC0_NAK(void)
{
    r_drvRIIC_NAK(&gRIIC0DRV);
}
/******************************************************************************
End of function  r_RIIC0_NAK
******************************************************************************/

/******************************************************************************
* Function Name: r_RIIC0_AL
* Description  : Interrupt handler for AL
* Arguments    : none
* Return Value : none
******************************************************************************/
static void r_RIIC0_AL(void)
{
    r_drvRIIC_AL(&gRIIC0DRV);
}
/******************************************************************************
End of function  r_RIIC0_AL
******************************************************************************/

/******************************************************************************
* Function Name: r_RIIC0_TMO
* Description  : Interrupt handler for TMO
* Arguments    : none
* Return Value : none
******************************************************************************/
static void r_RIIC0_TMO(void)
{
    r_drvRIIC_TMO(&gRIIC0DRV);
}
/******************************************************************************
End of function  r_RIIC0_TMO
******************************************************************************/

/******************************************************************************
 * Function Name: R_RIIC_CAMERA_write
 * Description  : Configuration register for Camera
******************************************************************************/
int32_t R_RIIC_CAMERA_write(uint32_t channel,
                               uint8_t d_adr,
                               uint16_t w_adr,
                               uint8_t data)
{
    int32_t  ret = 0xffffffff;
    uint8_t  buf[4];

    buf[0] = (uint8_t)(d_adr | SAMPLE_RIIC_RWCODE_W);
    buf[1] = (uint8_t)(w_adr  & 0x00ff);
    buf[2] = (uint8_t)(data  & 0x00ff);
    buf[3] = 0;

    if (0 == r_RIIC0_TxStart())
    {
        if (0 == r_RIIC0_TxData(buf, 3))
        {
            ret = 0;
        }
        else
        {
            TRACE(("R_RIIC_CAMERA_write: r_RIIC0_TxData failed\r\n"));
        }
    }
    if (r_RIIC0_TxStop())
    {
        TRACE(("R_RIIC_CAMERA_write: r_RIIC0_TxStop failed\r\n"));
        return -1;
    }
    taskSleep(1UL);

    return ret;
}
/*******************************************************************************
End of function R_RIIC_CAMERA_write
*******************************************************************************/

/******************************************************************************
 * Function Name: R_RIIC_HDMI_Enable
 * Description  : Enable HDMI module on RSK TFT APP BOARD via I2C
******************************************************************************/
int32_t R_RIIC_HDMI_Enable(void)
{
    uint8_t pbycommand[4];
    int32_t  ret = 0xffffffff;

    ret = r_RIIC0_Open();
    if((-1) != ret)
    {
        /* Write to the output register for IO1 */
        pbycommand[0] = (RZA1H_APP_I2C_TFP410 + SAMPLE_RIIC_RWCODE_W);
        pbycommand[1] = 0x08;
        pbycommand[2] = 0xbf;
        pbycommand[3] = 0;

        WritePortExpander(pbycommand, sizeof(pbycommand));

        r_RIIC0_Close();
    }
    return ret;
}

/*******************************************************************************
End of function R_RIIC_HDMI_Enable
*******************************************************************************/

/******************************************************************************
End  Of File
*******************************************************************************/
