/******************************************************************************
* DISCLAIMER

* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized.
* This software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES
* REGARDING THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY,
* INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
* PARTICULAR PURPOSE AND NON-INFRINGEMENT.  ALL SUCH WARRANTIES ARE EXPRESSLY
* DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES
* FOR ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS
* AFFILIATES HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this
* software and to discontinue the availability of this software.
* By using this software, you agree to the additional terms and
* conditions found by accessing the following link:
* http://www.renesas.com/disclaimer
******************************************************************************/
/* Copyright (C) 2011 Renesas Electronics Corporation. All rights reserved. */
/******************************************************************************
* File Name    : hwRIIC3.c
* Version      : 1.00
* Device       : RZA1
* Tool-Chain   : GNU
* H/W Platform : RSK2+RZA1H
* Description  : Hardware interface for RIIC3
******************************************************************************
* History : DD.MM.YYYY Version Description
*         : dd.mm.yyyy 1.00    First Release
******************************************************************************/

/******************************************************************************
Includes   <System Includes> , "Project Includes"
******************************************************************************/
#include <stdlib.h>
#include <stdio.h>

#include "hwRIIC3.h"
#include "drvRIIC.h"
#include "devdrv_intc.h"
#include "rza_io_regrw.h"
#include "trace.h"

/******************************************************************************
Defines
******************************************************************************/
#define RIIC3_INTERRUPT_PRIORITY            (25)


#define PX_CMD_READ_REG                     (0x00u)
#define PX_CMD_WRITE_OUT_REG                (0x01u)
#define PX_CMD_POLARITY_REG                 (0x02u)
#define PX_CMD_CONFIG_REG                   (0x03u)
#define PX_CMD_READ                         (1)
#define PX_CMD_WRITE                        (0)

#define RZA1H_RSK_I2C_PX_IO1                (0x40)
#define RZA1H_RSK_I2C_PX_IO2                (0x42)

/******************************************************************************
Constant Macros
******************************************************************************/

/******************************************************************************
Function Macros
******************************************************************************/

/******************************************************************************
Typedef definitions
******************************************************************************/

typedef void (*IFP)(uint32_t);

/******************************************************************************
Enumerated Types
******************************************************************************/

/******************************************************************************
Macro definitions
******************************************************************************/

/*****************************************************************************
Function Prototypes
******************************************************************************/

static void r_RIIC3_TEI(void);
static void r_RIIC3_RI(void);
static void r_RIIC3_TI(void);
static void r_RIIC3_SPI(void);
static void r_RIIC3_STI(void);
static void r_RIIC3_NAK(void);
static void r_RIIC3_AL(void);
static void r_RIIC3_TMO(void);

/******************************************************************************
Constant Data
******************************************************************************/

/******************************************************************************
Imported global variables and functions (from other files)
******************************************************************************/

/******************************************************************************
Private global variables
******************************************************************************/

/* The memory required by the RIIC device driver */
static RIICDRV  gRIIC3DRV;

/* A mutex event to protect from access by multiple tasks */
static PEVENT   gpRII3Mutex = NULL;

static uint8_t byPort1Map = 0;
static uint8_t byPort2Map = 0;

/*****************************************************************************
Exported global variables and functions (to be accessed by other files)
*****************************************************************************/

/******************************************************************************
* Function Name: write_port_expander
* Description  : Function to write to the port expander
* Arguments    : IN  pbyCommand - Pointer to the command to write
*                IN  stLength - The length of the command
* Return Value : 0 for success or -1 on error
******************************************************************************/
static int32_t write_port_expander(uint8_t *pbyCommand, size_t stLength)
{
    int32_t iresult = -1;
    if (0 == r_RIIC3_TxStart())
    {
        if (0 == r_RIIC3_TxData(pbyCommand, stLength))
        {
            iresult = 0;
        }
        else
        {
            printf(("WritePortExpander: r_RIIC3_TxData failed\r\n"));
        }
    }
    if (r_RIIC3_TxStop())
    {
        printf(("WritePortExpander: r_RIIC3TxStop failed\r\n"));
        return -1;
    }
    taskSleep(1UL);
    return iresult;
}
/******************************************************************************
End of function  write_port_expander
******************************************************************************/

/******************************************************************************
* Function Name: r_RIIC3_Open
* Description  : Function to open RIIC channel 3
* Arguments    : none
* Return Value : 0 for success or -1 on error
******************************************************************************/
int32_t r_RIIC3_Open(void)
{
    int32_t iResult = 0;
    volatile uint8_t byDummy = 0;

    UNUSED_VARIABLE(byDummy);

    /* Wait here for access to the hardware */
    eventWaitMutex(&gpRII3Mutex, EV_WAIT_INFINITE);

    /* Enable the clock to the module */
    rza_io_reg_write_8((uint8_t *)&CPG.STBCR9,
                      0,
                      CPG_STBCR9_MSTP94_SHIFT,
                      CPG_STBCR9_MSTP94);
    byDummy = CPG.STBCR9;

    /* ---- P1_6 : SCL3 ---- */
    /* Port initialise */
    rza_io_reg_write_16(&GPIO.PIBC1,  0, GPIO_PIBC1_PIBC16_SHIFT,   GPIO_PIBC1_PIBC16);
    rza_io_reg_write_16(&GPIO.PBDC1,  1, GPIO_PBDC1_PBDC16_SHIFT,   GPIO_PBDC1_PBDC16);
    rza_io_reg_write_16(&GPIO.PM1,    1, GPIO_PM1_PM16_SHIFT,       GPIO_PM1_PM16);
    rza_io_reg_write_16(&GPIO.PMC1,   0, GPIO_PMC1_PMC16_SHIFT,     GPIO_PMC1_PMC16);
    rza_io_reg_write_16(&GPIO.PIPC1,  1, GPIO_PIPC1_PIPC16_SHIFT,   GPIO_PIPC1_PIPC16);

    /* Port mode : Multiplex mode                     */
    /* Port function setting : 1st multiplex function */
    /* I/O control mode : Peripheral function         */
    /* Bidirectional mode : Enable                    */
    rza_io_reg_write_16(&GPIO.PBDC1,  1, GPIO_PBDC1_PBDC16_SHIFT,   GPIO_PBDC1_PBDC16);
    rza_io_reg_write_16(&GPIO.PFC1,   0, GPIO_PFC1_PFC16_SHIFT,     GPIO_PFC1_PFC16);
    rza_io_reg_write_16(&GPIO.PFCE1,  0, GPIO_PFCE1_PFCE16_SHIFT,   GPIO_PFCE1_PFCE16);
    rza_io_reg_write_16(&GPIO.PFCAE1, 0, GPIO_PFCAE1_PFCAE16_SHIFT, GPIO_PFCAE1_PFCAE16);
    rza_io_reg_write_16(&GPIO.PIPC1,  1, GPIO_PIPC1_PIPC16_SHIFT,   GPIO_PIPC1_PIPC16);
    rza_io_reg_write_16(&GPIO.PMC1,   1, GPIO_PMC1_PMC16_SHIFT,     GPIO_PMC1_PMC16);

    /* ---- P1_7 : SDA3 ---- */
    /* Port initialise */
    rza_io_reg_write_16(&GPIO.PIBC1,  0, GPIO_PIBC1_PIBC17_SHIFT,   GPIO_PIBC1_PIBC17);
    rza_io_reg_write_16(&GPIO.PBDC1,  1, GPIO_PBDC1_PBDC17_SHIFT,   GPIO_PBDC1_PBDC17);
    rza_io_reg_write_16(&GPIO.PM1,    1, GPIO_PM1_PM17_SHIFT,       GPIO_PM1_PM17);
    rza_io_reg_write_16(&GPIO.PMC1,   0, GPIO_PMC1_PMC17_SHIFT,     GPIO_PMC1_PMC17);
    rza_io_reg_write_16(&GPIO.PIPC1,  1, GPIO_PIPC1_PIPC17_SHIFT,   GPIO_PIPC1_PIPC17);

    /* Port mode : Multiplex mode                     */
    /* Port function setting : 1st multiplex function */
    /* I/O control mode : Peripheral function         */
    /* Bidirectional mode : Enable                    */
    rza_io_reg_write_16(&GPIO.PBDC1,  1, GPIO_PBDC1_PBDC17_SHIFT,   GPIO_PBDC1_PBDC17);
    rza_io_reg_write_16(&GPIO.PFC1,   0, GPIO_PFC1_PFC17_SHIFT,     GPIO_PFC1_PFC17);
    rza_io_reg_write_16(&GPIO.PFCE1,  0, GPIO_PFCE1_PFCE17_SHIFT,   GPIO_PFCE1_PFCE17);
    rza_io_reg_write_16(&GPIO.PFCAE1, 0, GPIO_PFCAE1_PFCAE17_SHIFT, GPIO_PFCAE1_PFCAE17);
    rza_io_reg_write_16(&GPIO.PIPC1,  1, GPIO_PIPC1_PIPC17_SHIFT,   GPIO_PIPC1_PIPC17);
    rza_io_reg_write_16(&GPIO.PMC1,   1, GPIO_PMC1_PMC17_SHIFT,     GPIO_PMC1_PMC17);

    /* Open the driver */
    if (r_drvRIIC_Open(&gRIIC3DRV,
                       (volatile struct st_riic_n *)&RIIC3,
                       RIIC_CKS_DIVISION_64,
                       19U,
                       16U))
    {
        eventReleaseMutex(&gpRII3Mutex);
        iResult = -1;
    }
    else
    {
        /* Register interrupts */
        R_INTC_RegistIntFunc(INTC_ID_INTIICTEI3, (IFP)r_RIIC3_TEI);
        R_INTC_RegistIntFunc(INTC_ID_INTIICRI3, (IFP)r_RIIC3_RI);
        R_INTC_RegistIntFunc(INTC_ID_INTIICTI3, (IFP)r_RIIC3_TI);
        R_INTC_RegistIntFunc(INTC_ID_INTIICSPI3, (IFP)r_RIIC3_SPI);
        R_INTC_RegistIntFunc(INTC_ID_INTIICSTI3, (IFP)r_RIIC3_STI);
        R_INTC_RegistIntFunc(INTC_ID_INTIICNAKI3, (IFP)r_RIIC3_NAK);
        R_INTC_RegistIntFunc(INTC_ID_INTIICALI3, (IFP)r_RIIC3_AL);
        R_INTC_RegistIntFunc(INTC_ID_INTIICTMOI3, (IFP)r_RIIC3_TMO);

        /* Set interrupt priorities */
        R_INTC_SetPriority(INTC_ID_INTIICTEI3, RIIC3_INTERRUPT_PRIORITY);
        R_INTC_SetPriority(INTC_ID_INTIICRI3, RIIC3_INTERRUPT_PRIORITY);
        R_INTC_SetPriority(INTC_ID_INTIICTI3, RIIC3_INTERRUPT_PRIORITY);
        R_INTC_SetPriority(INTC_ID_INTIICSPI3, RIIC3_INTERRUPT_PRIORITY);
        R_INTC_SetPriority(INTC_ID_INTIICSTI3, RIIC3_INTERRUPT_PRIORITY);
        R_INTC_SetPriority(INTC_ID_INTIICNAKI3, RIIC3_INTERRUPT_PRIORITY);
        R_INTC_SetPriority(INTC_ID_INTIICALI3, RIIC3_INTERRUPT_PRIORITY);
        R_INTC_SetPriority(INTC_ID_INTIICTMOI3, RIIC3_INTERRUPT_PRIORITY);

        /* Enable interrupts */
        R_INTC_Enable(INTC_ID_INTIICTEI3);
        R_INTC_Enable(INTC_ID_INTIICRI3);
        R_INTC_Enable(INTC_ID_INTIICTI3);
        R_INTC_Enable(INTC_ID_INTIICSPI3);
        R_INTC_Enable(INTC_ID_INTIICSTI3);
        R_INTC_Enable(INTC_ID_INTIICNAKI3);
        R_INTC_Enable(INTC_ID_INTIICALI3);
        R_INTC_Enable(INTC_ID_INTIICTMOI3);
    }
    return iResult;
}
/******************************************************************************
End of function  r_RIIC3_Open
******************************************************************************/

/******************************************************************************
* Function Name: r_RIIC3_Close
* Description  : Function to close RIIC channel 3
* Arguments    : none
* Return Value : none
******************************************************************************/
void r_RIIC3_Close(void)
{
    volatile uint8_t byDummy = 0;

    UNUSED_VARIABLE(byDummy);

    /* Close the driver */
    r_drvRIIC_Close(&gRIIC3DRV);

    /* Disable interrupts */
    R_INTC_Disable(INTC_ID_INTIICTEI3);
    R_INTC_Disable(INTC_ID_INTIICRI3);
    R_INTC_Disable(INTC_ID_INTIICTI3);
    R_INTC_Disable(INTC_ID_INTIICSPI3);
    R_INTC_Disable(INTC_ID_INTIICSTI3);
    R_INTC_Disable(INTC_ID_INTIICNAKI3);
    R_INTC_Disable(INTC_ID_INTIICALI3);
    R_INTC_Disable(INTC_ID_INTIICTMOI3);

    /* Disable the clock to the module */
    rza_io_reg_write_8((uint8_t *)&CPG.STBCR9,
                      1,
                      CPG_STBCR9_MSTP94_SHIFT,
                      CPG_STBCR9_MSTP94);
    byDummy = CPG.STBCR9;

    /* Allow access to the hardware */
    eventReleaseMutex(&gpRII3Mutex);
}
/******************************************************************************
End of function  r_RIIC3_Close
******************************************************************************/

/******************************************************************************
* Function Name: R_RIIC3_UpdateState_PE1
* Description  : Function to modify output state of Port Expander 1
*                on RIIC channel 3
* Arguments    : IN  mask - which pin(s) to modify
*                IN  state - new state for pin(s)
* Return Value : none
******************************************************************************/
void R_RIIC3_UpdateState_PE1(uint8_t mask, uint8_t state)
{
    byPort1Map &=  (~mask);
    byPort1Map |= state;

    {
        uint8_t pbycommand[3];

        /* Write to the output register for IO1 */
        pbycommand[0] = (RZA1H_RSK_I2C_PX_IO1 + PX_CMD_WRITE);
        pbycommand[1] = PX_CMD_WRITE_OUT_REG;
        pbycommand[2] = byPort1Map;
        write_port_expander(pbycommand, sizeof(pbycommand));
    }

}
/******************************************************************************
End of function  R_RIIC3_UpdateState_PE1
******************************************************************************/

/******************************************************************************
* Function Name: R_RIIC3_UpdateState_PE2
* Description  : Function to modify output state of Port Expander 2
*                on RIIC channel 3
* Arguments    : IN  mask - which pin(s) to modify
*                IN  state - new state for pin(s)
* Return Value : none
******************************************************************************/
void R_RIIC3_UpdateState_PE2(uint8_t mask, uint8_t state)
{
    byPort2Map &=  (~mask);
    byPort2Map |= state;

    {
        uint8_t pbycommand[3];

        /* Write to the output register for IO2 */
        pbycommand[0] = (RZA1H_RSK_I2C_PX_IO2 + PX_CMD_WRITE);
        pbycommand[1] = PX_CMD_WRITE_OUT_REG;
        pbycommand[2] = byPort2Map;
        write_port_expander(pbycommand, sizeof(pbycommand));
    }

}
/******************************************************************************
End of function  R_RIIC3_UpdateState_PE2
******************************************************************************/

/******************************************************************************
* Function Name: r_RIIC3_Initialise
* Description  : Function to initialise i2c channel with default port config
* Arguments    : none
* Return Value : 0 for success or -1 on error
******************************************************************************/

void r_RIIC3_Initialise(void)
{
    if(0 == r_RIIC3_Open())
    {
        uint8_t pbycommand[3];

        /* ---- Configure IO Port Expander 1 Setup ---- */
           byPort1Map = (uint8_t)(0xB7);  /* Set bits as per the table below */
        /* Indicates which I/O is to be modified
        IO[0] = LED1                   = 1 (OFF)
        IO[1] = LED2                   = 1 (OFF)
        IO[2] = LED3                   = 1 (OFF)
        IO[3] = NOR A25                = 0
        IO[4] = PMOD1 Reset            = 1
        IO[5] = PMOD2 Reset            = 1
        IO[6] = SD Header Power Enable = 0 (OFF)
        IO[7] = SD/MMC Power Enable    = 1 (ON) */

        /* Write to the output register for IO1 */
        pbycommand[0] = (RZA1H_RSK_I2C_PX_IO1 + PX_CMD_WRITE);
        pbycommand[1] = PX_CMD_WRITE_OUT_REG;
        pbycommand[2] = byPort1Map;
        write_port_expander(pbycommand, sizeof(pbycommand));

        /* ---- START Port Expander 1 Port Direction Setup ---- */
        /* Modifying all bits of the register */
        /* Set bits as per the table below */
        /*
         * IO[0] = LED1             (output - 0)
         * IO[1] = LED2             (output - 0)
         * IO[2] = LED3             (output - 0)
         * IO[3] = NOR_A25             (output - 0)
         * IO[4] = PMOD1_RST         (output - 0)
         * IO[5] = PMOD2_RST         (output - 0)
         * IO[6] = SD_CONN_PWR_EN     (output - 0)
         * IO[7] = SD_MMC_PWR_EN     (output - 0)
         */
        pbycommand[0] = (RZA1H_RSK_I2C_PX_IO1 + PX_CMD_WRITE);
        pbycommand[1] = PX_CMD_CONFIG_REG;
        pbycommand[2] = 0x00;
        write_port_expander(pbycommand, sizeof(pbycommand));


        /* ---- Configure IO Port Expander 2 Setup ---- */
           byPort2Map = (uint8_t)(0x11);  /* Set bits as per the table below */
        /*
        IO[0] = PX1_EN0 Enable  = 1 (LCD/DV signal select)
        IO[1] = PX1_EN1 Enable  = 0 (OFF)
        IO[2] = TFT_CS          = 0 (Optional SPI chip select for external TFTs)
        IO[3] = PX1_EN3 Enable  = 0 (SSIF/PWM signal select.)
        IO[4] = USB_OVR_CURRENT = 1 (USB over-current/thermal shutdown conditions.)
        IO[5] = USB_PWR_ENA     = 0 (VBUS0 power control. JP11 needs to be shorted)
        IO[6] = USB_PWR_ENB     = 0 (VBUS1 power control. JP12 needs to be shorted)
        IO[7] = PX1_EN7         = 0 (A18-A21 and SGOUTx signals select.)
        */

        /* Write to the output register for IO2 */
        pbycommand[0] = (RZA1H_RSK_I2C_PX_IO2 + PX_CMD_WRITE);
        pbycommand[1] = PX_CMD_WRITE_OUT_REG;
        pbycommand[2] = byPort2Map;
        write_port_expander(pbycommand, sizeof(pbycommand));

        /* ---- START Port Expander 2 Port Direction Setup ---- */
        /* Modifying all bits of the register */
        /* Set bits as per the table below */
        /*
         * IO[0] = PX1_EN0             (output - 0)
         * IO[1] = PX1_EN1             (output - 0)
         * IO[2] = TFT_CS             (output - 0)
         * IO[3] = PX1_EN3             (output - 0)
         * IO[4] = USB_OVR_CURRENT     (input - 1)
         * IO[5] = USB_PWR_ENA         (output - 0)
         * IO[6] = USB_PWR_ENB         (output - 0)
         * IO[7] = PX1_EN7             (output - 0)
        */
        pbycommand[0] = (RZA1H_RSK_I2C_PX_IO2 + PX_CMD_WRITE);
        pbycommand[1] = PX_CMD_CONFIG_REG;
        pbycommand[2] = 0x10;
        write_port_expander(pbycommand, sizeof(pbycommand));

        r_RIIC3_Close();
    }
}
/******************************************************************************
End of function  r_RIIC3_Initialise
******************************************************************************/

/******************************************************************************
* Function Name: r_RIIC3_TxStart
* Description  : Function to transmit the start condition
* Arguments    : none
* Return Value : 0 for success or -1 on error
******************************************************************************/
int32_t r_RIIC3_TxStart(void)
{
    return r_drvRIIC_TxStart(&gRIIC3DRV);
}
/******************************************************************************
End of function  r_RIIC3_TxStart
******************************************************************************/

/******************************************************************************
* Function Name: r_RIIC3_TxData
* Description  : Function to transmit data
* Arguments    : IN  pvData - Pointer to the data to transmit
*                IN  stLength - The length of data to transmit
* Return Value : 0 for success or -1 on error
******************************************************************************/
int32_t r_RIIC3_TxData(const void *pvData, size_t stLength)
{
    return r_drvRIIC_TxData(&gRIIC3DRV, pvData, stLength);
}
/******************************************************************************
End of function  r_RIIC3_TxData
******************************************************************************/

/******************************************************************************
* Function Name: r_RIIC3_RxData
* Description  : Function to receive data
* Arguments    : IN  pvData - Pointer to the data buffer
*                IN  stLength - The length of data to receive
* Return Value : 0 for success or -1 on error
******************************************************************************/
int32_t r_RIIC3_RxData(void *pvData, size_t stLength)
{
    return r_drvRIIC_RxData(&gRIIC3DRV, pvData, stLength);
}
/******************************************************************************
End of function  r_RIIC3_RxData
******************************************************************************/

/******************************************************************************
* Function Name: r_RIIC3_TxStop
* Description  : Function to send the stop condition
* Arguments    : none
* Return Value : 0 for success or -1 on error
******************************************************************************/
int32_t r_RIIC3_TxStop(void)
{
    return r_drvRIIC_TxStop(&gRIIC3DRV);
}
/******************************************************************************
End of function  r_RIIC3TxStop
******************************************************************************/

/******************************************************************************
Private functions
******************************************************************************/

/******************************************************************************
* Function Name: r_RIIC3_TEI
* Description  : Interrupt handler for TEI
* Arguments    : none
* Return Value : none
******************************************************************************/
static void r_RIIC3_TEI(void)
{
    r_drvRIIC_TEI(&gRIIC3DRV);
}
/******************************************************************************
End of function  r_RIIC3_TEI
******************************************************************************/

/******************************************************************************
* Function Name: r_RIIC3_RI
* Description  : Interrupt handler for RI
* Arguments    : none
* Return Value : none
******************************************************************************/
static void r_RIIC3_RI(void)
{
    r_drvRIIC_RI(&gRIIC3DRV);
}
/******************************************************************************
End of function  r_RIIC3_RI
******************************************************************************/

/******************************************************************************
* Function Name: r_RIIC3_TI
* Description  : Interrupt handler for TI
* Arguments    : none
* Return Value : none
******************************************************************************/
static void r_RIIC3_TI(void)
{
    r_drvRIIC_TI(&gRIIC3DRV);
}
/******************************************************************************
End of function  r_RIIC3_TI
******************************************************************************/

/******************************************************************************
* Function Name: r_RIIC3_SPI
* Description  : Interrupt handler for SPI
* Arguments    : none
* Return Value : none
******************************************************************************/
static void r_RIIC3_SPI(void)
{
    r_drvRIIC_SPI(&gRIIC3DRV);
}
/******************************************************************************
End of function  r_RIIC3_SPI
******************************************************************************/

/******************************************************************************
* Function Name: r_RIIC3_STI
* Description  : Interrupt handler for STI
* Arguments    : none
* Return Value : none
******************************************************************************/
static void r_RIIC3_STI(void)
{
    r_drvRIIC_STI(&gRIIC3DRV);
}
/******************************************************************************
End of function  r_RIIC3_STI
******************************************************************************/

/******************************************************************************
* Function Name: r_RIIC3_NAK
* Description  : Interrupt handler for NAK
* Arguments    : none
* Return Value : none
******************************************************************************/
static void r_RIIC3_NAK(void)
{
    r_drvRIIC_NAK(&gRIIC3DRV);
}
/******************************************************************************
End of function  r_RIIC3_NAK
******************************************************************************/

/******************************************************************************
* Function Name: r_RIIC3_AL
* Description  : Interrupt handler for AL
* Arguments    : none
* Return Value : none
******************************************************************************/
static void r_RIIC3_AL(void)
{
    r_drvRIIC_AL(&gRIIC3DRV);
}
/******************************************************************************
End of function  r_RIIC3_AL
******************************************************************************/

/******************************************************************************
* Function Name: r_RIIC3_TMO
* Description  : Interrupt handler for TMO
* Arguments    : none
* Return Value : none
******************************************************************************/
static void r_RIIC3_TMO(void)
{
    r_drvRIIC_TMO(&gRIIC3DRV);
}
/******************************************************************************
End of function  r_RIIC3_TMO
******************************************************************************/

/******************************************************************************
End  Of File
*******************************************************************************/
