/******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
*******************************************************************************
* Copyright (C) 2012 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************
* File Name    : hwscif2.c
* Version      : 1.00
* Device(s)    : RZA1
* Tool-Chain   : GNU
* OS           : FreeRTOS
* H/W Platform : RSK+RZA1H
* Description  : Simple sample device driver for a SCIF module
******************************************************************************
* History      : DD.MM.YYYY Ver. Description
*              : 01.08.2009 1.00 First Release
******************************************************************************/

/******************************************************************************
  WARNING!  IN ACCORDANCE WITH THE USER LICENCE THIS CODE MUST NOT BE CONVEYED
  OR REDISTRIBUTED IN COMBINATION WITH ANY SOFTWARE LICENSED UNDER TERMS THE
  SAME AS OR SIMILAR TO THE GNU GENERAL PUBLIC LICENCE
******************************************************************************/

/******************************************************************************
Includes   <System Includes> , "Project Includes"
******************************************************************************/
#include <stdio.h>
#include <string.h>

/* Ensure local malloc is used */
#include "..\inc\malloc.h"
//#include "malloc.h"

#include "devlink.h"
#include "drvScif.h"
#include "iodefine.h"
#include "devdrv_intc.h"
#include "FreeRTOSConfig.h"
#include "r_sysIf.h"
#include "rza_io_regrw.h"

//TODO: Include interrupt priority
//#include "interruptPriority.h"

/******************************************************************************
Defines
******************************************************************************/

/* Define the base pointer of the SCIF to use */
#define P_SCI_BASE                  SCIF2
/* Define the size of the software FIFOs */
#define SCIF_RX_SOFTWARE_FIFO_SIZE  1024UL
#define SCIF_TX_SOFTWARE_FIFO_SIZE  65536UL
/* Define the interrupt priority */
#define SCIF_INTERRUPT_PRIORITY     26//SERIAL_INTERUPT_PRIORITY
#define SCIF_INTERRUPT_DISABLED     0x00

/******************************************************************************
Function Prototypes
******************************************************************************/

static int scif2Open(PSTREAM pStream);
static void scif2Close(PSTREAM pStream);
static int scif2Read(PSTREAM pStream, uint8_t *pbyBuffer, uint32_t uiCount);
static int scif2Write(PSTREAM pStream, uint8_t *pbyBuffer, uint32_t uiCount);
static int scif2Control(PSTREAM pStream, CTLCODE ctlCode, void *pCtlStruct);
static void sciSetIRQ(_Bool bfEnable);
static void INT_SCIF2_BRI(void);
static void INT_SCIF2_ERI(void);
static void INT_SCIF2_RXI(void);
static void INT_SCIF2_TXI(void);

/******************************************************************************
Constant Data
******************************************************************************/

/* Define the driver function table for this device */
const DEVICE gScif2Driver =
{
    "SCIF2 Device Driver",
    scif2Open,
    scif2Close,
    scif2Read,
    scif2Write,
    scif2Control
};

/* Define the default configuration of this device */
static const SCIFCFG gScifDefaultConfig =
{
    /* Baud rate */
    115200UL,

    /* Line Coding - asynchronous with no flow control (N,8,1) */
    (SCI_PARITY_NONE | SCI_DATA_BITS_EIGHT | SCI_ONE_STOP_BIT),

    /* Peripheral clock frequency */
    configPERIPHERAL_CLOCK1_HZ,

    /* Pointer to interrupt control function */
    sciSetIRQ
};

/******************************************************************************
Global Variables
******************************************************************************/
/* The data structure required by the lower level driver */
static DDSCIF  gDDSCIF;

/* A reference counter for open and close functions */
static int giRefCount = 0;

/******************************************************************************
Public Functions
******************************************************************************/

/******************************************************************************
Function Name: scifOutputDebugString
Description:   Function to write a debug string (for TRACE((""));)
Arguments:     IN  pbyBuffer - Pointer to the source memory
               IN  uiCount - The number of bytes to write
Return value:  0 for success -1 on error
******************************************************************************/
int scifOutputDebugString(uint8_t *pbyBuffer, uint32_t uiCount)
{
    if (giRefCount)
    {
        cbPutPacket(gDDSCIF.pTxBuffer, (size_t)uiCount, pbyBuffer);
        cbCheckIn(gDDSCIF.pTxBuffer, (size_t)uiCount);

        /* Enable transmit interrupt */
        gDDSCIF.pPORT->SCSCR.BIT.TIE = 1;
        return 0;
    }
    return -1;
}
/******************************************************************************
End of function  scifOutputDebugString
******************************************************************************/

/******************************************************************************
Private Functions
******************************************************************************/

/******************************************************************************
Function Name: scif2Open
Description:   Function to open SCIF2
Arguments:     IN  pStream - Pointer to the file stream
Return value:  0 for success otherwise -1
******************************************************************************/
static int scif2Open(PSTREAM pStream)
{
    PCBUFF  pRxBuffer, pTxBuffer;
    volatile unsigned char dummy = 0;

    UNUSED_VARIABLE(dummy);

    /* File stream is not used */
    (void)pStream;
    /* Check to see if it is already open */
    if (giRefCount)
    {
        giRefCount++;
        return 0;
    }
    /* Get the interrupt vectors */
    R_INTC_RegistIntFunc(INTC_ID_BRI2, (void(*)(uint32_t))INT_SCIF2_BRI);
    R_INTC_RegistIntFunc(INTC_ID_ERI2, (void(*)(uint32_t))INT_SCIF2_ERI);
    R_INTC_RegistIntFunc(INTC_ID_RXI2, (void(*)(uint32_t))INT_SCIF2_RXI);
    R_INTC_RegistIntFunc(INTC_ID_TXI2, (void(*)(uint32_t))INT_SCIF2_TXI);
    /* Set the interrupt priority */
    R_INTC_SetPriority(INTC_ID_BRI2, SCIF_INTERRUPT_PRIORITY);
    R_INTC_SetPriority(INTC_ID_ERI2, SCIF_INTERRUPT_PRIORITY);
    R_INTC_SetPriority(INTC_ID_RXI2, SCIF_INTERRUPT_PRIORITY);
    R_INTC_SetPriority(INTC_ID_TXI2, SCIF_INTERRUPT_PRIORITY);
    /* Create the software FIFOs */
    pRxBuffer = cbCreate(HEAP_SRAM, SCIF_RX_SOFTWARE_FIFO_SIZE);
    if (!pRxBuffer)
    {
        return -1;
    }
    pTxBuffer = cbCreate(HEAP_SRAM, SCIF_TX_SOFTWARE_FIFO_SIZE);
    if (!pTxBuffer)
    {
        cbDestroy(HEAP_SRAM, gDDSCIF.pRxBuffer);
        return -1;
    }
    /* Configure the port pins for use by SCIF2 */
    /* TXD2 */
    PORT3.PMCn.BIT.PMCn0     = 1;
    PORT3.PFCAEn.BIT.PFCAEn0 = 1;
    PORT3.PFCEn.BIT.PFCEn0   = 0;
    PORT3.PFCn.BIT.PFCn0     = 1;
    PORT3.PIPCn.BIT.PIPCn0   = 1;
    /* RXD2 */
    PORT3.PMCn.BIT.PMCn2     = 1;
    PORT3.PFCAEn.BIT.PFCAEn2 = 0;
    PORT3.PFCEn.BIT.PFCEn2   = 1;
    PORT3.PFCn.BIT.PFCn2     = 1;
    PORT3.PIPCn.BIT.PIPCn2   = 1;

    /* Enable scif2 module */
    /* Supply clock to the SCIF(channel 2) */
    rza_io_reg_write_8((uint8_t *)&CPG.STBCR4,
                      0,
                      CPG_STBCR4_MSTP45_SHIFT,
                      CPG_STBCR4_MSTP45);
    dummy = CPG.STBCR4;

    /* Start the lower level driver */
    if (sciOpen(&gDDSCIF, &P_SCI_BASE, &gScifDefaultConfig, pRxBuffer, pTxBuffer))
    {
        /* Destroy the software FIFOS */
        cbDestroy(HEAP_SRAM, pTxBuffer);
        cbDestroy(HEAP_SRAM, pRxBuffer);
        return -1;
    }
    /* Increment the reference count */
    giRefCount++;
    return 0;
}
/******************************************************************************
End of function scif2Open
******************************************************************************/

/******************************************************************************
Function Name: scif2Close
Description:   Function to close the SCIF
Arguments:     IN  pStream - Pointer to the file stream
Return value:  none
******************************************************************************/
static void scif2Close(PSTREAM pStream)
{
    volatile unsigned char dummy = 0;

    UNUSED_VARIABLE(dummy);

    /* File stream is not used */
    (void)pStream;
    /* Decrement the reference counter */
    giRefCount--;
    if (!giRefCount)
    {
        /* Stop the SCIF driver */
        sciClose(&gDDSCIF);
        /* Set pins back to default function */
        //TODO: Set Sci pins back to port pins
        /* Revert the interrupt priority */
        R_INTC_SetPriority(INTC_ID_BRI2, SCIF_INTERRUPT_DISABLED);
        R_INTC_SetPriority(INTC_ID_ERI2, SCIF_INTERRUPT_DISABLED);
        R_INTC_SetPriority(INTC_ID_RXI2, SCIF_INTERRUPT_DISABLED);
        R_INTC_SetPriority(INTC_ID_TXI2, SCIF_INTERRUPT_DISABLED);
        /* Destroy the software FIFOS */
        cbDestroy(HEAP_SRAM, gDDSCIF.pTxBuffer);
        cbDestroy(HEAP_SRAM, gDDSCIF.pRxBuffer);
        /* Vectors not released as there is no API */

        /* Disable scif2 module */
        /* Supply clock to the SCIF(channel 2) */
        rza_io_reg_write_8((uint8_t *)&CPG.STBCR4,
                          1,
                          CPG_STBCR4_MSTP45_SHIFT,
                          CPG_STBCR4_MSTP45);
        dummy = CPG.STBCR4;
    }
}
/******************************************************************************
End of function scif2Close
******************************************************************************/

/******************************************************************************
Function Name: scif2Read
Description:   Function to read data from the serial port
Arguments:     IN  pStream - Pointer to the file stream
               IN  pbyBuffer - Pointer to the destination memory
               IN  uiCount - The number of bytes to read
Return value:  data count for success or Error Code on error
******************************************************************************/
static int scif2Read(PSTREAM pStream, uint8_t *pbyBuffer, uint32_t uiCount)
{
    /* File stream is not used */
    (void)pStream;
    /* Check to make sure that the SCIF has been opened */
    if (giRefCount)
    {
        /* Call the driver read function */
        if (!sciReadData(&gDDSCIF, pbyBuffer, (size_t)uiCount))
        {
            return (int)uiCount;
        }
    }
    return -1;
}
/******************************************************************************
End of function scif2Read
******************************************************************************/

/******************************************************************************
Function Name: scif2Write
Description:   Function to read data from the serial port
Arguments:     IN  pStream - Pointer to the file stream
               IN  pbyBuffer - Pointer to the source memory
               IN  uiCount - The number of bytes to write
Return value:  data count for success or Error Code on error
******************************************************************************/
static int scif2Write(PSTREAM pStream, uint8_t *pbyBuffer, uint32_t uiCount)
{
    /* File stream is not used */
    (void)pStream;
    /* Check to make sure that the SCIF has been opened */
    if (giRefCount)
    {
        static void *pvMutexLock = NULL;
        int    iMask = sysLock(&pvMutexLock);
        sciWriteData(&gDDSCIF, pbyBuffer, (size_t)uiCount);
        sysUnlock(&pvMutexLock, iMask);
        return uiCount;
    }
    return -1;
}
/******************************************************************************
End of function scif2Write
******************************************************************************/

/******************************************************************************
Function Name: scif2Control
Description:   Function to handle custom control functions for SCIF
Arguments:     IN  pStream - Pointer to the file stream
               IN  ctlCode - The custom control code
               IN  pCtlStruct - Pointer to the custom control structure
Return value:  0 for success -1 or error code on error
******************************************************************************/
static int scif2Control(PSTREAM pStream, CTLCODE ctlCode, void *pCtlStruct)
{
    /* File stream is not used */
    (void)pStream;
    if (giRefCount)
    {
        if  ((ctlCode == CTL_SCI_SET_CONFIGURATION)
        &&  (pCtlStruct))
        {
            PSCICFG  pSciCfg = (PSCICFG)pCtlStruct;
            /* Try to re-configure */
            if (sciReConfigure(&gDDSCIF, pSciCfg->dwBaud, pSciCfg->dwConfig))
            {
                /* If it fails set back to default */
                sciOpen(&gDDSCIF,
                        &P_SCI_BASE,
                        &gScifDefaultConfig,
                        gDDSCIF.pRxBuffer,
                        gDDSCIF.pTxBuffer);
                return -1;
            }
            return 0;
        }
        else if (CTL_GET_RX_BUFFER_COUNT == ctlCode)
        {
            return (int)cbUsed(gDDSCIF.pRxBuffer);
        }
        return -1;
    }
    return -1;
}
/******************************************************************************
End of function  scif2Control
******************************************************************************/

/******************************************************************************
Function Name: sciSetIRQ
Description:   Function to enable / disable the SCIF interrupts
Arguments:     IN  bfEnable - true to enable
Return value:  0 for success otherwise error code
******************************************************************************/
static void sciSetIRQ(_Bool bfEnable)
{
    if (bfEnable)
    {
        R_INTC_Enable(INTC_ID_BRI2);
        R_INTC_Enable(INTC_ID_ERI2);
        R_INTC_Enable(INTC_ID_RXI2);
        R_INTC_Enable(INTC_ID_TXI2);
    }
    else
    {
        R_INTC_Disable(INTC_ID_BRI2);
        R_INTC_Disable(INTC_ID_ERI2);
        R_INTC_Disable(INTC_ID_RXI2);
        R_INTC_Disable(INTC_ID_TXI2);
    }
}
/******************************************************************************
End of function sciSetIRQ
******************************************************************************/

/******************************************************************************
Function Name: INT_SCIF2_BRI
Description:   Interrupt Serivice Routine for INT_SCIF2_BRI
Arguments:     none
Return value:  none
******************************************************************************/
static void INT_SCIF2_BRI(void)
{
    sciINTBreak(&gDDSCIF);
}
/******************************************************************************
End of function INT_SCIF2_BRI
******************************************************************************/

/******************************************************************************
Function Name: INT_SCIF2_ERI
Description:   Interrupt Serivice Routine for INT_SCIF2_ERI
Arguments:     none
Return value:  none
******************************************************************************/
static void INT_SCIF2_ERI(void)
{
    sciINTError(&gDDSCIF);
}
/******************************************************************************
End of function INT_SCIF2_ERI
******************************************************************************/

/******************************************************************************
Function Name: INT_SCIF2_RXI
Description:   Interrupt Serivice Routine for INT_SCIF2_RXI
Arguments:     none
Return value:  none
******************************************************************************/
static void INT_SCIF2_RXI(void)
{
    sciINTRxi(&gDDSCIF);
}
/******************************************************************************
End of function INT_SCIF2_RXI
******************************************************************************/

/******************************************************************************
Function Name: INT_SCIF2_TXI
Description:   Interrupt Serivice Routine for INT_SCIF2_TXI
Arguments:     none
Return value:  none
******************************************************************************/
static void INT_SCIF2_TXI(void)
{
    sciINTTxi(&gDDSCIF);
}
/******************************************************************************
End of function INT_SCIF2_TXI
******************************************************************************/

/******************************************************************************
End  Of File
******************************************************************************/
