/******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only 
* intended for use with Renesas products. No other uses are authorized. 
* This software is owned by Renesas Electronics Corporation and is protected 
* under all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS 
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE 
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the 
* following link:
* http://www.renesas.com/disclaimer
*******************************************************************************
* Copyright (C) 2011 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************
* File Name    : r_event.h
* Version      : 1.0
* Device(s)    : Renesas
* Tool-Chain   : GNU
* OS           : FreeRTOS
* H/W Platform : RSK2+RZA1H
* Description  : Task aware signals through "events"
******************************************************************************
* History      : DD.MM.YYYY Ver. Description
*              : 01.08.2009 1.00 First Release
******************************************************************************/

/******************************************************************************
  WARNING!  IN ACCORDANCE WITH THE USER LICENCE THIS CODE MUST NOT BE CONVEYED
  OR REDISTRIBUTED IN COMBINATION WITH ANY SOFTWARE LICENSED UNDER TERMS THE
  SAME AS OR SIMILAR TO THE GNU GENERAL PUBLIC LICENCE
******************************************************************************/

#ifndef EVENT_H_INCLUDED
#define EVENT_H_INCLUDED

/*****************************************************************************
User Includes
******************************************************************************/

#include <string.h>
#include "r_typedefs.h"
#include "FreeRTOS.h"
#include "queue.h"

/*****************************************************************************
Defines
******************************************************************************/

#define EV_WAIT_INFINITE            portMAX_DELAY

/*****************************************************************************
Enumerated Types
******************************************************************************/

typedef enum _EVSTATE
{
    EV_RESET = 0,
    EV_SET,
    EV_INVALID
} EVSTATE,
*PEVSTATE;

/*****************************************************************************
Typedefs
******************************************************************************/

typedef xQueueHandle PEVENT, *PPEVENT;

/*****************************************************************************
Public Functions
******************************************************************************/

#ifdef __cplusplus
extern "C" {
#endif

/*****************************************************************************
Function Name: eventCreate
Description:   Function to create one or more events
Arguments:     OUT ppEventList - Pointer to the destination event pointer array
               IN  uiNumber - The number of events to create
Return value:  The number of events not created
*****************************************************************************/

extern  uint32_t eventCreate(PPEVENT ppEventList, uint32_t uiNumber);

/*****************************************************************************
Function Name: eventDestroy
Description:   Functon to destroy one or more events
Arguments:     IN  ppEventList - Pointer to the event list to destroy
               IN  uiNumber The number of events to destroy
Return value:  none
*****************************************************************************/

extern  void eventDestroy(PPEVENT ppEventList, uint32_t uiNumber);

/*****************************************************************************
Function Name: eventSet
Description:   Function to set an event
Arguments:     IN  pEvent - Pointer to the event to set
Return value:  true if the event was set
*****************************************************************************/

extern  _Bool eventSet(PEVENT pEvent);

/*****************************************************************************
Function Name: eventReset
Description:   Function to reset an event
Arguments:     IN  Pointer to the event to reset
Return value:  true if the event was reset
*****************************************************************************/

extern  _Bool eventReset(PEVENT pEvent);

/*****************************************************************************
Function Name: eventState
Description:   Function to return the current state of an event
Arguments:     IN  pEvent - Pointer to the event
Return value:  The state of the event
*****************************************************************************/

extern  EVSTATE eventState(PEVENT pEvent);

/*****************************************************************************
Function Name: eventStateEx
Description:   Function to return the state of an event and then reset it
Arguments:     IN  pEvent - Pointer to the event
Return value:  The state of the event
*****************************************************************************/

extern  EVSTATE eventStateEx(PEVENT pEvent);

/*****************************************************************************
Function Name: eventWait
Description:   Function to wait for one or more events to be set
Arguments:     IN  ppEventList - Pointer to the list of events
               IN  uiNumber - The number of events to wait on
               IN  bfSingle - true if just one event being set is to wake
                              the task or false if all the events must be set
                              to wake the task
Return value:  When bfSingle is true the index of the event in the list which
               was set. Always 0 when bfSingle is false
*****************************************************************************/

extern  uint32_t eventWait(PPEVENT ppEventList,
                           uint32_t    uiNumber,
                           _Bool    bfSingle);

/*****************************************************************************
Function Name: eventWaitMutex
Description:   Function to wait for a mutex event
Arguments:     IN / OUT ppEvent - Pointer to the mutex event pointer
               IN  dwTimeOut - The time out in mS or EV_WAIT_INFINITE
Return value:  true for success, false if the event could not be created
               or the wait timed out
*****************************************************************************/

extern  _Bool eventWaitMutex(PPEVENT ppEvent, uint32_t dwTimeOut);

/*****************************************************************************
Function Name: eventReleaseMutex
Description:   Function to release a mutex event
Arguments:     IN / OUT ppEvent - Pointer to the mutex event pointer
Return value:  none
*****************************************************************************/

extern  void eventReleaseMutex(PPEVENT ppEvent);

/*****************************************************************************
Function Name: eventUsage
Description:   Function to get the number of events used
Arguments:     OUT pstEventsInUse - Pointer to the number of events in use
               OUT pstEventTotal - Pointer to the peak number of events used
Return value:  none
*****************************************************************************/

extern  void eventUsage(size_t *pstEventsInUse, size_t *pstEventTotal);

#ifdef __cplusplus
}
#endif

#endif /* EVENT_H_INCLUDED */

/******************************************************************************
End  Of File
******************************************************************************/
 
