/******************************************************************************
 * DISCLAIMER
 * This software is supplied by Renesas Electronics Corporation and is only
 * intended for use with Renesas products. No other uses are authorized. This
 * software is owned by Renesas Electronics Corporation and is protected under
 * all applicable laws, including copyright laws.
 * THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
 * THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
 * LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
 * AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
 * TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
 * ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
 * FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
 * ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
 * BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 * Renesas reserves the right, without notice, to make changes to this software
 * and to discontinue the availability of this software. By using this software,
 * you agree to the additional terms and conditions found by accessing the
 * following link:
 * http://www.renesas.com/disclaimer
 *******************************************************************************
 * Copyright (C) 2012 Renesas Electronics Corporation. All rights reserved.
 *******************************************************************************
 * File Name    : r_task.h
 * Version      : 1.0
 * Device(s)    : Renesas
 * Tool-Chain   : Renesas SH2A V9+
 * OS           : None
 * H/W Platform : SH2A
 * Description  : The task control application interface functions
 *******************************************************************************
 * History      : DD.MM.YYYY Ver. Description
 *              : 01.08.2009 1.00 First Release
 ******************************************************************************/

/******************************************************************************
 WARNING!  IN ACCORDANCE WITH THE USER LICENCE THIS CODE MUST NOT BE CONVEYED
 OR REDISTRIBUTED IN COMBINATION WITH ANY SOFTWARE LICENSED UNDER TERMS THE
 SAME AS OR SIMILAR TO THE GNU GENERAL PUBLIC LICENCE
 ******************************************************************************/

#ifndef TASK_H_INCLUDED
#define TASK_H_INCLUDED

/*****************************************************************************
 User Includes
 ******************************************************************************/
#include <stddef.h>
#include "r_typedefs.h"
#include "r_sysIf.h"
#include "r_event.h"
#include "r_timer.h"
#include "r_mbox.h"
#include "control.h"

/* Ensure local malloc is used */
#include "..\inc\malloc.h"

/*****************************************************************************
 Defines
 ******************************************************************************/

#define TC_INVALID_TASK_ID          ((uint32_t)NULL)
/* A minimum of 8 characters or "Unknown" will not fit! */
#define TC_MAX_TASK_NAME_SIZE       configMAX_TASK_NAME_LEN
#define TC_DEFAULT_STACK_SIZE       configDEFAULT_STACK_SIZE
#define TC_SMALL_STACK_SIZE         configSMALL_STACK_SIZE
#define TC_DEFAULT_HEAP             configMEMORY_TYPE_FOR_ALLOCATOR
#define TC_MAX_PRIORITY             (configMAX_PRIORITIES - 1)

/*****************************************************************************
 Function Macros
 ******************************************************************************/

#define __try(x)                    /* Not supported by FreeRTOS */
#define __catch(x)                  /* Not supported by FreeRTOS */
#define TC_THIS_TASK_PRIORITY       taskGetPriority(taskGetID())

/*****************************************************************************
 Typedefs
 ******************************************************************************/

typedef void (*PTASKFN)(void *pParameter);

typedef struct _SYSUSAGE
{
    /* The number of files in use */
    size_t stFilesInUse;

    /* The maximum number of files that can be opened */
    size_t stFileTotal;

    /* The number of events in use */
    size_t stEventsInUse;

    /* The total number of available events */
    size_t stEventTotal;

    /* The number of timers in use */
    size_t stTimersInUse;

    /* The total muber of timers available */
    size_t stTimerTotal;
} SYSUSAGE, *PSYSUSAGE;

/*****************************************************************************
 Public Functions
 ******************************************************************************/

#ifdef __cplusplus
extern "C"
{
#endif

/*****************************************************************************
 Function Name: taskCreate
 Description:   Function to create a task
 Arguments:     IN  pTaskFunction - Pointer to the task function
 IN  pParameters - Pointer to the task paramaters
 IN  stStackSize - The required stack size for the task
 IN  memType - The memory type to allocate the stack from
 IN  pszTaskName - The task name
 IN  uiPriority - The priority of the task
 Return value:  The task ID or -1U on error
 *****************************************************************************/

extern uint32_t taskCreate(PTASKFN pTaskFunction,
                           void *pParameters,
                           size_t stStackSize,
                           MEMTYPE memType,
                           char *pszTaskName,
                           uint32_t uiPriority);

/*****************************************************************************
 Function Name: taskDestroy
 Description:   Function to destroy a task
 Arguments:     IN  uiTaskID - The ID of the task to destroy
 Return value:  true if the task was destroyed
 *****************************************************************************/

extern _Bool taskDestroy(uint32_t uiTaskID);

/*****************************************************************************
 Function Name: taskGetID
 Description:   Function to get the task ID
 Arguments:     none
 Return value:  The ID of the task
 *****************************************************************************/

extern uint32_t taskGetID(void);

/*****************************************************************************
 Function Name: taskSleep
 Description:   Function to sleep for a number of mS
 Arguments:     IN  dwSleep_mS - The time to sleep in mS
 Return value:  none
 *****************************************************************************/

extern void taskSleep(uint32_t dwSleep_mS);

/*****************************************************************************
 Function Name: taskGetPriority
 Description:   Function to get the task priority
 Arguments:     IN uiTaskID - The ID of the task to get the priority of
 Return value:  The priority of the task or -1U if not found
 *****************************************************************************/

extern uint32_t taskGetPriority(uint32_t uiTaskID);

/*****************************************************************************
 Function Name: taskSetPriority
 Description:   Function to set the priority of a task
 Arguments:     IN  uiTaskID - The ID of the task to get the priority for
 IN  uiPriority - The priority of the task
 Return value:  true if the priority was set
 *****************************************************************************/

extern _Bool taskSetPriority(uint32_t uiTaskID, uint32_t uiPriority);

/*****************************************************************************
 Function Name: taskSetState
 Description:   Function set a task suspend / resume state
 Arguments:     IN  uiTaskID - The ID of the task to change state
 IN  bfSuspend - true to suspend the task
 false to resume operation
 Return value:  true if the state was changed
 *****************************************************************************/

extern _Bool taskSetState(uint32_t uiTaskID, _Bool bfSuspend);

/*****************************************************************************
 Function Name: taskName
 Description:   Function to get the task name
 Arguments:     OUT pszName - Pointer to the destination name
 IN  stLength - The length of the buffer
 IN  uiTaskID - The task ID
 Return value:  The number of chars delivered
 *****************************************************************************/

extern size_t taskName(char *pszName, size_t stLength, uint32_t uiTaskID);

/*****************************************************************************
Function Name: taskInformation
Description:   Function to get information on the running tasks in the form of
               a dynamically allocated string. The string should be freed with
               a call to free
Arguments:     none
Return value:  Pointer to a zero terminated string
*****************************************************************************/

extern char *taskInformation(void);

/*****************************************************************************
 Function Name: taskSwitch
 Description:   Function to cause a task switch
 Arguments:     none
 Return value:  none
 *****************************************************************************/

extern void taskSwitch(void);

/*****************************************************************************
 Function Name: taskExit
 Description:   Function to cause the current task to exit
 Arguments:     none
 Return value:  none
 *****************************************************************************/

extern void taskExit(void);

/*****************************************************************************
 Function Name: systemUsage
 Description:   Function to get the system reasource usage information
 Arguments:     OUT pSysUsage - Pointer to the destination system usage info
 Return value:  none
 *****************************************************************************/

extern void systemUsage(PSYSUSAGE pSysUsage);

#ifdef __cplusplus
}
#endif

#endif /* TASK_H_INCLUDED */

/******************************************************************************
 End  Of File
 ******************************************************************************/
