/******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
*******************************************************************************
* Copyright (C) 2011 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************
* File Name    : sysif.c
* Version      : 1.02
* Description  : The system interface functions to abstract the WebEngine
*                from any particuar operating system.
******************************************************************************
* History : DD.MM.YYYY Version Description
*         : 04.02.2010 1.00    First Release
*         : 10.06.2010 1.01    Updated type definitions
*         : 10.11.2010 1.02    Updated get signal state
******************************************************************************/

/******************************************************************************
  WARNING!  IN ACCORDANCE WITH THE USER LICENCE THIS CODE MUST NOT BE CONVEYED
  OR REDISTRIBUTED IN COMBINATION WITH ANY SOFTWARE LICENSED UNDER TERMS THE
  SAME AS OR SIMILAR TO THE GNU GENERAL PUBLIC LICENCE
******************************************************************************/

/******************************************************************************
Includes   <System Includes> , "Project Includes"
******************************************************************************/

#include "r_typedefs.h"
#include "compiler_settings.h"

/* Ensure local malloc is used */
#include "..\inc\malloc.h"
//#include "malloc.h"

#include "r_sysIf.h"
#include "FreeRTOS.h"
#include "task.h"
#include "semphr.h"
#include "control.h"

/******************************************************************************
Typedef definitions
******************************************************************************/

typedef int(*FP_VECTOR)(void);

/******************************************************************************
Function Prototypes
******************************************************************************/

/******************************************************************************
External function prototypes and variables
******************************************************************************/

extern unsigned long ulPortInterruptNesting;

/******************************************************************************
Constant Data
******************************************************************************/

/******************************************************************************
Private Variables
******************************************************************************/

static xSemaphoreHandle gpvStreamSemaphore = NULL;
static void             *gpvTaskHandle = NULL;

/******************************************************************************
Exported global variables and functions (to be accessed by other files)
******************************************************************************/

/*****************************************************************************
* Function Name: sysYield
* Description  : Function to yield to other tasks or threads
* Arguments    : none
* Return Value : none
******************************************************************************/
void sysYield(void)
{
    taskYIELD();
}
/*****************************************************************************
End of function  sysYield
******************************************************************************/

/*****************************************************************************
* Function Name: sysLock
* Description  : Function to lock a critical section
* Arguments    : IN  pvLockObj - Pointer to an operating system locking object
*                                or NULL if not required
* Return Value : The lock value or -1 on error
******************************************************************************/
int sysLock(void *pvLockObj)
{
    if (!pvLockObj)
    {
        /* Disable the irq and return 1 to show that the interrupts are masked */
        taskENTER_CRITICAL();
        return 1;
    }
    else
    {
        xSemaphoreHandle xSemaphore;
        xSemaphoreHandle *pxSemaphore = (xSemaphoreHandle*)pvLockObj;
        if (!*pxSemaphore)
        {
            /* Create the semaphore */
            xSemaphore = xSemaphoreCreateMutex();

            /* Return the mutex to the user */
            *pxSemaphore = xSemaphore;
        }
        else
        {
            xSemaphore = *pxSemaphore;
        }

        /* Check that the semaphore was created */
        if (xSemaphore)
        {
            /* Check to see if this function is called within an ISR */
            if (0 == ulPortInterruptNesting)
            {
                /* Get mutex */
                if (xSemaphoreTake(xSemaphore, portMAX_DELAY))
                {
                    return 0;
                }
            }
            else
            {
                /* Get mutex */
                if (xSemaphoreTakeFromISR(xSemaphore, NULL))
                {
                    return 0;
                }
            }
        }

        /* Lock with a mutex has failed but mask interrupts instead */
        taskENTER_CRITICAL();

        /* Failed to create the mutex or to take it */
        return -1;
    }
}
/*****************************************************************************
End of function  sysLock
******************************************************************************/

/*****************************************************************************
* Function Name: sysUnlock
* Description  : Function to unlock a critical section
* Arguments    : IN  pvLockObj - Pointer to an operating system locking object
*                                or NULL if not requred
*                IN  iUnlock - The value returned from sysLock function
* Return Value : none
******************************************************************************/
void sysUnlock(void *pvLockObj, int iUnlock)
{
    if (!pvLockObj)
    {
        taskEXIT_CRITICAL();
    }
    else
    {
        xSemaphoreHandle xSemaphore = *((xSemaphoreHandle*)pvLockObj);

        /* Check to see if this function is called within an ISR */
        if (0 == ulPortInterruptNesting)
        {
            /* Release mutex */
            xSemaphoreGive(xSemaphore);
        }
        else
        {
            /* Release mutex */
            xSemaphoreGiveFromISR(xSemaphore, NULL);
        }
    }
}
/*****************************************************************************
End of function  sysUnlock
******************************************************************************/

/*****************************************************************************
Function Name: sysWaitAccess
Description:   Function to allow the same task to enter a critical section and
               other tasks wait
Arguments:     none
Return value:  none
*****************************************************************************/
void sysWaitAccess(void)
{
    void *pvTaskHandle = xTaskGetCurrentTaskHandle();
    if (gpvTaskHandle == pvTaskHandle)
    {
        /* allow re-entrant access */
    }
    else
    {
        /* Wait for access to the list */
        if (NULL == gpvStreamSemaphore)
        {
            gpvStreamSemaphore = xSemaphoreCreateMutex();
        }

        /* Wait mutex */
        xSemaphoreTake(gpvStreamSemaphore, portMAX_DELAY);

        /* Assign the task ID for future reentrant access */
        gpvTaskHandle = pvTaskHandle;
    }
}
/*****************************************************************************
End of function  sysWaitAccess
******************************************************************************/

/*****************************************************************************
Function Name: sysReleaseAccess
Description:   Function to allow other tasks to access a re-entrant critial
               section
Arguments:     none
Return value:  none
*****************************************************************************/
void sysReleaseAccess(void)
{
    gpvTaskHandle = NULL;

    /* Release mutex */
    xSemaphoreGive(gpvStreamSemaphore);
}
/*****************************************************************************
End of function  sysReleaseAccess
******************************************************************************/

/*****************************************************************************
FreeRTOS low level system interface functions
******************************************************************************/

/******************************************************************************
* Function Name: pvPortMalloc
* Description  : Function to allocate memory for FreeRTOS
* Arguments    : IN  stSize - The size of memory to allocate
* Return Value : Pointer to the memory or NULL on error
******************************************************************************/
void *pvPortMalloc(size_t stSize)
{
    return exMalloc(configMEMORY_TYPE_FOR_ALLOCATOR, stSize);
}
/******************************************************************************
End of function  pvPortMalloc
******************************************************************************/

/******************************************************************************
* Function Name: vPortFree
* Description  : Function to free memory for FreeRTOS
* Arguments    : none
* Return Value : none
******************************************************************************/
void vPortFree(void *pvFree)
{
    exFree(configMEMORY_TYPE_FOR_ALLOCATOR, pvFree);
}
/******************************************************************************
End of function  vPortFree
******************************************************************************/

/******************************************************************************
* Function Name: xPortGetFreeHeapSize
* Description  : Function to get the free heap size for FreeRTOS
* Arguments    : none
* Return Value : none
******************************************************************************/
size_t xPortGetFreeHeapSize(void)
{
    HEAPUSAGE memUsage;
    heapUsage(&memUsage, configMEMORY_TYPE_FOR_ALLOCATOR);
    return memUsage.stTotalFree;
}
/******************************************************************************
End of function  xPortGetFreeHeapSize
******************************************************************************/

/******************************************************************************
* Function Name: vPortInitialiseBlocks
* Description  : Dummy function for FreeRTOS memory management
* Arguments    : none
* Return Value : none
******************************************************************************/
void vPortInitialiseBlocks(void)
{
    /* No function required */
}
/******************************************************************************
End of function  vPortInitialiseBlocks
******************************************************************************/

/******************************************************************************
Private functions
******************************************************************************/

/******************************************************************************
End  Of File
******************************************************************************/
