/******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
*******************************************************************************
* Copyright (C) 2012 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************
* File Name    : r_taskControl.c
* Version      : 1.00
* Device(s)    : Renesas
* Tool-Chain   : Renesas SH2A V9+
* OS           : None
* H/W Platform : SH2A
* Description  : The main task control functions and internal functions
******************************************************************************
* History      : DD.MM.YYYY Ver. Description
*              : 01.08.2009 1.00 First Release
******************************************************************************/

/******************************************************************************
  WARNING!  IN ACCORDANCE WITH THE USER LICENCE THIS CODE MUST NOT BE CONVEYED
  OR REDISTRIBUTED IN COMBINATION WITH ANY SOFTWARE LICENSED UNDER TERMS THE
  SAME AS OR SIMILAR TO THE GNU GENERAL PUBLIC LICENCE
******************************************************************************/

/******************************************************************************
Includes   <System Includes> , "Project Includes"
******************************************************************************/
#include <stddef.h>
#include <string.h>

/*****************************************************************************
User Includes
******************************************************************************/

#include "r_task.h"
#include "FreeRTOS.h"
#include "task.h"

/* Ensure local malloc is used */
#include "..\inc\malloc.h"
//#include "malloc.h"

#include "queue.h"
#include "portMacro.h"
#include "mabMalloc.h"
#include "Trace.h"
#include "xvasprintf.h"

/*****************************************************************************
Function Macros
******************************************************************************/

/* Comment this line out to turn ON module trace in this file */
#undef _TRACE_ON_

#ifndef _TRACE_ON_
#undef TRACE
#define TRACE(x)
#endif

/*****************************************************************************
Function Prototypes
******************************************************************************/

/* From lowsrc.c */
extern void fileUsage(size_t *pstInUse, size_t *pstTotal);

/*****************************************************************************
External Variables
******************************************************************************/

/*****************************************************************************
Global Variables
******************************************************************************/

/*****************************************************************************
Public Functions
******************************************************************************/

/*****************************************************************************
Function Name: taskCreate
Description:   Function to create a task
Arguments:     IN  pTaskFunction - Pointer to the task function
               IN  pParameters - Pointer to the task paramaters
               IN  stStackSize - The required stack size for the task
               IN  memType - The memory type to allocate the stack from
               IN  pszTaskName - The task name
               IN  uiPriority - The priority of the task
Return value:  The task ID or -1 on error
*****************************************************************************/
uint32_t taskCreate(PTASKFN  pTaskFunction,
                    void    *pParameters,
                    size_t   stStackSize,
                    MEMTYPE  memType,
                    char    *pszTaskName,
                    uint32_t uiPriority)
{
    /* Allocate the memory for the task stack */
    void    *pvTaskStack = exMalloc(memType, stStackSize * sizeof(portSTACK_TYPE));
    if (pvTaskStack)
    {
        xTaskHandle pvTask;
        vTaskSuspendAll();

        /* Create the task */
        if (pdPASS == xTaskGenericCreate(pTaskFunction,
                                         pszTaskName,
                                         stStackSize,
                                         pParameters,
                                         uiPriority,
                                         &pvTask,
                                         pvTaskStack,
                                         NULL))
        {
            PMBL    pMbl = (PMBL)pvTaskStack;

            /* Point at the start of the memory allocated to this task */
            pMbl--;

            /* When allocated it will have been assigned the ID of the task that
               created it. It needs to be modified to the ID of the task itself */
            pMbl->pvTaskHandle = pvTask;
            xTaskResumeAll();
            return (uint32_t)pvTask;
        }
        xTaskResumeAll();
    }
    return -1;
}
/*****************************************************************************
End of function  taskCreate
******************************************************************************/

/*****************************************************************************
Function Name: taskDestroy
Description:   Function to destroy a task
Arguments:     IN  uiTaskID - The ID of the task to destroy
Return value:  true if the task was destroyed
*****************************************************************************/
_Bool taskDestroy(uint32_t uiTaskID)
{
    vTaskDelete((xTaskHandle)uiTaskID);
    return true;
}
/*****************************************************************************
End of function  taskDestroy
******************************************************************************/

/*****************************************************************************
Function Name: taskGetID
Description:   Function to get the task ID
Arguments:     none
Return value:  The ID of the task
*****************************************************************************/
uint32_t taskGetID(void)
{
    return (uint32_t)xTaskGetCurrentTaskHandle();
}
/*****************************************************************************
End of function  taskGetID
******************************************************************************/

/*****************************************************************************
Function Name: taskSleep
Description:   Function to sleep for a number of mS
Arguments:     IN  dwSleep_mS - The time to sleep in mS
Return value:  none
*****************************************************************************/
void taskSleep(uint32_t dwSleep_mS)
{
    if (dwSleep_mS)
    {
        portTickType xTimeDelay = dwSleep_mS / portTICK_RATE_MS;
        vTaskDelay(xTimeDelay);
    }
    else
    {
        taskYIELD();
    }
}
/*****************************************************************************
End of function  taskSleep
******************************************************************************/

/*****************************************************************************
Function Name: taskGetPriority
Description:   Function to get the task priority
Arguments:     IN uiTaskID - The ID of the task to get the priority of
Return value:  The priority of the task or -1U if not found
*****************************************************************************/
uint32_t taskGetPriority(uint32_t uiTaskID)
{
    if ((xTaskHandle)uiTaskID != (xTaskHandle)TC_INVALID_TASK_ID)
    {
        return (uint32_t)uxTaskPriorityGet((xTaskHandle)uiTaskID);
    }
    else
    {
        return -1;
    }
}
/*****************************************************************************
End of function  taskGetPriority
******************************************************************************/

/*****************************************************************************
Function Name: taskSetPriority
Description:   Function to set the priority of a task
Arguments:     IN  uiTaskID - The ID of the task to get the priority for
               IN  uiPriority - The priority of the task
Return value:  true if the priority was set
*****************************************************************************/
_Bool taskSetPriority(uint32_t uiTaskID, uint32_t uiPriority)
{
    if ((xTaskHandle)uiTaskID != (xTaskHandle)TC_INVALID_TASK_ID)
    {
        vTaskPrioritySet((xTaskHandle)uiTaskID, uiPriority);
        return true;
    }
    else
    {
        return false;
    }
}
/*****************************************************************************
End of function  taskSetPriority
******************************************************************************/

/*****************************************************************************
Function Name: taskSetState
Description:   Function set a task suspend / resume state
Arguments:     IN  uiTaskID - The ID of the task to change state
               IN  bfSuspend - true to suspend the task
                               false to resume operation
Return value:  true if the state was changed
*****************************************************************************/
_Bool taskSetState(uint32_t uiTaskID, _Bool bfSuspend)
{
    if (bfSuspend)
    {
        switch (eTaskGetState((xTaskHandle)uiTaskID))
        {
            case eReady:
            case eRunning:
            case eBlocked:
            {
                vTaskSuspend((xTaskHandle)uiTaskID);
                return true;
            }
            default:
            case eDeleted:
            case eSuspended:
            {
                return false;
            }
        }
    }
    else
    {
        switch (eTaskGetState((xTaskHandle)uiTaskID))
        {
            default:
            case eReady:
            case eRunning:
            case eBlocked:
            case eDeleted:
            {
                return false;
            }
            case eSuspended:
            {
                vTaskResume((xTaskHandle)uiTaskID);
                return true;
            }
        }
    }
}
/*****************************************************************************
End of function  taskSetState
******************************************************************************/

/*****************************************************************************
Function Name: taskName
Description:   Function to get the task name
Arguments:     OUT pszName - Pointer to the destination name
               IN  uiLength - The length of the buffer
               IN  uiTaskID - The task ID
Return value:  The number of chars delivered
*****************************************************************************/
size_t taskName(char *pszName, size_t stLength, uint32_t uiTaskID)
{
    char    *pszTaskName = pcTaskGetTaskName((xTaskHandle)uiTaskID);
    if (pszTaskName)
    {
        strncpy(pszName, pszTaskName, stLength);
        return stLength;
    }
    return 0;
}
/*****************************************************************************
End of function  taskName
******************************************************************************/

/*****************************************************************************
Function Name: taskInformation
Description:   Function to get information on the running tasks in the form of
               a dynamically allocated string. The string should be freed with
               a call to free
Arguments:     none
Return value:  Pointer to a zero terminated string
*****************************************************************************/
char *taskInformation(void)
{
    CAT         Cat;
    uint32_t    uiNumTasks = uxTaskGetNumberOfTasks();
    size_t      stLength = uiNumTasks * sizeof(xTaskStatusType);
    uint32_t    uiAllocation = 4096;
    xTaskStatusType *pxTaskStatusArray = malloc(stLength);
    if (pxTaskStatusArray)
    {
        xTaskStatusType *pStatus = pxTaskStatusArray;
        static const char *const ppszTaskState[] =
        {
            "Running  ",
            "Ready    ",
            "Blocked  ",
            "Suspended",
            "Deleted  "
        };
        static const char *const pszPad = "                 ";

        /* Get the system state */
        uiNumTasks = uxTaskGetSystemState(pxTaskStatusArray,
                                          stLength,
                                          NULL);

        /* Initialise the concatenated string object */
        xcatcreate(&Cat);

        /* Print the header */
        xcatprintf(&Cat,
                   uiAllocation,
                   "Task                                    Stack \r\n"
                   "Name             State   Priority       Avail   Handle\r\n"
                   "----------------------------------------------------------\r\n");

        /* Print the status of each task */
        while (uiNumTasks--)
        {
            /* Calculate any padding required after the task name */
            int32_t iPad = (configMAX_TASK_NAME_LEN  + 1) - strlen(pStatus->pcTaskName);
            xcatprintf(&Cat,
                       uiAllocation,
                       "%s%.*s%s\t%u\t%u\t0x%p\r\n",
                       pStatus->pcTaskName,
                       iPad, pszPad,
                       ppszTaskState[pStatus->eCurrentState],
                       pStatus->uxCurrentPriority,
                       pStatus->usStackHighWaterMark,
                       pStatus->xHandle);
            pStatus++;
        }
        free(pxTaskStatusArray);
    }
    return xcatstring(&Cat);
}
/*****************************************************************************
End of function  taskInformation
******************************************************************************/

/*****************************************************************************
Function Name: taskSwitch
Description:   Function to cause a task switch
Arguments:     none
Return value:  none
*****************************************************************************/
void taskSwitch(void)
{
    taskYIELD();
}
/*****************************************************************************
End of function  taskSwitch
******************************************************************************/

/*****************************************************************************
Function Name: taskExit
Description:   Function to cause a task to exit
Arguments:     none
Return value:  none
*****************************************************************************/
void taskExit(void)
{
    vTaskDelete(NULL);
}
/*****************************************************************************
End of function  taskExit
******************************************************************************/

/*****************************************************************************
Function Name: systemUsage
Description:   Function to get the system reasource usage information
Arguments:     OUT pSysUsage - Pointer to the destination system usage info
Return value:  none
*****************************************************************************/
void systemUsage(PSYSUSAGE pSysUsage)
{
    fileUsage(&pSysUsage->stFilesInUse, &pSysUsage->stFileTotal);
    eventUsage(&pSysUsage->stEventsInUse, &pSysUsage->stEventTotal);
    timerUsage(&pSysUsage->stTimersInUse, &pSysUsage->stTimerTotal);
}
/*****************************************************************************
End of function  systemUsage
******************************************************************************/

/******************************************************************************
End  Of File
******************************************************************************/
