/******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
*******************************************************************************
* Copyright (C) 2012 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************
* File Name    : timerControl.c
* Version      : 1.00
* Device(s)    : Renesas
* Tool-Chain   : GNU
* OS           : FreeRTOS
* H/W Platform : RZA1
* Description  : Timer functions
*******************************************************************************
* History      : DD.MM.YYYY Ver. Description
*              : 01.08.2009 1.00 First Release
******************************************************************************/

/******************************************************************************
  WARNING!  IN ACCORDANCE WITH THE USER LICENCE THIS CODE MUST NOT BE CONVEYED
  OR REDISTRIBUTED IN COMBINATION WITH ANY SOFTWARE LICENSED UNDER TERMS THE
  SAME AS OR SIMILAR TO THE GNU GENERAL PUBLIC LICENCE
******************************************************************************/

/******************************************************************************
Includes   <System Includes> , "Project Includes"
******************************************************************************/

#include "r_task.h"
#include "FreeRTOS.h"
#include "timers.h"
#include "Trace.h"

/******************************************************************************
Macro definitions
******************************************************************************/

#define TIMER_QUEUE_BLOCK_TIME              (1024UL)

/*****************************************************************************
Typedefs
******************************************************************************/

/*****************************************************************************
Function Prototypes
******************************************************************************/

static void tmCallBackFunction(xTimerHandle pvTimerHandle);
static void tmCallBackEvent(xTimerHandle pvTimerHandle);
static void tmAddTimerHandle(xTimerHandle pvTimerHandle);
static xTimerHandle tmGetTimerHandle(PEVENT pEvent);

/******************************************************************************
External Variables
******************************************************************************/

extern int_t g_itimer;

/*****************************************************************************
Global Variables
******************************************************************************/

/* An array of timer handles that have been allocated to time events */
xTimerHandle    gppEventTimers[configTIMER_QUEUE_LENGTH];
size_t          gstTimerCount = 0UL;

/******************************************************************************
Public Functions
******************************************************************************/

/*****************************************************************************
Function Name: timerStartCallBack
Description:   Function to start a call-back timer
Arguments:     IN  pTimer - Pointer to the timer struct
               IN  pTimerFunction - Pointer to the timer function to call
               IN  pvParameter - Pointer to the parameter to pass the function
               IN  ulTime_mS - The time in mS
               IN  Type - The type
Return value:  0 for success or -1 on error
*****************************************************************************/
int timerStartCallBack(PCBTMR   pTimer,
                       PCFNTMR  pTimerFunction,
                       void     *pvParameter,
                       uint32_t ulTime_mS,
                       TMRTYPE  Type)
{
    xTimerHandle pvTimerHandle;
    unsigned portBASE_TYPE uxAutoReload = (SINGLE_SHOT_TIMER == Type)
                                        ? pdFALSE : pdTRUE;

    /* Initialise the timer data */
    pTimer->pTimerFunction = pTimerFunction;
    pTimer->pvParameter = pvParameter;
    pTimer->ulTime = ulTime_mS;
    pTimer->Type = Type;

    /* Create a timer */
    pvTimerHandle = xTimerCreate("CALLBACK",
                                 (ulTime_mS / portTICK_RATE_MS),
                                 uxAutoReload,
                                 pTimer,
                                 tmCallBackFunction);

    /* Check that it was created */
    if (pvTimerHandle)
    {
        /* Assign the timer handle */
        pTimer->pvTimerHandle = pvTimerHandle;

        /* Start the timer */
        if (pdPASS == xTimerStart(pvTimerHandle, TIMER_QUEUE_BLOCK_TIME))
        {
            gstTimerCount++;
            return 0;
        }
        else
        {
            /* Delete it */
            xTimerDelete(pvTimerHandle, TIMER_QUEUE_BLOCK_TIME);
            pTimer->pvTimerHandle = NULL;
        }
    }
    return -1;
}
/*****************************************************************************
End of function  timerStartCallBack
******************************************************************************/

/*****************************************************************************
Function Name: timerStopCallBack
Description:   Function to stop a call-back timer
Arguments:     IN  pTimer - Pointer to the call-back timer structure
Return value:  0 for success
*****************************************************************************/
int timerStopCallBack(PCBTMR pTimer)
{
    if ((pdPASS == xTimerStop((xTimerHandle)pTimer->pvTimerHandle,
                              TIMER_QUEUE_BLOCK_TIME))
    &&  (pdPASS == xTimerDelete((xTimerHandle)pTimer->pvTimerHandle,
                                TIMER_QUEUE_BLOCK_TIME)))
    {
        gstTimerCount--;
        return 0;
    }
    return -1;
}
/*****************************************************************************
End of function  timerStopCallBack
******************************************************************************/

/*****************************************************************************
Function Name: timerStartEvent
Description:   Function to start an event timer
Arguments:     IN  tmrType - The type of timer to start, SINGLE_SHOT_TIMER or
                             PERIODIC_TIMER
               IN  ulTime_mS - The time in mS
Return value:  Pointer to the event or NULL on error
*****************************************************************************/
PEVENT timerStartEvent(TMRTYPE tmrType, uint32_t ulTime_mS)
{
    if (ulTime_mS)
    {
        PEVENT  pTimerEvent;

        /* Create the event for the timer */
        if (0 == eventCreate(&pTimerEvent, 1))
        {
            xTimerHandle pvTimerHandle;
            unsigned portBASE_TYPE uxAutoReload = (SINGLE_SHOT_TIMER == tmrType)
                                                ? pdFALSE : pdTRUE;

            /* Create a timer */
            pvTimerHandle = xTimerCreate("EVENT",
                                         (ulTime_mS / portTICK_RATE_MS),
                                         uxAutoReload,
                                         pTimerEvent,
                                         tmCallBackEvent);

            /* Check that it was created */
            if (pvTimerHandle)
            {
                /* Start the timer */
                if (pdPASS == xTimerStart(pvTimerHandle, TIMER_QUEUE_BLOCK_TIME))
                {
                    /* Add the timer to the cache */
                    tmAddTimerHandle(pvTimerHandle);
                    gstTimerCount++;
                    return pTimerEvent;
                }
                else
                {
                    /* Delete it */
                    xTimerDelete(pvTimerHandle, TIMER_QUEUE_BLOCK_TIME);
                }
            }

            /* Destroy the event */
            eventDestroy(&pTimerEvent, 1);
        }
    }
    return NULL;
}
/*****************************************************************************
End of function  timerStartEvent
******************************************************************************/

/*****************************************************************************
Function Name: timerStopEvent
Description:   Function to stop an event timer
Arguments:     IN  pEvent - Pointer to the event timer to stop
Return value:  none
*****************************************************************************/
void timerStopEvent(PEVENT pEvent)
{
    xTimerHandle pvTimerHandle = tmGetTimerHandle(pEvent);
    if (pvTimerHandle)
    {
        eventDestroy(&pEvent, 1);

        /* Stop it */
        xTimerStop(pvTimerHandle, TIMER_QUEUE_BLOCK_TIME);

        /* Delete it */
        xTimerDelete(pvTimerHandle, TIMER_QUEUE_BLOCK_TIME);
        gstTimerCount--;
    }
}
/*****************************************************************************
End of function  timerStopEvent
******************************************************************************/

/*****************************************************************************
Function Name: timerStartMeasurement
Description:   Function to start a measurement timer (low resolution)
Arguments:     OUT pTimeStamp - Pointer to the destination time stamp
Return value:  none
*****************************************************************************/
void timerStartMeasurement(PTMSTMP pTimeStamp)
{
    control(g_itimer, CTL_GET_TIME_STAMP, pTimeStamp);
}
/*****************************************************************************
End of function  timerStartMeasurement
******************************************************************************/

/*****************************************************************************
Function Name: timerStopMeasurement
Description:   Function to stop a measurement timer and return the elapsed time
Arguments:     IN  pTimeStamp - Pointer to the staring time stamp
Return value:  The elapsed time in seconds
*****************************************************************************/
float timerStopMeasurement(PTMSTMP pTimeStamp)
{
    TMSTMP  timeNow;
    if (control(g_itimer, CTL_GET_TIME_STAMP, &timeNow) == 0)
    {
        uint32_t    ulTime;
        int32_t     iDelta;
        float       fResult;

        /* Calculate the number of mS */
        if (timeNow.ulMilisecond >= pTimeStamp->ulMilisecond)
        {
            ulTime = timeNow.ulMilisecond - pTimeStamp->ulMilisecond;
        }
        else
        {
            //TODO: Overflow value is wrong
            ulTime = (0xFFFFFFFF - pTimeStamp->ulMilisecond) + timeNow.ulMilisecond;
        }

        /* Set the number of mS */
        fResult = (1e-3 * (float)ulTime);

        /* Calculate the sub 1mS time */
        if (timeNow.usSubMilisecond >= pTimeStamp->usSubMilisecond)
        {
            iDelta = timeNow.usSubMilisecond - pTimeStamp->usSubMilisecond;
        }
        else
        {
            const uint32_t uiCountMax = (portTICK_CNT_FREQ_HZ / 1000);
            iDelta = (uiCountMax - pTimeStamp->usSubMilisecond) + timeNow.usSubMilisecond;

            /* If the timer has wrapped then there is possibly an extra mS
               counted which has not happened */
            if (fResult > 1e-3)
            {
                fResult -= 1e-3;
            }
        }
        fResult += (float)((float)iDelta * 1.0f / (float)portTICK_CNT_FREQ_HZ);

        /* Add on the number of mS */
        return fResult;
    }

    /* Throw a divide by 0 exception */
    return 0.0f;
}
/*****************************************************************************
End of function  timerStopMeasurement
******************************************************************************/

/*****************************************************************************
Function Name: timerUsage
Description:   Function to get the timer usage information
Arguments:     OUT pstInUse - Pointer to the numebr of timers in use
               OUT pstTotal - Pointer to the total number of timers available
Return value:  none
*****************************************************************************/
void timerUsage(size_t *pstInUse, size_t *pstTotal)
{
    /* Return the details */
    if (pstInUse)
    {
        *pstInUse = gstTimerCount;
    }
    if (pstTotal)
    {
        *pstTotal = (size_t)configTIMER_QUEUE_LENGTH;
    }
}
/*****************************************************************************
End of function  timerUsage
******************************************************************************/

/******************************************************************************
Internal Functions
******************************************************************************/

/******************************************************************************
* Function Name: tmCallBackFunction
* Description  : Timer call-back function for call-back timers
* Arguments    : IN  pvTimerHandle - Pointer to the timer handle
* Return Value : none
******************************************************************************/
static void tmCallBackFunction(xTimerHandle pvTimerHandle)
{
    /* Get a pointer to the timer structure */
    PCBTMR   pTimer = (PCBTMR)pvTimerGetTimerID(pvTimerHandle);

    /* Call the timer call-back */
    pTimer->pTimerFunction(pTimer->pvParameter);

    /* If it is a single shot timer */
    if (SINGLE_SHOT_TIMER == pTimer->Type)
    {
        /* Stop it */
        xTimerStop(pvTimerHandle, TIMER_QUEUE_BLOCK_TIME);

        /* Delete it */
        xTimerDelete(pvTimerHandle, TIMER_QUEUE_BLOCK_TIME);
        gstTimerCount--;
    }
}
/******************************************************************************
End of function  tmCallBackFunction
******************************************************************************/

/******************************************************************************
* Function Name: tmCallBackEvent
* Description  : Timer call back for event timers
* Arguments    : IN  pvTimerHandle - Pointer to the timer handle
* Return Value : none
******************************************************************************/
static void tmCallBackEvent(xTimerHandle pvTimerHandle)
{
    PEVENT  pEvent = (PCBTMR)pvTimerGetTimerID(pvTimerHandle);
    eventSet(pEvent);
}
/******************************************************************************
End of function  tmCallBackEvent
******************************************************************************/

/******************************************************************************
* Function Name: tmAddTimerHandle
* Description  : Function to add the timer handle to the cache
* Arguments    : IN  pvTimerHandle - The timer handle to add
* Return Value : none
******************************************************************************/
static void tmAddTimerHandle(xTimerHandle pvTimerHandle)
{
    int32_t iCount = configTIMER_QUEUE_LENGTH;
    vTaskSuspendAll();
    while (iCount--)
    {
        /* Look for an empty slot */
        if (NULL == gppEventTimers[iCount])
        {
            /* Save the timer reference */
            gppEventTimers[iCount] = pvTimerHandle;

            /* And stop looking */
            iCount = 0;
        }
    }
    xTaskResumeAll();
}
/******************************************************************************
End of function  tmAddTimerHandle
******************************************************************************/

/******************************************************************************
* Function Name: tmGetTimerHandle
* Description  : Function to get the timer handle from the cache
* Arguments    : IN  pEvent - Pointer to the event associated with the timer
* Return Value : Pointer to the timer handle
******************************************************************************/
static xTimerHandle tmGetTimerHandle(PEVENT pEvent)
{
    int32_t iCount = configTIMER_QUEUE_LENGTH;
    xTimerHandle pvResult = NULL;
    vTaskSuspendAll();
    while (iCount--)
    {
        /* Look for a valid timer handle in the cache */
        if ((gppEventTimers[iCount])

        /* That matches the event passes */
        &&  (pvTimerGetTimerID(gppEventTimers[iCount]) == pEvent))
        {
            /* Get the timer handle */
            pvResult = gppEventTimers[iCount];

            /* Clear the entry */
            gppEventTimers[iCount] = NULL;

            /* And stop looking */
            iCount = 0;
        }
    }
    xTaskResumeAll();
    return pvResult;
}
/******************************************************************************
End of function  tmGetTimerHandle
******************************************************************************/

/******************************************************************************
Private Functions
******************************************************************************/

/******************************************************************************
End  Of File
******************************************************************************/
