/******************************************************************************
 * DISCLAIMER
 * This software is supplied by Renesas Electronics Corporation and is only
 * intended for use with Renesas products. No other uses are authorized. This
 * software is owned by Renesas Electronics Corporation and is protected under
 * all applicable laws, including copyright laws.
 * THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
 * THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
 * LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
 * AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
 * TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
 * ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
 * FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
 * ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
 * BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 * Renesas reserves the right, without notice, to make changes to this software
 * and to discontinue the availability of this software. By using this software,
 * you agree to the additional terms and conditions found by accessing the
 * following link:
 * http://www.renesas.com/disclaimer
 *******************************************************************************
 * Copyright (C) 2012 Renesas Electronics Corporation. All rights reserved.
 *******************************************************************************
 * File Name    : sysUsage.c
 * Version      : 1.01
 * Description  : Functions to estimate system resource information.
 *******************************************************************************
 * History      : DD.MM.YYYY Version Description
 *              : 01.01.2011 1.00    First Release
 *              : 31.03.2011 1.01    Fixed event memory leak
 ******************************************************************************/

/******************************************************************************
 WARNING!  IN ACCORDANCE WITH THE USER LICENCE THIS CODE MUST NOT BE CONVEYED
 OR REDISTRIBUTED IN COMBINATION WITH ANY SOFTWARE LICENSED UNDER TERMS THE
 SAME AS OR SIMILAR TO THE GNU GENERAL PUBLIC LICENCE
 ******************************************************************************/

/******************************************************************************
 Includes   <System Includes> , "Project Includes"
 ******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include "r_task.h"

/* Ensure local malloc is used */
#include "..\inc\malloc.h"
//#include "malloc.h"

#include "sysUsage.h"
#include "xvasprintf.h"

/******************************************************************************
 Defines
 ******************************************************************************/
/* Define the time that one sub mS timer count takes (t = 1/f) */
#define SUB_MS_TIMER                (1.0f / (48000000.0f / 32.0f))
#define SUB_LENGTH                  5120

/******************************************************************************
 Typedefs
 ******************************************************************************/

typedef struct _SRD
{
    /* Pointer to the "data array" (ppfData[][]) which is really a pointer to
     the start address of the pointer array - but that is how the C language
     does it. Here we do it with code because the array size is defined at
     run time not complile time */
    float **ppfData;
    int   iNumRows;
    int   iNumColumns;

    /* The start address of the pointer array */
    float *pfData;
} SRD, *PSRD;

typedef struct _SRI
{
    _Bool       bfStarted;
    uint32_t    uiMeasurementTaskID;
    uint32_t    uiMeasurementPeriod_mS;
    PEVENT      pevMatrixMutex;
    PEVENT      pevCacheMutex;
    PEVENT      pevFormatTimer;
    int32_t     iCurrentColumn;
    PSRD        pSrd;
    uint32_t    uiFormatTaskID;
    float       gfIdleTimeAccumulator_S;
    struct
    {
        size_t stLength;
        int8_t *pszJsonData;
    } pDataCache[(SYRES_NUM + NUMBER_OF_HEAPS)];
} SRI, *PSRI;

/******************************************************************************
 Function Prototypes
 ******************************************************************************/

static PSRD sriCreate (int32_t iNumRows, int32_t iNumColumns);
static void sriFormatTask (PSRI pSri);
static void sriMonitorTask (PSRI pSri);

/******************************************************************************
 External variables and functions
 ******************************************************************************/

extern void *pxCurrentTCB;
extern void *xTaskGetIdleTaskHandle (void);

/******************************************************************************
 Global Variables
 ******************************************************************************/

static SRI gSri =
{
    false
};

/*******************************************************************************
 Public Functions
 ******************************************************************************/

/******************************************************************************
 Function Name: sriStart
 Description:   Function to start the system resource information monitor
 Parameters:    IN  iMeasurementPeriod_mS - The measurement period in mS
 IN  iHistory - The number of updates to store
 Return value:  0 for success
 ******************************************************************************/
int32_t sriStart (uint32_t uiMeasurementPeriod_mS, int32_t iHistory)
{
    if (false == gSri.bfStarted)
    {
        /* Measure the system resources enumerated in _SYRES and
         the heaps defined in malloc.h _MEMTYPE */
        int32_t iNumRows = SYRES_NUM + NUMBER_OF_HEAPS;

        /* The history is the number of columns in the "matrix" */
        int32_t iNumColumns = iHistory;

        /* Initialise the data */
        memset(&gSri, 0, sizeof(SRI));

        /* Create the data which can be address like:
        ppfData[iRow][iColumn] */
        gSri.pSrd = sriCreate(iNumRows, iNumColumns);
        if (gSri.pSrd)
        {
            gSri.uiMeasurementPeriod_mS = uiMeasurementPeriod_mS;
            gSri.pevMatrixMutex = NULL;
            gSri.iCurrentColumn = 0;
            gSri.uiMeasurementTaskID = taskCreate((PTASKFN)sriMonitorTask,
                                                  &gSri,
                                                  TC_DEFAULT_STACK_SIZE,
                                                  HEAP_SRAM,
                                                  "Resource Monitor",
                                                  TASK_RESOURCE_MONITOR_PRI);
            if (TC_INVALID_TASK_ID != gSri.uiMeasurementTaskID)
            {

                /* Start a timer to signal the data formatting task */
                gSri.pevFormatTimer = timerStartEvent(PERIODIC_TIMER, 1000UL);
                if (gSri.pevFormatTimer)
                {
                    gSri.uiFormatTaskID = taskCreate((PTASKFN)sriFormatTask,
                                                     &gSri,
                                                     TC_DEFAULT_STACK_SIZE,
                                                     HEAP_SRAM,
                                                     "JSON Formatter",
                                                     TASK_JSON_FORMATTER_PRI);
                    if (TC_INVALID_TASK_ID != gSri.uiFormatTaskID)
                    {
                        gSri.bfStarted = true;
                        return 0;
                    }
                }
                taskDestroy(gSri.uiMeasurementTaskID);
            }
            exFree(HEAP_SDRAM, gSri.pSrd);
        }
    }
    return -1;
}
/******************************************************************************
 End of function  sriStart
 ******************************************************************************/

/******************************************************************************
 Function Name: sriStop
 Description:   Function to stop the system resource information monitor
 Parameters:    none
 Return value:  none
 ******************************************************************************/
void sriStop (void)
{
    if (gSri.bfStarted)
    {
        uint32_t iType = (SYRES_NUM + NUMBER_OF_HEAPS);

        /* Delete the Measurement task */
        taskDestroy(gSri.uiMeasurementTaskID);

        /* Suspend the Formatter Task */
        taskSetState(gSri.uiFormatTaskID, true);

        /* Clean up all allocated memory for the formatter task */
        while (iType--)
        {
            if (gSri.pDataCache[iType].pszJsonData)
            {
                free(gSri.pDataCache[iType].pszJsonData);
            }
        }

        /* Delete the Formatter task */
        taskDestroy(gSri.uiFormatTaskID);

        /* Stop the Fomatter event timer */
        timerStopEvent(gSri.pevFormatTimer);

        /* Release the data array memory */
        exFree(HEAP_SDRAM, gSri.pSrd);

        /* Release the event */
        eventDestroy(&gSri.pevMatrixMutex, 1);

        /* Release the event */
        eventDestroy(&gSri.pevCacheMutex, 1);

        /* Reset the reource information Monitor configuration */
        memset(&gSri, 0, sizeof(SRI));
    }
}
/******************************************************************************
 End of function  sriStop
 ******************************************************************************/

/******************************************************************************
 Function Name: sriGet
 Description:   Function to get some system resource information
 Parameters:    IN  iType - The data type (or row) of data to get
 Return value:  Pointer to a dynamically allocated string containing the data
 or NULL on error
 ******************************************************************************/
int8_t *sriGet (int32_t iType)
{
    int8_t *pszJsonData = NULL;
    if (iType < (SYRES_NUM + NUMBER_OF_HEAPS))
    {
        eventWaitMutex(&gSri.pevCacheMutex, EV_WAIT_INFINITE);

        /* Check that the data exists if not create it */
        if (gSri.pDataCache[iType].pszJsonData)
        {
            size_t stLength = gSri.pDataCache[iType].stLength;

            /* Allocate storage for the data */
            pszJsonData = (int8_t*) malloc(stLength);
            memcpy(pszJsonData, gSri.pDataCache[iType].pszJsonData, stLength);
        }
        eventReleaseMutex(&gSri.pevCacheMutex);
    }
    return pszJsonData;
}
/******************************************************************************
 End of function  sriGet
 ******************************************************************************/

/*****************************************************************************
 * Function Name: sriMeasureCpu
 * Description:   Function to measure the idle CPU time for an estimate of CPU usage
 * Arguments:     none
 * Return value:  none
 *****************************************************************************/
void sriMeasureCpu (void)
{
    if (true == gSri.bfStarted)
    {
        static TMSTMP gIdleCpuTimer;
        static _Bool gbfIdleTimerStarted = false;

        /* Check to see if the task being switched in is the idle task */
        if ((xTaskGetIdleTaskHandle() == pxCurrentTCB)
        &&  (false == gbfIdleTimerStarted))
        {

            /* Start a measurement timer */
            timerStartMeasurement(&gIdleCpuTimer);

            /* Set a flag to show that it has been started */
            gbfIdleTimerStarted = true;
        }
        else if (true == gbfIdleTimerStarted)
        {
            /* Stop the measurement timer and add to the idle measurement */
            gSri.gfIdleTimeAccumulator_S += timerStopMeasurement(&gIdleCpuTimer);

            /* Clear the flag to show that the timer should be re-started
             the next time the idle task is switched in */
            gbfIdleTimerStarted = false;
        }
    }
}
/******************************************************************************
 End of function  sriMeasureCpu
 ******************************************************************************/
/******************************************************************************
 Private Functions
 ******************************************************************************/

/******************************************************************************
 Function Name: sriCreate
 Description:   Function to create a matrix in which to hold the system resource
 usage information
 Parameters:    IN  iNumRows - The numer of rows
 IN  iNumColumns - The number of columns
 Return value:  Pointer to the matrix or NULL on error
 ******************************************************************************/
static PSRD sriCreate (int32_t iNumRows, int32_t iNumColumns)
{
    PSRD pSrd = NULL;

    /* Check for reasonable arguments to prevent memory errors later */
    if ((iNumRows >= 1)
    &&  (iNumColumns >= 1))
    {
        size_t stPointerArraySize = (iNumRows * sizeof(float*));
        size_t stDataArraySize = (iNumRows * iNumColumns * sizeof(float));

        /* Calculate the entire memory required */
        size_t  stMatrixSize =
                (stPointerArraySize + stDataArraySize + sizeof(SRD));

        /* Allocate the memory */
        pSrd = (PSRD) exMalloc(HEAP_SDRAM, stMatrixSize);
        if (pSrd)
        {
            float **ppPointerArray = &pSrd->pfData;
            float **ppPointerArrayEnd = ppPointerArray + iNumRows;
            float *pColumn = (float*) ppPointerArrayEnd;

            /* Initialise the matrix data */
            memset(pSrd, 0, stMatrixSize);

            /* Set the size */
            pSrd->iNumRows = iNumRows;
            pSrd->iNumColumns = iNumColumns;

            /* Initialise the main pointer array */
            pSrd->ppfData = ppPointerArray;
            while (ppPointerArray < ppPointerArrayEnd)
            {
                *ppPointerArray++ = pColumn;
                pColumn += iNumColumns;
            }
        }
    }
    return pSrd;
}
/******************************************************************************
 End of function  sriCreate
 ******************************************************************************/

/******************************************************************************
 Function Name: sriFormat
 Description:   Function to format the desired data type into JSON format for
 flot
 Parameters:    IN  iType - The data type (or row) of data to get
 Return value:  Pointer to a dynamically allocated string containing the data
 or NULL on error
 ******************************************************************************/
int8_t *sriFormat (int32_t iType)
{
    CAT Cat;

    /* Calculate the measurement period */
    float fMeasurementPeriod = (float) gSri.uiMeasurementPeriod_mS * (float) 1e-3;

    /* Calculate the total measurement history time stored in the matrix */
    float fTotalTime = fMeasurementPeriod * (float) gSri.pSrd->iNumColumns;

    /* The data output is to look like this:
     {
     label: 'Europe (EU27)',
     data: [[1999, 3.0], [2000, 3.9], [2001, 2.0], [2002, 1.2]]
     }
     */

    /* Initialise the concatenated string object */
    xcatcreate(&Cat);

    /* Check that the type is valid */
    if (iType < (SYRES_NUM + NUMBER_OF_HEAPS))
    {
        int32_t iColumn, iDataCount;

        /* Format the label and line type */
        xcatprintf(&Cat,
                   SUB_LENGTH,
                   "{\r\n"
                   "\tcolor: \"rgb(0, 0, 0)\",\r\n"
                   "\tshadowSize: 0,\r\n"
                   "\tdata: [");

        /* Get the mutex for access to the data */
        eventWaitMutex(&gSri.pevMatrixMutex, EV_WAIT_INFINITE);

        /* Format the data - starting with the current column */
        iColumn = (int32_t) (gSri.iCurrentColumn + 1U);
        iDataCount = (int32_t) (gSri.pSrd->iNumColumns - 1U);
        while (iDataCount--)
        {
            float   fX = (float)((float)(gSri.pSrd->iNumColumns - iDataCount)
                       * fMeasurementPeriod) - fTotalTime;
            float   fY = gSri.pSrd->ppfData[iType][iColumn];
            xcatprintf(&Cat,
                       SUB_LENGTH,
                       "[%.2f, %.1f]",
                       fX,
                       fY);
            /* If there is more data add the delimiter */
            if (iDataCount)
            {
                xcatprintf(&Cat, SUB_LENGTH, ", ");
            }
            else
            {
                xcatprintf(&Cat, SUB_LENGTH, "]\r\n}\r\n");
            }
            iColumn++;
            if (iColumn >= gSri.pSrd->iNumColumns)
            {
                iColumn = 0;
            }
        }

        /* Release the mutex */
        eventReleaseMutex(&gSri.pevMatrixMutex);
    }
    return (int8_t *) Cat.pchBuffer;
}
/******************************************************************************
 End of function sriFormat
 ******************************************************************************/

/******************************************************************************
 * Function Name: sriFormatTask
 * Description  : The task to format the data to JavaScript Object Notation
 * Arguments    : IN  pSri - Pointer to the system resource information data
 * Return Value : none
 ******************************************************************************/
static void sriFormatTask (PSRI pSri)
{
    int32_t iType;
    while (true)
    {
        /* Wait for the 1 second signal */
        eventWait(&pSri->pevFormatTimer, 1, true);
        iType = (SYRES_NUM + NUMBER_OF_HEAPS);
        while (iType--)
        {
            int8_t *pFree;
            int8_t *pszJsonData;
            size_t stLength;

            /* Format the data */
            pszJsonData = sriFormat(iType);
            stLength = strlen((char *) pszJsonData) + 1;

            /* Update the cache */
            eventWaitMutex(&pSri->pevCacheMutex, EV_WAIT_INFINITE);
            pFree = pSri->pDataCache[iType].pszJsonData;
            pSri->pDataCache[iType].pszJsonData = pszJsonData;
            pSri->pDataCache[iType].stLength = stLength;

            /* Release the mutex */
            eventReleaseMutex(&pSri->pevCacheMutex);

            /* Free the old data */
            if (pFree)
            {
                free(pFree);
            }

            /* Reduce the CPU overhead */
            taskSleep(1UL);
        }
    }
}
/******************************************************************************
 End of function sriFormatTask
 ******************************************************************************/

/******************************************************************************
 Function Name: sriMonitorTask
 Description:   The task to monitor the system resource usage
 Parameters:    IN  pSri - Pointer to the system resource information data
 Return value:  none
 ******************************************************************************/
static void sriMonitorTask (PSRI pSri)
{
    int iColumn = 0;

    /* Get a pointer to the matrix */
    float **ppfData = pSri->pSrd->ppfData;
    float fPeriod_S = (float) pSri->uiMeasurementPeriod_mS / 1000;
    while (true)
    {
        SYSUSAGE sysUsage;
        HEAPUSAGE memUsage;
        MEMTYPE memType;
        TMSTMP measurementTimer;
        float fMeasurementTime, fEstimatedIdleCpuTime;

        /* Start a measurement timer */
        timerStartMeasurement(&measurementTimer);

        /* Sleep for the measurement period */
        taskSleep(pSri->uiMeasurementPeriod_mS);
        fEstimatedIdleCpuTime = gSri.gfIdleTimeAccumulator_S;

        /* Reset the idle measurement time accumulator */
        gSri.gfIdleTimeAccumulator_S = 0.0f;

        /* Get the actual measurement time */
        fMeasurementTime = timerStopMeasurement(&measurementTimer);

        /* The matrix of data is updated by this task and read by another.
         This means that the access to the data must be mutually exclusive */
        eventWaitMutex(&pSri->pevMatrixMutex, EV_WAIT_INFINITE);

        /* Calculate the CPU usage as a percentage */
        if (fEstimatedIdleCpuTime > fMeasurementTime)
        {
            ppfData[SYRES_CPU_USAGE][iColumn] = 0.0f;
        }
        else
        {
            ppfData[SYRES_CPU_USAGE][iColumn] = ((fMeasurementTime - fEstimatedIdleCpuTime)
                                              / fMeasurementTime) * 100.0f;
        }

        /* Get the instantaneous resource usage information */
        systemUsage(&sysUsage);

        /* Calculate the event usage as a percentage */
        ppfData[SYRES_EVENT_USAGE][iColumn] =
            ((float)sysUsage.stEventsInUse / (float)sysUsage.stEventTotal)
            * 100.0f;
        /* Calculate the file usage as a percentage */
        ppfData[SYRES_FILES_USAGE][iColumn] =
            ((float)sysUsage.stFilesInUse / (float)sysUsage.stFileTotal)
            * 100.0f;
        /* Calculate the timer usage as a percentage */
        ppfData[SYRES_TIMER_USAGE][iColumn] =
            ((float)sysUsage.stTimersInUse / (float)sysUsage.stTimerTotal)
            * 100.0f;
        /* For each heap */
        for (memType = (MEMTYPE) 0; memType < NUMBER_OF_HEAPS; memType++)
        {
            /* Get the memory usage information */
            heapUsage(&memUsage, memType);

            /* Calculate the usage as a percentage */
            ppfData[SYRES_NUM + memType][iColumn] =
                ((float)memUsage.stPeakAllocated / (float)memUsage.stTotalHeap)
                * 100.0f;
        }

        /* Set the current column */
        pSri->iCurrentColumn = (int32_t) iColumn;

        /* Release the mutex */
        eventReleaseMutex(&pSri->pevMatrixMutex);

        /* Set the next column for the history */
        iColumn++;
        if (iColumn >= pSri->pSrd->iNumColumns)
        {
            iColumn = 0;
        }
    }
}
/******************************************************************************
 End of function  sriMonitorTask
 ******************************************************************************/

/******************************************************************************
 End  Of File
 ******************************************************************************/
