/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
*
* Copyright (C) 2015 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************/
/*******************************************************************************
* File Name     : ramTest.c
* Device(s)     : RZ/A1H (R7S721001)
* Tool-Chain    : GNUARM-RZv14.01-EABI
* H/W Platform  : RSK+RZA1H CPU Board
* Description   : Function to perform simple RAM tests
*******************************************************************************/
/*******************************************************************************
* History       : DD.MM.YYYY Version Description
*               : 21.08.2015 1.00    Initial Version
*******************************************************************************/

/******************************************************************************
  WARNING!  IN ACCORDANCE WITH THE USER LICENCE THIS CODE MUST NOT BE CONVEYED
  OR REDISTRIBUTED IN COMBINATION WITH ANY SOFTWARE LICENSED UNDER TERMS THE
  SAME AS OR SIMILAR TO THE GNU GENERAL PUBLIC LICENCE
******************************************************************************/

/******************************************************************************
Includes   <System Includes> , "Project Includes"
******************************************************************************/

#include "r_typedefs.h"
#include "ramTest.h"
#include "compiler_settings.h"

/******************************************************************************
Function Prototypes
******************************************************************************/

static bus_width_t ramTestLocation(bus_width_t *pbwLocation, uint32_t lDelayCount);

/******************************************************************************
Exported global variables and functions (to be accessed by other files)
******************************************************************************/
/* The code should reside in ROM so define a section for the rest of this file
   so that the code can be located appropriately */

RESET_CODE_SECTION

/******************************************************************************
* Function Name: ramSizeTest
* Description  : Function to perform a simple non-destructive RAM size test
* Arguments    : IN  pbwRamStart - Pointer to the start of the RAM
*                IN  stLength - The length of the RAM in BUS_WIDTH
* Return Value : The length of the RAM which passed the test or 0 on failure
******************************************************************************/
size_t ramSizeTest(bus_width_t *pbwRamStart, size_t stLength)
{
    size_t      stAddrLine;
    bus_width_t bwKeepSafe;
    bus_width_t bwAddressCheck;

    /* Basic read/write test on first address */
    if (ramTestLocation(pbwRamStart, MEM_DELAY_CNT))
    {
        /* If it fails report as NO RAM */
        return (size_t)0;
    }

    /* Try one location on each address line */
    bwKeepSafe = (*pbwRamStart);

    /* Write to all F's base address */
    (*pbwRamStart) = ((bus_width_t)-1L);

    /* Set the first address line to test */
    stAddrLine = sizeof(bus_width_t);

    /* Test each address line in the length */
    while (stAddrLine < stLength)
    {
        /* Basic read/write test */
        if (ramTestLocation((pbwRamStart + stAddrLine),  MEM_DELAY_CNT))
        {
            /* If it fails then the RAM is faulty  or incorrectly configured.
               Even if this test is on a location that does not exist then
               it should at least shadow to one that does */
            return (size_t)0;
        }

        /* Take a copy of the data at this location */
        bwAddressCheck = *(pbwRamStart + stAddrLine);

        /* If address line is missing then this will spot it because the
           location test will have over written the base location */
        *(pbwRamStart + stAddrLine) = 0;

        /* Test base address */
        if (*pbwRamStart != ((bus_width_t)-1L))
        {
            /* Either this is a missing address line Or a smaller quantity
               of RAM has been found */
            *(pbwRamStart + stAddrLine) = bwAddressCheck;
            *pbwRamStart = bwKeepSafe;
            return (size_t)(stAddrLine);
        }

        /* Restore the address check value */
        *(pbwRamStart + stAddrLine) = bwAddressCheck;

        /* Test the next address line */
        stAddrLine = stAddrLine << 1;
    }

    /* Replace base address */
    *pbwRamStart = bwKeepSafe;

    return stLength;
}
/******************************************************************************
End of function  ramSizeTest
******************************************************************************/

/******************************************************************************
* Function Name: ramFillTest
* Description  : Function to fill the RAM with test data and read it back
* Arguments    : IN  pbwRamStart - Pointer to the start of the RAM
*                IN  stLength - The length of the RAM in bytes
* Return Value : Pointer to the first location that fails the read back verify or
*                0 on success
******************************************************************************/
bus_width_t *ramFillTest(bus_width_t *pbwRamStart, size_t stLength)
{
    bus_width_t   bwData = (bus_width_t)0;
    bus_width_t   *pbwTest = pbwRamStart;
    bus_width_t   *pbwEnd =  (bus_width_t*)((int8_t*)pbwRamStart + stLength);

    /* Fill with data */
    while (pbwTest < pbwEnd)
    {
        *pbwTest = (bus_width_t)pbwTest;//bwData++;
        pbwTest++;
    }
    bwData = (bus_width_t)0;
    pbwTest = pbwRamStart;

    /* Verify data */
    while (pbwTest < pbwEnd)
    {
        if (*pbwTest != (bus_width_t)pbwTest)//bwData)
        {
            return pbwTest;
        }
        pbwTest++;
        bwData++;
    }
    return (bus_width_t *)0;
}
/******************************************************************************
End of function  ramFillTest
******************************************************************************/

/******************************************************************************
Private global variables and functions
******************************************************************************/

/******************************************************************************
* Function Name: ramReadWrite
* Description  : Function to read and write from a single location
* Arguments    : IN  pbwRam - Pointer to the location to write and read from
*                IN  bwTest - The value to write and read
*                IN  lDelayCount - A delay cpunt inserted between read and writes
* Return Value : A map of the failed bits
******************************************************************************/
static bus_width_t ramReadWrite(bus_width_t *pbwRam, bus_width_t bwTest,
                                                 long lDelayCount)
{
    bus_width_t   bwKeepSafe, bwReadBack, bwResult;
    long        lDelay;
    bwResult = 0;

    /* Read byte and keep safe */
    bwKeepSafe = *pbwRam;

    /* Test 1 - logic 1 true */
    *pbwRam = bwTest;

    /* wait */
    for (lDelay = 0; lDelay < lDelayCount; lDelay++)
    {
        /* wait */;
    }
    /* Read back */
    bwReadBack = *pbwRam;
    if (bwReadBack != bwTest)
    {
        bwResult |= (bus_width_t)(~bwReadBack & bwTest);
    }

    /* test 2 - logic 0 true */
    *pbwRam = (bus_width_t)~bwTest;

    /* wait */
    for (lDelay = 0; lDelay < lDelayCount; lDelay++)
    {
        /* wait */;
    }
    /* Read back */
    bwReadBack = *pbwRam;
    if (bwReadBack != (bus_width_t)~bwTest)
    {
        bwResult |= (bus_width_t)(bwReadBack & bwTest);
    }

    /* restore data */
    *pbwRam = bwKeepSafe;

    return bwResult;
}
/******************************************************************************
End of function  ramReadWrite
******************************************************************************/

/******************************************************************************
* Function Name: ramTestLocation
* Description  : Function to test a location with a walking ones test
* Arguments    : IN  pbwLocation - Pointer to the memory location to test
*                IN  lDelayCount - The delay to insert between read and writes
* Return Value : bit map of stuck bits
******************************************************************************/
static bus_width_t ramTestLocation(bus_width_t *pbwLocation,
                                   uint32_t lDelayCount)
{
    int_t       iCount;
    bus_width_t   bwResult = 0;
    bus_width_t   bwTestByte = 0x01;

    /* Walking ones round */
    for (iCount = 0; iCount < (int)(sizeof(bus_width_t) *  8); iCount++)
    {
        /* Read and write each bit */
        if (ramReadWrite(pbwLocation, bwTestByte, lDelayCount) != 0)
        {
            /* Log each bit that fails */
            bwResult |= bwTestByte;
        }

        /* Select next bit */
        bwTestByte = (bus_width_t)(bwTestByte << 1);
    }
    return bwResult;
}
/******************************************************************************
End of function  ramTestLocation
******************************************************************************/

/******************************************************************************
End  Of File
******************************************************************************/
